#include "GtCamera.h"

GtCamera::GtCamera(void) : GtNode()
{
	m_eCameraType = eCM_AIRMOVE;
	m_vPosition = GtPoint3f(0.0f, 0.0f, 3.0f);
	m_vRight  = GtPoint3f(1.0f, 0.0f, 0.0f);
	m_vUp = GtPoint3f(0.0f, 1.0f, 0.0f);
	m_vLook = GtPoint3f(0.0f, 0.0f, 1.0f);
}

GtCamera::GtCamera(eGT_CAMERATYPE eCameraType) : GtNode()
{
	m_eCameraType = eCameraType;
	m_vPosition = GtPoint3f(0.0f, 0.0f, 3.0f);
	m_vRight  = GtPoint3f(1.0f, 0.0f, 0.0f);
	m_vUp = GtPoint3f(0.0f, 1.0f, 0.0f);
	m_vLook = GtPoint3f(0.0f, 0.0f, 1.0f);
}

GtCamera::~GtCamera(void)
{
}

void GtCamera::SetCameraType(eGT_CAMERATYPE eCameraType)
{
	m_eCameraType = eCameraType;
}

void GtCamera::Reset(void)
{
	m_vPosition = GtPoint3f(0.0f, 80.0f, 0.0f);
	m_vRight  = GtPoint3f(1.0f, 0.0f, 0.0f);
	m_vUp = GtPoint3f(0.0f, 1.0f, 0.0f);
	m_vLook = GtPoint3f(0.0f, 0.0f, 1.0f);
}

void GtCamera::Fly(float value)
{
	if( m_eCameraType == eCM_LANDMOVE )
		m_vPosition.y += value;

	if( m_eCameraType == eCM_AIRMOVE )
		m_vPosition += m_vUp * value;
}

void GtCamera::Walk(float value)
{
	if( m_eCameraType == eCM_LANDMOVE )
		m_vPosition += GtPoint3f(m_vLook.x, 0.0f, m_vLook.z) * value;

	if( m_eCameraType == eCM_AIRMOVE )
		m_vPosition += m_vLook * value;
}

void GtCamera::Strafe(float value)
{
	if( m_eCameraType == eCM_LANDMOVE )
		m_vPosition += GtPoint3f(m_vRight.x, 0.0f, m_vRight.z) * value;

	if( m_eCameraType == eCM_AIRMOVE )
		m_vPosition += m_vRight * value;
}

void GtCamera::RotationRoll(const float angle, GtNode::eGT_TransformSpace relativeTo)
{
	if(m_eCameraType == eCM_AIRMOVE)
	{
		GtMatrix4 mat;
		mat.MakeRotation(angle, m_vLook.x, m_vLook.y, m_vLook.z);

		m_vRight.TransformNormalRow(mat);
		m_vUp.TransformNormalRow(mat);
	}
}

void GtCamera::RotationPitch(const float angle, GtNode::eGT_TransformSpace relativeTo)
{
	GtMatrix4 mat;
	mat.MakeRotation(angle, m_vRight.x, m_vRight.y, m_vRight.z);

	m_vUp.TransformNormalRow(mat);
	m_vLook.TransformNormalRow(mat);
}

void GtCamera::RotationYaw(const float angle, GtNode::eGT_TransformSpace relativeTo)
{
	GtMatrix4 mat;
	if(m_eCameraType == eCM_LANDMOVE)
		mat.MakeYRotation(angle);
	if(m_eCameraType == eCM_AIRMOVE)
		mat.MakeRotation(angle, 0.0f, 1.0f, 0.0f);
	
	m_vRight.TransformNormalRow(mat);
	m_vLook.TransformNormalRow(mat);
}

void GtCamera::GetViewMatrix(GtMatrix4 *pMat)
{
	m_vLook = m_vLook.Normalized();
	m_vUp = m_vLook.Cross(m_vRight);
	m_vUp = m_vUp.Normalized();
	m_vRight = m_vUp.Cross(m_vLook);
	m_vRight = m_vRight.Normalized();

	float x = -(m_vRight.Dot(m_vPosition));
	float y = -(m_vUp.Dot(m_vPosition));
	float z = -(m_vLook.Dot(m_vPosition));

	pMat->m11 = m_vRight.x;		pMat->m12 = m_vUp.x;	pMat->m13 = m_vLook.x;		pMat->m14 = 0.0f;
	pMat->m21 = m_vRight.y;		pMat->m22 = m_vUp.y;	pMat->m23 = m_vLook.y;		pMat->m24 = 0.0f;
	pMat->m31 = m_vRight.z;		pMat->m32 = m_vUp.z;	pMat->m33 = m_vLook.z;		pMat->m34 = 0.0f;
	pMat->m41 = x;				pMat->m42 = y;			pMat->m43 = z;				pMat->m44 = 1.0f;
}