#include "GtD3DRenderer.h"

void ConvertGtMatrixFromD3DMatrix(GtMatrix4 *pOut, D3DXMATRIX *pIn)
{
	pOut->m11 = pIn->_11; pOut->m12 = pIn->_12; pOut->m13 = pIn->_13; pOut->m14 = pIn->_14;
	pOut->m21 = pIn->_21; pOut->m22 = pIn->_22; pOut->m23 = pIn->_23; pOut->m24 = pIn->_24;
	pOut->m31 = pIn->_31; pOut->m32 = pIn->_32; pOut->m33 = pIn->_33; pOut->m34 = pIn->_34;
	pOut->m41 = pIn->_41; pOut->m42 = pIn->_42; pOut->m43 = pIn->_43; pOut->m44 = pIn->_44;
}

void ConvertD3DMatrixFromGtMatrix(D3DXMATRIX *pOut, GtMatrix4 *pIn)
{
	pOut->_11 = pIn->m11; pOut->_12 = pIn->m12; pOut->_13 = pIn->m13; pOut->_14 = pIn->m14;
	pOut->_21 = pIn->m21; pOut->_22 = pIn->m22; pOut->_23 = pIn->m23; pOut->_24 = pIn->m24;
	pOut->_31 = pIn->m31; pOut->_32 = pIn->m32; pOut->_33 = pIn->m33; pOut->_34 = pIn->m34;
	pOut->_41 = pIn->m41; pOut->_42 = pIn->m42; pOut->_43 = pIn->m43; pOut->_44 = pIn->m44;
}

void ConvertGtPoint3fFromD3DVector(GtPoint3f *pOut, D3DXVECTOR3 *pIn)
{
	pOut->x = pIn->x;
	pOut->y = pIn->y;
	pOut->z = pIn->z;
}

void ConvertD3DVectorFromGtPoint3(D3DXVECTOR3 *pOut, GtPoint3f *pIn)
{
	pOut->x = pIn->x;
	pOut->y = pIn->y;
	pOut->z = pIn->z;
}

GtD3DRenderer *GtD3DRenderer::m_pInstance = NULL;

GtD3DRenderer* GtD3DRenderer::GetInstance(void)
{
	if(m_pInstance == NULL)
		m_pInstance = new GtD3DRenderer;
	return m_pInstance;
}

void GtD3DRenderer::ReleaseInstance(void)
{
	if(m_pInstance)
		delete m_pInstance;
	m_pInstance = NULL;
}

GtD3DRenderer::GtD3DRenderer(void)
{
	memset(&m_vPosition, 0, sizeof(GtPoint3f));
	memset(&m_vScale, 0, sizeof(GtPoint3f));
	memset(&m_vRotate, 0, sizeof(GtPoint3f));
}

GtD3DRenderer::~GtD3DRenderer(void)
{
}

void GtD3DRenderer::InitLight(void)
{
	ZeroMemory( &m_pD3DLight, sizeof(D3DLIGHT9) );
	SetLightMode(D3DLIGHT_POINT);
	SetLightPosVec(GtPoint3f(0.0f, 25000.0f, -25000.0f));
	SetLightAmbient(GtPoint3f(0.1f, 0.1f, 0.1f));
	SetLightDiffuse(GtPoint3f(1.0f, 1.0f, 1.0f));
	SetLightSpecular(GtPoint3f(0.5f, 0.5f, 0.5f));
	m_pD3DLight.Range = 50000.0f;
	m_pD3DLight.Attenuation0 = 0.85f;
}

bool GtD3DRenderer::SetLightMode(D3DLIGHTTYPE iType)
{
	if(iType > D3DLIGHT_DIRECTIONAL)
		return false;

	m_pD3DLight.Type = (D3DLIGHTTYPE)iType;
	return true;
}

void GtD3DRenderer::SetLightPosVec(GtPoint3f pt)
{
	GtPoint3f dir;
	m_pD3DLight.Position = D3DXVECTOR3(pt.x, pt.y, pt.z);
	dir = GtPoint3f(pt.x, pt.y, pt.z);
	dir = pt.Normalized();
	m_pD3DLight.Direction = D3DXVECTOR3(dir.x, dir.y, dir.z);
	m_pD3DLight.Direction = -1.0f * m_pD3DLight.Direction;
}

void GtD3DRenderer::SetLightAmbient(GtPoint3f pt)
{
	m_pD3DLight.Ambient.r = pt.x;
	m_pD3DLight.Ambient.g = pt.y;
	m_pD3DLight.Ambient.b = pt.z;
	m_pD3DLight.Ambient.a = 1.0f;
}

void GtD3DRenderer::SetLightDiffuse(GtPoint3f pt)
{
	m_pD3DLight.Diffuse.r = pt.x;
	m_pD3DLight.Diffuse.g = pt.y;
	m_pD3DLight.Diffuse.b = pt.z;
	m_pD3DLight.Diffuse.a = 1.0f;
}

void GtD3DRenderer::SetLightSpecular(GtPoint3f pt)
{
	m_pD3DLight.Specular.r = pt.x;
	m_pD3DLight.Specular.g = pt.y;
	m_pD3DLight.Specular.b = pt.z;
	m_pD3DLight.Specular.a = 1.0f;
}

void GtD3DRenderer::SetRange(float fRange)
{
	m_pD3DLight.Range = fRange;
}

void GtD3DRenderer::SetAttenuation(float fAtten)
{
	m_pD3DLight.Attenuation0 = fAtten;
}

HRESULT GtD3DRenderer::Create(HWND hWnd, int iWindowX, int iWindowY, bool bWindowed)
{
	m_iWindowX = iWindowX;
	m_iWindowY = iWindowY;

	//̽ ü 
    if( NULL == ( m_pD3D = Direct3DCreate9( D3D_SDK_VERSION ) ) )
        return E_FAIL;

    //̽  ü
	D3DPRESENT_PARAMETERS d3dpp;
	ZeroMemory( &d3dpp, sizeof(d3dpp) );
	d3dpp.BackBufferWidth = iWindowX;
	d3dpp.BackBufferHeight = iWindowY;
	d3dpp.Windowed = TRUE;
	d3dpp.SwapEffect = D3DSWAPEFFECT_DISCARD;
	d3dpp.BackBufferFormat = D3DFMT_UNKNOWN;
	d3dpp.EnableAutoDepthStencil = TRUE;
	d3dpp.AutoDepthStencilFormat = D3DFMT_D16;
	d3dpp.PresentationInterval = D3DPRESENT_INTERVAL_IMMEDIATE;

	D3DCAPS9 caps;
	m_pD3D->GetDeviceCaps(D3DADAPTER_DEFAULT, D3DDEVTYPE_HAL, &caps);
 
	//ؽ μ  Ȯ
	int vp = 0;
	if( caps.DevCaps & D3DDEVCAPS_HWTRANSFORMANDLIGHT )
		vp = D3DCREATE_HARDWARE_VERTEXPROCESSING;
	else
	  vp = D3DCREATE_SOFTWARE_VERTEXPROCESSING;

    //̽ 
    if( FAILED( m_pD3D->CreateDevice(D3DADAPTER_DEFAULT, D3DDEVTYPE_HAL, hWnd, D3DCREATE_SOFTWARE_VERTEXPROCESSING, &d3dpp, &m_pD3DDevice) ) )
	{	return E_FAIL;	}

	m_hWnd = hWnd;

	InitPipeLine();

	return S_OK;
}

void GtD3DRenderer::Release(void)
{
	if(m_pD3DDevice)
	{
		m_pD3DDevice->Release();
		m_pD3DDevice = NULL;
	}
	if(m_pD3D)
	{
		m_pD3D->Release();
		m_pD3D = NULL;
	}
}

HWND &GtD3DRenderer::GetHwnd(void)
{
	return m_hWnd;
}

LPDIRECT3DDEVICE9 &GtD3DRenderer::GetDevice(void)
{
	return m_pD3DDevice;
}

D3DLIGHT9 &GtD3DRenderer::GetLight(void)
{
	return m_pD3DLight;
}

void GtD3DRenderer::InitPipeLine(void)
{
	SetWorld();
	m_fCameraSpeed = 2.0f;
	m_iCameraMode = GtCamera::eCM_AIRMOVE;
	m_cmCamera.SetCameraType((GtCamera::eGT_CAMERATYPE)m_iCameraMode);
	m_cmCamera.SetPosition(GtPoint3f(0.0f, 0.0f, 0.0f));

	InitLight();
	SetLight(TRUE);
	SetMaterial();

	SetWorld();
	SetView();
	SetProjection();
	SetViewport();

	m_pD3DDevice->SetRenderState(D3DRS_CULLMODE, D3DCULL_CCW);				//ø
	m_pD3DDevice->SetRenderState(D3DRS_ZENABLE, TRUE);						//Zѱ
	m_pD3DDevice->SetRenderState(D3DRS_SRCBLEND, D3DBLEND_SRCALPHA);		//  μ
	m_pD3DDevice->SetRenderState(D3DRS_DESTBLEND, D3DBLEND_INVSRCALPHA);	//  μ
}

void GtD3DRenderer::SetWorld(void)
{
	GtMatrix4 matScale, matRotate, matTranslate;
	matScale.MakeScaling(m_vScale.x, m_vScale.y, m_vScale.z);
	matRotate.MakeXYZRotation(m_vRotate.x, m_vRotate.y, m_vRotate.z);
	matTranslate.MakeTranslation(m_vPosition.x, m_vPosition.z, m_vPosition.z);

	m_matTM[eRPLM_WORLD] = matScale * matRotate * matTranslate;
}

void GtD3DRenderer::SetLight(BOOL bSwitch)
{
	if(bSwitch)
		m_pD3DDevice->SetLight( 0, &m_pD3DLight );
    m_pD3DDevice->LightEnable( 0, bSwitch );
    m_pD3DDevice->SetRenderState( D3DRS_LIGHTING, bSwitch );
	m_pD3DDevice->SetRenderState( D3DRS_SPECULARENABLE , bSwitch );
}

void GtD3DRenderer::LightUpdate(void)
{
	m_pD3DDevice->SetLight( 0, &m_pD3DLight );
}

//( ڼ  ʿ Ŭ и)
void GtD3DRenderer::SetMaterial()
{
	D3DMATERIAL9	mtrl;
	ZeroMemory(&mtrl, sizeof(D3DMATERIAL9));
	mtrl.Specular.r = mtrl.Diffuse.r = mtrl.Ambient.r = 1.0f;
	mtrl.Specular.b = mtrl.Diffuse.g = mtrl.Ambient.g = 1.0f;
	mtrl.Specular.g = mtrl.Diffuse.b = mtrl.Ambient.b = 1.0f;
	mtrl.Specular.a = mtrl.Diffuse.a = mtrl.Ambient.a = 1.0f;
	mtrl.Power = 8.0f;
	m_pD3DDevice->SetMaterial(&mtrl);
}

void GtD3DRenderer::SetMaterialUp(float fColor)
{
	D3DMATERIAL9	mtrl;
	ZeroMemory(&mtrl, sizeof(D3DMATERIAL9));
	mtrl.Specular.r = mtrl.Diffuse.r = mtrl.Ambient.r = fColor;
	mtrl.Specular.b = mtrl.Diffuse.g = mtrl.Ambient.g = fColor;
	mtrl.Specular.g = mtrl.Diffuse.b = mtrl.Ambient.b = fColor;
	mtrl.Specular.a = mtrl.Diffuse.a = mtrl.Ambient.a = fColor;
	mtrl.Power = 8.0f;
	m_pD3DDevice->SetMaterial(&mtrl);
}

//亯ȯ
void GtD3DRenderer::SetView()
{
	m_cmCamera.GetViewMatrix(&m_matTM[eRPLM_VIEW]);
	D3DXMATRIX matView;
	ConvertD3DMatrixFromGtMatrix(&matView, &m_matTM[eRPLM_VIEW]);
    m_pD3DDevice->SetTransform(D3DTS_VIEW, &matView);
}
//ȯ(尡 ڼ 庰 ʿ..  FovLH)
void GtD3DRenderer::SetProjection(void)
{
	D3DXMATRIX matProj;
	ConvertD3DMatrixFromGtMatrix(&matProj, &m_matTM[eRPLM_PROJ]);
	D3DXMatrixPerspectiveFovLH( &matProj, FOV, float(m_iWindowX)/float(m_iWindowY), ZNEAR, ZFAR );
	m_pD3DDevice->SetTransform( D3DTS_PROJECTION, &matProj );
	D3DXMatrixPerspectiveFovLH( &matProj, 1.5f, 1.0f, ZNEAR, ZFAR );
	ConvertGtMatrixFromD3DMatrix(&m_matTM[eRPLM_PROJ], &matProj);
}

void GtD3DRenderer::SetViewport(void)
{
	D3DVIEWPORT9 matViewport;
	matViewport.X = 0;
	matViewport.Y = 0;
	matViewport.MinZ = 0.0f;
	matViewport.MaxZ = 1.0f;
	matViewport.Width = m_iWindowX;
	matViewport.Height = m_iWindowY;
	m_pD3DDevice->SetViewport(&matViewport);
}

void GtD3DRenderer::BeginScene(void)
{
	m_pD3DDevice->Clear(0, NULL, D3DCLEAR_TARGET | D3DCLEAR_ZBUFFER, D3DCOLOR_XRGB(0,0,255), 1.0f, 0);
	m_pD3DDevice->BeginScene();
}

void GtD3DRenderer::EndScene(void)
{
	// ׸
	m_pD3DDevice->EndScene();
	// ȭ Ѵ.
	m_pD3DDevice->Present(NULL, NULL, NULL, NULL);
}