#pragma once

#include <d3d9.h>		// Header File For Direct3D
#include <d3dx9.h>		// Header File For Direct3DX

//Direct3D ̺귯
#pragma comment(lib, "d3dxof.lib")
#pragma comment(lib, "dxguid.lib")
#pragma comment(lib, "d3d9.lib")				// Direct3DԼ Library
#pragma comment(lib, "d3dx9.lib")				// Direct3DXԼ Library

//Direct3D ȣ
#define CAMERAZ		-300.0f
#define RADIAN		D3DX_PI/180.0f
#define ZNEAR		10.0f
#define ZFAR		100000.0f
#define FOV			D3DX_PI/4.0f

#include "GtMatrix4.h"
#include "GtCamera.h"
#include "GtSoftwareRender_Type.h"

enum eRenderingPipeLineMatrix
{
	eRPLM_WORLD,
	eRPLM_VIEW,
	eRPLM_PROJ,

	eRPLM_END
};

extern void ConvertGtMatrixFromD3DMatrix(GtMatrix4 *pOut, D3DXMATRIX *pIn);
extern void ConvertD3DMatrixFromGtMatrix(D3DXMATRIX *pOut, GtMatrix4 *pIn);
extern void ConvertGtPoint3fFromD3DVector(GtPoint3f *pOut, D3DXVECTOR3 *pIn);
extern void ConvertD3DVectorFromGtPoint3(D3DXVECTOR3 *pOut, GtPoint3f *pIn);

class GtD3DRenderer
{
public:
	static		GtD3DRenderer		*m_pInstance;
	static		GtD3DRenderer		*GetInstance();
	static		void				ReleaseInstance();
private:
	LPDIRECT3D9				m_pD3D;
	LPDIRECT3DDEVICE9		m_pD3DDevice;
public:
	GtMatrix4				m_mtTransformTM;		//
	HWND					m_hWnd;
public:
	D3DLIGHT9				m_pD3DLight;

public:
	GtPoint3f				m_vPosition;
	GtPoint3f				m_vScale;
	GtPoint3f				m_vRotate;
	GtMatrix4				m_matTM[eRPLM_END];

public:
	GtCamera				m_cmCamera;
	int						m_iCameraMode;
	float					m_fCameraSpeed;

private:
	int						m_iWindowX;
	int						m_iWindowY;

public:
	GtD3DRenderer(void);
	~GtD3DRenderer(void);

public:	// ̽ 
	HRESULT Create(HWND hWnd, int iWindowX, int iWindowY, bool bWindowed);
	void Release(void);

public:
	void InitPipeLine(void);
	void SetWorld(void);
	void SetView(void);
	void SetProjection(void);
	void SetViewport(void);
	void SetLight(BOOL bSwitch);
	void SetMaterial(void);
	void SetMaterialUp(float fColor);

public:
	void		BeginScene(void);
	void		EndScene(void);
	
public:			//Ʈ
	void					InitLight(void);
	void					LightUpdate(void);
	bool					SetLightMode(D3DLIGHTTYPE iType);
	void					SetLightPosVec(GtPoint3f pt);
	void					SetLightAmbient(GtPoint3f pt);
	void					SetLightDiffuse(GtPoint3f pt);
	void					SetLightSpecular(GtPoint3f pt);
	void					SetRange(float fRange);
	void					SetAttenuation(float fAtten);

	HWND					&GetHwnd(void);
	LPDIRECT3DDEVICE9		&GetDevice(void);
	D3DLIGHT9				&GetLight(void);
//	LPDIRECT3DDEVICE9		GetDevice(void);
};