#include "GtDIB.h"

GtDIB::GtDIB(void* pBuff, unsigned int uiBuffOffset)
{
	m_pBuff = pBuff;
	m_nBuffOffset = uiBuffOffset;
	memset(&m_FileHeader, 0, sizeof(GT_DIB_FILE_HEADER));
	memset(&m_InfoHeader, 0, sizeof(GT_DIB_INFO_HEADER));
	memset(&m_RgbQuad, 0, sizeof(GT_RGBQUAD));
}

GtDIB::~GtDIB(void)
{
}

bool GtDIB::LoadDIBBitmap(const char *pName)
{
	FILE *fp = fopen(pName, "rb");

	if(!fp)
	{
		assert(0);
		return false;
	}

	fread(&m_FileHeader, sizeof(GT_DIB_FILE_HEADER), 1, fp);
	fread(&m_InfoHeader, sizeof(GT_DIB_INFO_HEADER), 1, fp);
	
	if(m_InfoHeader.usBitCount != 24)
		fread(&m_RgbQuad, sizeof(GT_RGBQUAD), 1, fp);

	int nDataWidthSize = m_InfoHeader.lWidth * (m_InfoHeader.usBitCount >> 3);
	int nDataTotalSize = nDataWidthSize * m_InfoHeader.lHeight;

	m_pBitmapBuff = new unsigned char[nDataTotalSize];

	int i = nDataTotalSize - nDataWidthSize;
	for(; i>=0; i -= nDataWidthSize)
		fread(&m_pBitmapBuff[i], nDataWidthSize, 1, fp);

	fclose(fp);

	return true;
}

void GtDIB::DrawBitmap(int x, int y)
{
	int nDataWidthSize = m_InfoHeader.lWidth * (m_InfoHeader.usBitCount >> 3);
	unsigned char *pOff = (unsigned char*)m_pBuff;

	pOff += m_nBuffOffset * ( x * (m_InfoHeader.usBitCount >> 3) + y * ((SCR_W * (m_InfoHeader.usBitCount >> 3) + 3 ) & ~3));

	int nOff = 0;
	for(int i=0; i<m_InfoHeader.lHeight; i++)
	{
		memcpy(pOff, &m_pBitmapBuff[nOff], nDataWidthSize);

		nOff += nDataWidthSize;
		pOff += ((SCR_W * (m_InfoHeader.usBitCount >> 3) + 3) & ~3);
	}
}

void GtDIB::SetBuff(void *pBuff, unsigned int nBuffOffset)
{
	m_pBuff = pBuff;
	m_nBuffOffset = nBuffOffset;
}

int GtDIB::GetUVPixel(float u, float v)
{
	int x = ((int)((int)(u * m_InfoHeader.lWidth) - 0.5f));
	int y = ((int)((int)(v * m_InfoHeader.lHeight) - 0.5f));

	int nOff = ( y * (m_InfoHeader.lWidth) + x) * (m_InfoHeader.usBitCount >> 3);

	return GT_RGB(	m_pBitmapBuff[nOff+2], 
					m_pBitmapBuff[nOff+1], 
					m_pBitmapBuff[nOff]
				 );
}

int GtDIB::GetPosPixel(int x, int y)
{
	int _x = m_InfoHeader.lWidth - x;
	int _y = m_InfoHeader.lHeight - y;
	int nOff = ( _y * (m_InfoHeader.lWidth) + _x) * (m_InfoHeader.usBitCount >> 3);

	return GT_RGB(	m_pBitmapBuff[nOff], 
					m_pBitmapBuff[nOff-1], 
					m_pBitmapBuff[nOff-2]
				 );
}