#pragma once

#include"GtPoint3.h"

class GtMatrix3
{
	inline GtMatrix3();
	GtMatrix3(const GtPoint3f &col0, const GtPoint3f& col1, const GtPoint3f& col2);


	// Row  Լ
	inline void GetRow(unsigned int uiRow, GtPoint3f& row) const;
	inline void SetRow(unsigned int uiRow, const GtPoint3f& row);
	inline void GetRow(unsigned int uiRow, float& f0, float& f1, float& f2) const;
	inline void SetRow(unsigned int uiRow, float f0, float f1, float f2);
	inline void GetRow(unsigned int uiRow, float* pRow) const;
	inline void SetRow(unsigned int uiRow, const float* pRow);

	// Column  Լ
	inline void GetCol(unsigned int uiCol, GtPoint3f& col) const;
	inline void SetCol(unsigned int uiCol, const GtPoint3f& col);
	inline void GetCol(unsigned int uiCol, float& f0, float& f1, float& f2);
	inline void SetCol(unsigned int uiCol, float f0, float f1, float f2);
	inline void GetCol(unsigned int uiCol, float* pCol); const;
	inline void SetCol(unsigned int uiCol, const float* pCol);

	//Entry  Լ
	inline float GetEntry(unsigned int uiRow, unsigned int uiCol) const;
	inline void	 SetEntry(unsigned int uiRow, unsigned int uiCol, float fEntry);

	inline void MakeZero();
	void MakeIdentity();
	void MakeDiagonal(float fDiag0, float fDiag1, float fDiag2);
	void MakeXRotation(float fAngle);
	void MakeYRotation(float fAngle);
	void MakeZRotation(float fAngle);
	void MakeRotation(float fAngle, float x, float y, float z);
	inline void MakeRotation(float fAngle, const GtPoint3<T>& axis);

	void MakeScaling(float x, float y, float z);
	void MakeTranslation(float x, float y, float z);

	bool		operator ==(const GtMatrix3& mat) const;
	inline bool operator !=(const GtMatrix3& mat) const;

	GtMatrix3 operator +(const GtMatrix3& mat) const;
	GtMatrix3 operator -(const GtMatrix3& mat) const;
	inline GtMatrix3 operator *(const GtMatrix3& mat) const;

	inline GtMatrix3 operator *(float fScalar) const;
	inline GtMatrix3 operator *(const GtPoint3f& pt) const;

	// Ÿ 
	GtMatrix3 Inverse() const;
	bool Inverse(GtMatrix3& inv) const;
	GtMatrix3 Transpose() const;
	void ExtractAngleAndAxis(float& fAngle, float &x, float &y, float &z) const;

	void FromEulerAnglesXYZ(float fXAngle, float fYAngle, float fZAngle);
	void FromEulerAnglesXZY(float fXAngle, float fZAngle, float fYAngle);
	void FromEulerAnglesYXZ(float fYAngle, float fXAngle, float fZAngle);
	void FromEulerAnglesYZX(float fYAngle, float fZAngle, float fXAngle);
	void FromEulerAnglesZXY(float fZAngle, float fXAngle, float fYAngle);
	void FromEulerAnglesZYX(float fZAngle, float fYAngle, float fXAngle);

	GtMatrix3 Transpose(const GtMatrix3& mat) const;

	GtMatrix3 Congruence(const GtMatrix3& rot) const;

	static void TransformVertices(const GtMatrix3& rot, const GtPoint3f& trn, unsigned int uiVertexCount, const GtPoint3f* pInVertex, GtPoint3f* pOutVertex);
	static void TransformNormals(const GtMatrix3& rot, unsigned int uiNormalCount, const GtPoint3f* pInNormal, GtPoint3f* pOutNormal);
	static void TransformVerticesAndNormals(const GtMatrix3& rot, const GtPoint3f& trn, unsigned int uiCount, const GtPoint3f* pInVertex, GtPoint3f* pOutVertex, const GtPoint3f* pInNormal, GtPoint3f* pOutNormal);

	static const GtMatrix3 ZERO;
	static const GtMatrix3 IDENTITY;
};