#include "GtMatrix4.h"

GtMatrix4::GtMatrix4() :	m11(0)	,m12(0)		,m13(0)		,m14(0)	,
							m21(0)		,m22(0)	,m23(0)		,m24(0)	,
							m31(0)		,m32(0)		,m33(0)	,m34(0)	,
							m41(0)		,m42(0)		,m43(0)		,m44(0)
{
}

GtMatrix4::GtMatrix4(const GtPoint3f& col0, const GtPoint3f& col1, const GtPoint3f&  col2)
{
	m11 = col0.x;		m12 = col0.y;		m13 = col0.z;	m14 = 0.0f;
	m21 = col1.x;		m22 = col1.y;		m23 = col1.z;	m24 = 0.0f;
	m31 = col2.x;		m32 = col2.y;		m33 = col2.z;	m34 = 0.0f;
	m41 = 0.0f;			m42 = 0.0f;			m43 = 0.0f;		m44 = 1.0f;
}

GtMatrix4::GtMatrix4(const GtPoint4f& col0, const GtPoint4f& col1, const GtPoint4f&  col2,const GtPoint4f& col3)
{
	m11 = col0.X();		m12 = col0.Y();		m13 = col0.Z();		m14 = col0.W();
	m21 = col1.X();		m22 = col1.Y();		m23 = col1.Z();		m24 = col1.W();
	m31 = col2.X();		m32 = col2.Y();		m33 = col2.Z();		m34 = col2.W();
	m41 = col3.X();		m42 = col3.Y();		m43 = col3.Z();		m44 = 1.0f;
}

GtMatrix4::GtMatrix4(float (*_m)[4])
{
	m11 = (_m)[0][0];	m12 = (_m)[0][1];	m13 = (_m)[0][2];	m14 = (_m)[0][3];
	m21 = (_m)[1][0];	m22 = (_m)[1][1];	m23 = (_m)[1][2];	m24 = (_m)[1][3];
	m31 = (_m)[2][0];	m32 = (_m)[2][1];	m33 = (_m)[2][2];	m34 = (_m)[2][3];
	m41 = (_m)[3][0];	m42 = (_m)[3][1];	m43 = (_m)[3][2];	m44 = (_m)[3][3];
}

GtMatrix4::GtMatrix4(const GtMatrix4& mat)
{
	m11 = mat.m11;	m12 = mat.m12;	m13 = mat.m13;	m14 = mat.m14;
	m21 = mat.m21;	m22 = mat.m22;	m23 = mat.m23;	m24 = mat.m24;
	m31 = mat.m31;	m32 = mat.m32;	m33 = mat.m33;	m34 = mat.m34;
	m41 = mat.m41;	m42 = mat.m42;	m43 = mat.m43;	m44 = mat.m44;
}

GtMatrix4::GtMatrix4(float _11, float _12, float _13, float _14, 
					 float _21, float _22, float _23, float _24, 
					 float _31, float _32, float _33, float _34, 
					 float _41, float _42, float _43, float _44)
{
	m11 = _11;	m12 = _12;	m13 = _13;	m14 = _14;
	m21 = _21;	m22 = _22;	m23 = _23;	m24 = _24;
	m31 = _31;	m32 = _32;	m33 = _33;	m34 = _34;
	m41 = _41;	m42 = _42;	m43 = _43;	m44 = _44;
}

GtMatrix4 GtMatrix4::MakeTranspose(void)
{
	GtMatrix4 result;
	
	GtPoint4f row[4];
	GetRow( 0, row[0] );
	GetRow( 1, row[1] );
	GetRow( 2, row[2] );
	GetRow( 3, row[3] );

    return GtMatrix4( row[0], row[1], row[2], row[3] );
}
/*
//  Determinent     
float GtMatrix4::MakeDeterminent(void)
{
	return m[0][0]*m[1][1]*m[2][2]*m[3][3] + m[0][0]*m[1][2]*m[2][3]*m[3][1] + m[0][0]*m[1][3]*m[2][1]*m[3][2] + 
	+	m[0][1]*m[1][0]*m[2][3]*m[3][2] + m[0][1]*m[1][2]*m[2][0]*m[3][3] + m[0][1]*m[1][3]*m[2][2]*m[3][0]  
	+	m[0][2]*m[1][0]*m[2][1]*m[3][3] + m[0][2]*m[1][1]*m[2][3]*m[3][0] + m[0][2]*m[1][3]*m[2][0]*m[3][1]  
	+	m[0][3]*m[1][0]*m[2][2]*m[3][1] + m[0][3]*m[1][1]*m[2][0]*m[3][2] + m[0][3]*m[1][2]*m[2][1]*m[3][0] 
	-	m[0][0]*m[1][1]*m[2][3]*m[3][2] - m[0][0]*m[1][2]*m[2][1]*m[3][3] - m[0][0]*m[1][3]*m[2][2]*m[3][1]
	-	m[0][1]*m[1][0]*m[2][2]*m[3][3] - m[0][1]*m[1][2]*m[2][3]*m[3][0] - m[0][1]*m[1][3]*m[2][0]*m[3][2]
	-	m[0][2]*m[1][0]*m[2][3]*m[3][1] - m[0][2]*m[1][1]*m[2][0]*m[3][3] - m[0][2]*m[1][3]*m[2][1]*m[3][0]
	-	m[0][3]*m[1][0]*m[2][1]*m[3][2] - m[0][3]*m[1][1]*m[2][2]*m[3][0] - m[0][3]*m[1][2]*m[2][0]*m[3][1];
}

GtMatrix4 GtMatrix4::MakeInverse() const
{
	GtMatrix4 result;
	result = *this;
	float determinent = 1.0f/result.MakeDeterminent();

	result.m[0][0] = m[1][1]*m[2][2]*m[3][3] + m[1][2]*m[2][3]*m[3][1] + m[1][3]*m[2][1]*m[3][2] - m[1][1]*m[2][3]*m[3][2] - m[1][2]*m[2][1]*m[3][3] - m[1][3]*m[2][2]*m[3][1];
	result.m[0][1] = m[0][1]*m[2][3]*m[3][2] + m[0][2]*m[2][1]*m[3][3] + m[0][3]*m[2][2]*m[3][1] - m[0][1]*m[2][2]*m[3][3] - m[0][2]*m[2][3]*m[3][1] - m[0][3]*m[2][1]*m[3][2]; 
	result.m[0][2] = m[0][1]*m[1][2]*m[3][3] + m[0][2]*m[1][3]*m[3][1] + m[0][3]*m[1][1]*m[3][2] - m[0][1]*m[1][3]*m[3][2] - m[0][2]*m[1][1]*m[3][3] - m[0][3]*m[1][2]*m[3][1];
	result.m[0][3] = m[0][1]*m[1][3]*m[2][2] + m[0][2]*m[1][1]*m[2][3] + m[0][3]*m[1][2]*m[2][1] - m[0][1]*m[1][2]*m[2][3] - m[0][2]*m[1][3]*m[2][1] - m[0][3]*m[1][1]*m[2][2];
	result.m[1][0] = m[1][0]*m[2][3]*m[3][2] + m[1][2]*m[2][0]*m[3][3] + m[1][3]*m[2][2]*m[3][0] - m[1][0]*m[2][2]*m[3][3] - m[1][2]*m[2][3]*m[3][0] - m[1][3]*m[2][0]*m[3][2];
	result.m[1][1] = m[0][0]*m[2][2]*m[3][3] + m[0][2]*m[2][3]*m[3][0] + m[0][3]*m[2][0]*m[3][2] - m[0][0]*m[2][3]*m[3][2] - m[0][2]*m[2][0]*m[3][3] - m[0][3]*m[2][2]*m[3][0];
	result.m[1][2] = m[0][0]*m[1][3]*m[3][2] + m[0][2]*m[1][3]*m[3][3] + m[0][3]*m[1][2]*m[3][0] - m[0][0]*m[1][2]*m[3][3] - m[0][2]*m[1][3]*m[3][0] - m[0][3]*m[1][0]*m[3][2];
	result.m[1][3] = m[0][0]*m[1][2]*m[2][3] + m[0][2]*m[1][3]*m[2][0] + m[0][3]*m[1][0]*m[2][2] - m[0][0]*m[1][3]*m[2][2] - m[0][2]*m[1][0]*m[2][3] - m[0][3]*m[1][2]*m[2][0];
	result.m[2][0] = m[1][0]*m[2][1]*m[3][3] + m[1][1]*m[2][3]*m[3][0] + m[1][3]*m[2][0]*m[3][1] - m[1][0]*m[2][3]*m[3][1] - m[1][1]*m[2][0]*m[3][3] - m[1][3]*m[2][1]*m[3][0];
	result.m[2][1] = m[0][0]*m[2][3]*m[3][1] + m[0][1]*m[2][0]*m[3][3] + m[0][3]*m[2][1]*m[3][0] - m[0][0]*m[2][1]*m[3][3] - m[0][1]*m[2][3]*m[3][0] - m[0][3]*m[2][0]*m[3][1];
	result.m[2][2] = m[0][0]*m[1][1]*m[3][3] + m[0][1]*m[1][3]*m[3][0] + m[0][3]*m[1][0]*m[3][1] - m[0][0]*m[1][3]*m[3][1] - m[0][1]*m[1][0]*m[3][3] - m[0][3]*m[1][1]*m[3][0];
	result.m[2][3] = m[0][0]*m[1][3]*m[2][1] + m[0][1]*m[1][0]*m[2][3] + m[0][3]*m[1][1]*m[2][0] - m[0][0]*m[1][1]*m[2][3] - m[0][1]*m[1][3]*m[2][0] - m[0][3]*m[1][0]*m[2][1];
	result.m[3][0] = m[1][0]*m[2][2]*m[3][1] + m[1][1]*m[2][0]*m[3][2] + m[1][2]*m[2][1]*m[3][0] - m[1][0]*m[2][1]*m[3][2] - m[1][1]*m[2][2]*m[3][0] - m[1][2]*m[2][0]*m[3][1];
	result.m[3][1] = m[0][0]*m[2][1]*m[3][2] + m[0][1]*m[2][2]*m[3][0] + m[0][2]*m[2][0]*m[3][1] - m[0][0]*m[2][2]*m[3][1] - m[0][1]*m[2][0]*m[3][2] - m[0][2]*m[2][1]*m[3][0];
	result.m[3][2] = m[0][0]*m[1][2]*m[3][1] + m[0][1]*m[1][0]*m[3][2] + m[0][2]*m[1][1]*m[3][0] - m[0][0]*m[1][1]*m[3][2] - m[0][1]*m[1][2]*m[3][0] - m[0][2]*m[1][0]*m[3][1];
	result.m[3][3] = m[0][0]*m[1][1]*m[2][2] + m[0][1]*m[1][2]*m[2][0] + m[0][2]*m[1][0]*m[2][1] - m[0][0]*m[1][2]*m[2][1] - m[0][1]*m[1][0]*m[2][2] - m[0][2]*m[1][1]*m[2][0];

	for(int i=0; i<16; i++)
		result.loop[i]*=determinent;

	return result;
}*/
/*
//    
GtMatrix4 GtMatrix4::MakeInverse() const
{
	long indxc[4], indxr[4], ipiv[4];
	long i, icol, irow, j, ir, ic;
	float big, dum, pivinv, temp, bb;
	float a[4][4];
	float b[4][4];

	for(int i=0; i<3; i++)
	{
		for(int j=0; j<3; j++)
		{
			b[i][j] = (*this).m[i][j];
		}
	}
	ipiv[0] = -1;
	ipiv[1] = -1;
	ipiv[2] = -1;
	ipiv[3] = -1;
	a[0][0] = b[0][0];
	a[1][0] = b[1][0];
	a[2][0] = b[2][0];
	a[3][0] = b[3][0];
	a[0][1] = b[0][1];
	a[1][1] = b[1][1];
	a[2][1] = b[2][1];
	a[3][1] = b[3][1];
	a[0][2] = b[0][2];
	a[1][2] = b[1][2];
	a[2][2] = b[2][2];
	a[3][2] = b[3][2];
	a[0][3] = b[0][3];
	a[1][3] = b[1][3];
	a[2][3] = b[2][3];
	a[3][3] = b[3][3];
	for (i = 0; i < 4; i++) {
	big = 0.0f;
	for (j = 0; j < 4; j++) {
	if (ipiv[j] != 0) {
	if (ipiv[0] == -1) {
	if ((bb = (float) fabs(a[j][0])) > big) {
	big = bb;
	irow = j;
	icol = 0;
	}
	} else if (ipiv[0] > 0) {
	return 0;
	}
	if (ipiv[1] == -1) {
	if ((bb = (float) fabs((float) a[j][1])) > big) {
	big = bb;
	irow = j;
	icol = 1;
	}
	} else if (ipiv[1] > 0) {
	return 0;
	}
	if (ipiv[2] == -1) {
	if ((bb = (float) fabs((float) a[j][2])) > big) {
	big = bb;
	irow = j;
	icol = 2;
	}
	} else if (ipiv[2] > 0) {
	return 0;
	}
	if (ipiv[3] == -1) {
	if ((bb = (float) fabs((float) a[j][3])) > big) {
	big = bb;
	irow = j;
	icol = 3;
	}
	} else if (ipiv[3] > 0) {
	return 0;
	}
	}
	}
	++(ipiv[icol]);
	if (irow != icol) {
	temp = a[irow][0];
	a[irow][0] = a[icol][0];
	a[icol][0] = temp;
	temp = a[irow][1];
	a[irow][1] = a[icol][1];
	a[icol][1] = temp;
	temp = a[irow][2];
	a[irow][2] = a[icol][2];
	a[icol][2] = temp;
	temp = a[irow][3];
	a[irow][3] = a[icol][3];
	a[icol][3] = temp;
	}
	indxr[i] = irow;
	indxc[i] = icol;
	if (a[icol][icol] == 0.0) {
	return 0;
	}
	pivinv = 1.0f / a[icol][icol];
	a[icol][icol] = 1.0f;
	a[icol][0] *= pivinv;
	a[icol][1] *= pivinv;
	a[icol][2] *= pivinv;
	a[icol][3] *= pivinv;
	if (icol != 0) {
	dum = a[0][icol];
	a[0][icol] = 0.0f;
	a[0][0] -= a[icol][0] * dum;
	a[0][1] -= a[icol][1] * dum;
	a[0][2] -= a[icol][2] * dum;
	a[0][3] -= a[icol][3] * dum;
	}
	if (icol != 1) {
	dum = a[1][icol];
	a[1][icol] = 0.0f;
	a[1][0] -= a[icol][0] * dum;
	a[1][1] -= a[icol][1] * dum;
	a[1][2] -= a[icol][2] * dum;
	a[1][3] -= a[icol][3] * dum;
	}
	if (icol != 2) {
	dum = a[2][icol];
	a[2][icol] = 0.0f;
	a[2][0] -= a[icol][0] * dum;
	a[2][1] -= a[icol][1] * dum;
	a[2][2] -= a[icol][2] * dum;
	a[2][3] -= a[icol][3] * dum;
	}
	if (icol != 3) {
	dum = a[3][icol];
	a[3][icol] = 0.0f;
	a[3][0] -= a[icol][0] * dum;
	a[3][1] -= a[icol][1] * dum;
	a[3][2] -= a[icol][2] * dum;
	a[3][3] -= a[icol][3] * dum;
	}
	}
	if (indxr[3] != indxc[3]) {
	ir = indxr[3];
	ic = indxc[3];
	temp = a[0][ir];
	a[0][ir] = a[0][ic];
	a[0][ic] = temp;
	temp = a[1][ir];
	a[1][ir] = a[1][ic];
	a[1][ic] = temp;
	temp = a[2][ir];
	a[2][ir] = a[2][ic];
	a[2][ic] = temp;
	temp = a[3][ir];
	a[3][ir] = a[3][ic];
	a[3][ic] = temp;
	}
	if (indxr[2] != indxc[2]) {
	ir = indxr[2];
	ic = indxc[2];
	temp = a[0][ir];
	a[0][ir] = a[0][ic];
	a[0][ic] = temp;
	temp = a[1][ir];
	a[1][ir] = a[1][ic];
	a[1][ic] = temp;
	temp = a[2][ir];
	a[2][ir] = a[2][ic];
	a[2][ic] = temp;
	temp = a[3][ir];
	a[3][ir] = a[3][ic];
	a[3][ic] = temp;
	}
	if (indxr[1] != indxc[1]) {
	ir = indxr[1];
	ic = indxc[1];
	temp = a[0][ir];
	a[0][ir] = a[0][ic];
	a[0][ic] = temp;
	temp = a[1][ir];
	a[1][ir] = a[1][ic];
	a[1][ic] = temp;
	temp = a[2][ir];
	a[2][ir] = a[2][ic];
	a[2][ic] = temp;
	temp = a[3][ir];
	a[3][ir] = a[3][ic];
	a[3][ic] = temp;
	}
	if (indxr[0] != indxc[0]) {
	ir = indxr[0];
	ic = indxc[0];
	temp = a[0][ir];
	a[0][ir] = a[0][ic];
	a[0][ic] = temp;
	temp = a[1][ir];
	a[1][ir] = a[1][ic];
	a[1][ic] = temp;
	temp = a[2][ir];
	a[2][ir] = a[2][ic];
	a[2][ic] = temp;
	temp = a[3][ir];
	a[3][ir] = a[3][ic];
	a[3][ic] = temp;
	}

	GtMatrix4 result;

	for(int i=0; i<4; i++)
	{
		for(int j=0;j <4; j++)
		{
			result.m[i][j] = a[i][j];
		}
	}

	return result;
}*/
/*
//    
GtMatrix4 GtMatrix4::MakeInverse() const
{
	GtMatrix4 result = *this;
	float arr[16];
	for(int i=0; i<16; i++)
	{
		arr[i] = (*this).loop[i];
	}
	float det = arr[3]*arr[6]*arr[9]*arr[12] - arr[2]*arr[7]*arr[9]*arr[12] - arr[3]*arr[5]*arr[10]*arr[12] + arr[1]*arr[7]*arr[10]*arr[12] + arr[2]*arr[5]*arr[11]*arr[12] - arr[1]*arr[6]*arr[11]*arr[12] - arr[3]*arr[6]*arr[8]*arr[13] + arr[2]*arr[7]*arr[8]*arr[13] + arr[3]*arr[4]*arr[10]*arr[13] - arr[0]*arr[7]*arr[10]*arr[13] - arr[2]*arr[4]*arr[11]*arr[13] + arr[0]*arr[6]*arr[11]*arr[13] + arr[3]*arr[5]*arr[8]*arr[14] - arr[1]*arr[7]*arr[8]*arr[14] - arr[3]*arr[4]*arr[9]*arr[14] + arr[0]*arr[7]*arr[9]*arr[14] + arr[1]*arr[4]*arr[11]*arr[14] - arr[0]*arr[5]*arr[11]*arr[14] - arr[2]*arr[5]*arr[8]*arr[15] + arr[1]*arr[6]*arr[8]*arr[15] + arr[2]*arr[4]*arr[9]*arr[15] - arr[0]*arr[6]*arr[9]*arr[15] - arr[1]*arr[4]*arr[10]*arr[15] + arr[0]*arr[5]*arr[10]*arr[15];

	result.loop[ 0] =(-arr[7]*arr[10]*arr[13] + arr[6]*arr[11]*arr[13] + arr[7]*arr[9]*arr[14] - arr[5]*arr[11]*arr[14] - arr[6]*arr[9]*arr[15] + arr[5]*arr[10]*arr[15])/det;
	result.loop[ 1] =(arr[3]*arr[10]*arr[13] - arr[2]*arr[11]*arr[13] - arr[3]*arr[9]*arr[14] + arr[1]*arr[11]*arr[14] + arr[2]*arr[9]*arr[15] - arr[1]*arr[10]*arr[15])/det;
	result.loop[ 2] =(-arr[3]*arr[6]*arr[13] + arr[2]*arr[7]*arr[13] + arr[3]*arr[5]*arr[14] - arr[1]*arr[7]*arr[14] - arr[2]*arr[5]*arr[15] + arr[1]*arr[6]*arr[15])/det;
	result.loop[ 3] =(arr[3]*arr[6]*arr[9] - arr[2]*arr[7]*arr[9] - arr[3]*arr[5]*arr[10] + arr[1]*arr[7]*arr[10] + arr[2]*arr[5]*arr[11] - arr[1]*arr[6]*arr[11])/det;
	result.loop[ 4] =(arr[7]*arr[10]*arr[12] - arr[6]*arr[11]*arr[12] - arr[7]*arr[8]*arr[14] + arr[4]*arr[11]*arr[14] + arr[6]*arr[8]*arr[15] - arr[4]*arr[10]*arr[15])/det;
	result.loop[ 5] =(-arr[3]*arr[10]*arr[12] + arr[2]*arr[11]*arr[12] + arr[3]*arr[8]*arr[14] - arr[0]*arr[11]*arr[14] - arr[2]*arr[8]*arr[15] + arr[0]*arr[10]*arr[15])/det;
	result.loop[ 6] =(arr[3]*arr[6]*arr[12] - arr[2]*arr[7]*arr[12] - arr[3]*arr[4]*arr[14] + arr[0]*arr[7]*arr[14] + arr[2]*arr[4]*arr[15] - arr[0]*arr[6]*arr[15])/det;
	result.loop[ 7] =(-arr[3]*arr[6]*arr[8] + arr[2]*arr[7]*arr[8] + arr[3]*arr[4]*arr[10] - arr[0]*arr[7]*arr[10] - arr[2]*arr[4]*arr[11] + arr[0]*arr[6]*arr[11])/det;
	result.loop[ 8] =(-arr[7]*arr[9]*arr[12] + arr[5]*arr[11]*arr[12] + arr[7]*arr[8]*arr[13] - arr[4]*arr[11]*arr[13] - arr[5]*arr[8]*arr[15] + arr[4]*arr[9]*arr[15])/det;
	result.loop[ 9] =(arr[3]*arr[9]*arr[12] - arr[1]*arr[11]*arr[12] - arr[3]*arr[8]*arr[13] + arr[0]*arr[11]*arr[13] + arr[1]*arr[8]*arr[15] - arr[0]*arr[9]*arr[15])/det;
	result.loop[10] =(-arr[3]*arr[5]*arr[12] + arr[1]*arr[7]*arr[12] + arr[3]*arr[4]*arr[13] - arr[0]*arr[7]*arr[13] - arr[1]*arr[4]*arr[15] + arr[0]*arr[5]*arr[15])/det;
	result.loop[11] =(arr[3]*arr[5]*arr[8] - arr[1]*arr[7]*arr[8] - arr[3]*arr[4]*arr[9] + arr[0]*arr[7]*arr[9] + arr[1]*arr[4]*arr[11] - arr[0]*arr[5]*arr[11])/det;
	result.loop[12] =(arr[6]*arr[9]*arr[12] - arr[5]*arr[10]*arr[12] - arr[6]*arr[8]*arr[13] + arr[4]*arr[10]*arr[13] + arr[5]*arr[8]*arr[14] - arr[4]*arr[9]*arr[14])/det;
	result.loop[13] =(-arr[2]*arr[9]*arr[12] + arr[1]*arr[10]*arr[12] + arr[2]*arr[8]*arr[13] - arr[0]*arr[10]*arr[13] - arr[1]*arr[8]*arr[14] + arr[0]*arr[9]*arr[14])/det;
	result.loop[14] =(arr[2]*arr[5]*arr[12] - arr[1]*arr[6]*arr[12] - arr[2]*arr[4]*arr[13] + arr[0]*arr[6]*arr[13] + arr[1]*arr[4]*arr[14] - arr[0]*arr[5]*arr[14])/det;
	result.loop[15] =(-arr[2]*arr[5]*arr[8] + arr[1]*arr[6]*arr[8] + arr[2]*arr[4]*arr[9] - arr[0]*arr[6]*arr[9] - arr[1]*arr[4]*arr[10] + arr[0]*arr[5]*arr[10])/det;

	return result;
}*/

//  ڿ   
GtMatrix4 GtMatrix4::MakeInverse() const
{
	GtMatrix4 result;
	result = *this;
	// Intel  
	float tmp[12]; // temp array for pairs 
	float src[16]; // array of transpose source matrix 
	float det; // determinant 
	// transpose matrix 
	for (int i = 0; i < 4; i++)
	{
		src[i]		=	result.loop[i*4];
		src[i + 4]	=	result.loop[i*4 + 1];
		src[i + 8]	=	result.loop[i*4 + 2];
		src[i + 12]	=	result.loop[i*4 + 3];
	}
	// calculate pairs for first 8 elements (cofactors) 
	tmp[0]  = src[10] * src[15];
	tmp[1]  = src[11] * src[14];
	tmp[2]  = src[9]  * src[15];
	tmp[3]  = src[11] * src[13];
	tmp[4]  = src[9]  * src[14];
	tmp[5]  = src[10] * src[13];
	tmp[6]  = src[8]  * src[15];
	tmp[7]  = src[11] * src[12];
	tmp[8]  = src[8]  * src[14];
	tmp[9]  = src[10] * src[12];
	tmp[10] = src[8]  * src[13];
	tmp[11] = src[9]  * src[12];
	// calculate first 8 elements (cofactors) 
	result.loop[0] =  tmp[0]*src[5] + tmp[3]*src[6] + tmp[4]*src[7];
	result.loop[0] -= tmp[1]*src[5] + tmp[2]*src[6] + tmp[5]*src[7];
	result.loop[1] =  tmp[1]*src[4] + tmp[6]*src[6] + tmp[9]*src[7];
	result.loop[1] -= tmp[0]*src[4] + tmp[7]*src[6] + tmp[8]*src[7];
	result.loop[2] =  tmp[2]*src[4] + tmp[7]*src[5] + tmp[10]*src[7];
	result.loop[2] -= tmp[3]*src[4] + tmp[6]*src[5] + tmp[11]*src[7];
	result.loop[3] =  tmp[5]*src[4] + tmp[8]*src[5] + tmp[11]*src[6];
	result.loop[3] -= tmp[4]*src[4] + tmp[9]*src[5] + tmp[10]*src[6];
	result.loop[4] =  tmp[1]*src[1] + tmp[2]*src[2] + tmp[5]*src[3];
	result.loop[4] -= tmp[0]*src[1] + tmp[3]*src[2] + tmp[4]*src[3];
	result.loop[5] =  tmp[0]*src[0] + tmp[7]*src[2] + tmp[8]*src[3];
	result.loop[5] -= tmp[1]*src[0] + tmp[6]*src[2] + tmp[9]*src[3];
	result.loop[6] =  tmp[3]*src[0] + tmp[6]*src[1] + tmp[11]*src[3];
	result.loop[6] -= tmp[2]*src[0] + tmp[7]*src[1] + tmp[10]*src[3];
	result.loop[7] =  tmp[4]*src[0] + tmp[9]*src[1] + tmp[10]*src[2];
	result.loop[7] -= tmp[5]*src[0] + tmp[8]*src[1] + tmp[11]*src[2];

	// calculate pairs for second 8 elements (cofactors) 
	tmp[0]  = src[2]*src[7];
	tmp[1]  = src[3]*src[6];
	tmp[2]  = src[1]*src[7];
	tmp[3]  = src[3]*src[5];
	tmp[4]  = src[1]*src[6];
	tmp[5]  = src[2]*src[5];
	tmp[6]  = src[0]*src[7];
	tmp[7]  = src[3]*src[4];
	tmp[8]  = src[0]*src[6];
	tmp[9]  = src[2]*src[4];
	tmp[10] = src[0]*src[5];
	tmp[11] = src[1]*src[4];
	// calculate second 8 elements (cofactors) 
	result.loop[8] =  tmp[0]  *  src[13] + tmp[3]*src[14]  + tmp[4]  * src[15];
	result.loop[8] -= tmp[1]  *  src[13] + tmp[2]*src[14]  + tmp[5]  * src[15];
	result.loop[9] =  tmp[1]  *  src[12] + tmp[6]*src[14]  + tmp[9]  * src[15];
	result.loop[9] -= tmp[0]  *  src[12] + tmp[7]*src[14]  + tmp[8]  * src[15];
	result.loop[10] = tmp[2]  *  src[12] + tmp[7]*src[13]  + tmp[10] * src[15];
	result.loop[10]-= tmp[3]  *  src[12] + tmp[6]*src[13]  + tmp[11] * src[15];
	result.loop[11] = tmp[5]  *  src[12] + tmp[8]*src[13]  + tmp[11] * src[14];
	result.loop[11]-= tmp[4]  *  src[12] + tmp[9]*src[13]  + tmp[10] * src[14];
	result.loop[12] = tmp[2]  *  src[10] + tmp[5]*src[11]  + tmp[1]  * src[9];
	result.loop[12]-= tmp[4]  *  src[11] + tmp[0]*src[9]   + tmp[3]  * src[10];
	result.loop[13] = tmp[8]  *  src[11] + tmp[0]*src[8]   + tmp[7]  * src[10];
	result.loop[13]-= tmp[6]  *  src[10] + tmp[9]*src[11]  + tmp[1]  * src[8];
	result.loop[14] = tmp[6]  *  src[9]  + tmp[11]*src[11] + tmp[3]  * src[8];
	result.loop[14]-= tmp[10] *  src[11] + tmp[2]*src[8]   + tmp[7]  * src[9];
	result.loop[15] = tmp[10] *  src[10] + tmp[4]*src[8]   + tmp[9]  * src[9];
	result.loop[15]-= tmp[8]  *  src[9]  + tmp[11]*src[10] + tmp[5]  * src[8];
	// calculate determinant 
	det=src[0]*result.loop[0]+src[1]*result.loop[1]+src[2]*result.loop[2]+src[3]*result.loop[3];
	// calculate matrix inverse 
	det = 1/det;
	for (int j = 0; j < 16; j++)
		result.loop[j] *= det;

	return result;
}

void GtMatrix4::MakeZero(void)
{
	m11 = 0.0f;			m12 = 0.0f;			m13 = 0.0f;		m14 = 0.0f;
	m21 = 0.0f;			m22 = 0.0f;			m23 = 0.0f;		m24 = 0.0f;
	m31 = 0.0f;			m32 = 0.0f;			m33 = 0.0f;		m34 = 0.0f;
	m41 = 0.0f;			m42 = 0.0f;			m43 = 0.0f;		m44 = 0.0f;
}

void GtMatrix4::MakeIdentity(void)
{
	m11 = 1.0f;			m12 = 0.0f;			m13 = 0.0f;		m14 = 0.0f;
	m21 = 0.0f;			m22 = 1.0f;			m23 = 0.0f;		m24 = 0.0f;
	m31 = 0.0f;			m32 = 0.0f;			m33 = 1.0f;		m34 = 0.0f;
	m41 = 0.0f;			m42 = 0.0f;			m43 = 0.0f;		m44 = 1.0f;
}

void GtMatrix4::MakeXRotation(float fAngle)
{
    float sn, cs;
    GtSinCos(fAngle, sn, cs);
    
	m[0][0] = 1.0f;
	m[0][1] = 0.0f;
	m[0][2] = 0.0f;
	m[0][3] = 0.0f;

	m[1][0] = 0.0f;
	m[1][1] = cs;
	m[1][2] = sn;
	m[1][3] = 0.0f;

	m[2][0] = 0.0f;
	m[2][1] = -sn;
	m[2][2] = cs;
	m[2][3] = 0.0f;

	m[3][0] = 0.0f;
	m[3][1] = 0.0f;
	m[3][2] = 0.0f;
	m[3][3] = 1.0f;
}

void GtMatrix4::MakeYRotation(float fAngle)
{
    float sn, cs;
	GtSinCos(fAngle, sn, cs);
    
	m[0][0] = cs;
	m[0][1] = 0.0f;
	m[0][2] = -sn;
	m[0][3] = 0.0f;

	m[1][0] = 0.0f;
	m[1][1] = 1.0f;
	m[1][2] = 0.0f;
	m[1][3] = 0.0f;

	m[2][0] = sn;
	m[2][1] = 0.0f;
	m[2][2] = cs;
	m[2][3] = 0.0f;

	m[3][0] = 0.0f;
	m[3][1] = 0.0f;
	m[3][2] = 0.0f;
	m[3][3] = 1.0f;
}

void GtMatrix4::MakeZRotation(float fAngle)
{
    float sn, cs;
	GtSinCos(fAngle, sn, cs);

	m[0][0] = cs;
	m[0][1] = sn;
	m[0][2] = 0.0f;
	m[0][3] = 0.0f;

	m[1][0] = -sn;
	m[1][1] = cs;
	m[1][2] = 0.0f;
	m[1][3] = 0.0f;

	m[2][0] = 0.0f;
	m[2][1] = 0.0f;
	m[2][2] = 1.0f;
	m[2][3] = 0.0f;

	m[3][0] = 0.0f;
	m[3][1] = 0.0f;
	m[3][2] = 0.0f;
	m[3][3] = 1.0f;
}

void GtMatrix4::MakeXYZRotation(float x, float y, float z)
{
	GtMatrix4 matX, matY, matZ;
	matX.MakeIdentity();
	matY.MakeIdentity();
	matZ.MakeIdentity();
	matX.MakeXRotation(x);
	matY.MakeYRotation(y);
	matZ.MakeZRotation(z);
	*this = matX * matY * matZ;
}

void GtMatrix4::MakeRotation(float fAngle, float x, float y, float z)
{
    float sn, cs;
    GtSinCos(fAngle, sn, cs);

    float omcs = 1.0f-cs;
    float x2 = x*x;
    float y2 = y*y;
    float z2 = z*z;
    float xym = x*y*omcs;
    float xzm = x*z*omcs;
    float yzm = y*z*omcs;
    float xsin = x*sn;
    float ysin = y*sn;
    float zsin = z*sn;
    
    m[0][0] = x2*omcs+cs;
    m[0][1] = xym+zsin;
    m[0][2] = xzm-ysin;
	m[0][3] = 0.0f;

    m[1][0] = xym-zsin;
    m[1][1] = y2*omcs+cs;
    m[1][2] = yzm+xsin;
	m[1][3] = 0.0f;

    m[2][0] = xzm+ysin;
    m[2][1] = yzm-xsin;
    m[2][2] = z2*omcs+cs;
	m[2][3] = 0.0f;
	
    m[3][0] = 0.0f;
    m[3][1] = 0.0f;
    m[3][2] = 0.0f;
	m[3][3] = 1.0f;
}

void GtMatrix4::MakeScaling(float x, float y, float z)
{
	m[0][0] = x;
	m[0][1] = 0.0f;
	m[0][2] = 0.0f;
	m[0][3] = 0.0f;

	m[1][0] = 0.0f;
	m[1][1] = y;
	m[1][2] = 0.0f;
	m[1][3] = 0.0f;

	m[2][0] = 0.0f;
	m[2][1] = 0.0f;
	m[2][2] = z;
	m[2][3] = 0.0f;

	m[3][0] = 0.0f;
	m[3][1] = 0.0f;
	m[3][2] = 0.0f;
	m[3][3] = 1.0f;
}

void GtMatrix4::MakeTranslation(float x, float y, float z)
{
	m[0][0] = 1.0f;
	m[0][1] = 0.0f;
	m[0][2] = 0.0f;
	m[0][3] = 0.0f;

	m[1][0] = 0.0f;
	m[1][1] = 1.0f;
	m[1][2] = 0.0f;
	m[1][3] = 0.0f;

	m[2][0] = 0.0f;
	m[2][1] = 0.0f;
	m[2][2] = 1.0f;
	m[2][3] = 0.0f;

	m[3][0] = x;
	m[3][1] = y;
	m[3][2] = z;
	m[3][3] = 1.0f;
}

void GtMatrix4::LookAtLH(GtPoint3f& ptEyePt,GtPoint3f& ptLookatPt,GtPoint3f& ptUpPt)
{
	GtPoint3f zAxis = (ptLookatPt - ptEyePt).Normalized();
	GtPoint3f xAxis = ptUpPt.UnitCross( zAxis );
	GtPoint3f yAxis = zAxis.Cross( xAxis );

	m[0][0] = xAxis.x;	m[0][1] = yAxis.x;	m[0][2] = zAxis.x;	m[0][3] = 0.0f;
	m[1][0] = xAxis.y;	m[1][1] = yAxis.y;	m[1][2] = zAxis.y;	m[1][3] = 0.0f;
	m[2][0] = xAxis.z;	m[2][1] = yAxis.z;	m[2][2] = zAxis.z;	m[2][3] = 0.0f;
	m[3][0] = -(xAxis.Dot(ptEyePt));	
	m[3][1] = -(yAxis.Dot(ptEyePt));	
	m[3][2] = -(zAxis.Dot(ptEyePt)); 	
	m[3][3] = 1.0f;
}

void GtMatrix4::LookAtRH(GtPoint3f& ptEyePt,GtPoint3f& ptLookatPt,GtPoint3f& ptUpPt)
{
	GtPoint3f zAxis = (ptEyePt - ptLookatPt).Normalized();
	GtPoint3f xAxis = ptUpPt.UnitCross(zAxis);
	GtPoint3f yAxis = zAxis.Cross(xAxis);

	m[0][0] = xAxis.x;	m[0][1] = yAxis.x;	m[0][2] = zAxis.x;	m[0][3] = 0.0f;
	m[1][0] = xAxis.y;	m[1][1] = yAxis.y;	m[1][2] = zAxis.y;	m[1][3] = 0.0f;
	m[2][0] = xAxis.z;	m[2][1] = yAxis.z;	m[2][2] = zAxis.z;	m[2][3] = 0.0f;
	m[3][0] = -(xAxis.Dot(ptEyePt));	
	m[3][1] = -(yAxis.Dot(ptEyePt));	
	m[3][2] = -(zAxis.Dot(ptEyePt)); 	
	m[3][3] = 1.0f;
}

void GtMatrix4::PerspectiveFovLH(const float fFovy,const float fAspect,const float fZnear,const float fZfar)
{
	float h = 1/GtTanf(fFovy/2.0f);
	float w = h/fAspect;
	float q = fZfar/(fZfar-fZnear);

	m11 = w;			m12 = 0.0f;			m13 = 0.0f;			m14 = 0.0f;
	m21 = 0.0f;			m22 = h;			m23 = 0.0f;			m24 = 0.0f;
	m31 = 0.0f;			m32 = 0.0f;			m33 = q;			m34 = 1.0f;
	m41 = 0.0f;			m42 = 0.0f;			m43 = -fZnear * q;	m44 = 0.0f;
}

void GtMatrix4::PerspectiveFovRH(const float fFovy,const float fAspect,const float fZnear,const float fZfar)
{
	float h = 1/GtTanf(fFovy/2.0f);
	float w = h / fAspect;

	m11 = w;			m12 = 0.0f;			m13 = 0.0f;								m14 =  0.0f;
	m21 = 0.0f;			m22 = h;			m23 = 0.0f;								m24 =  0.0f;
	m31 = 0.0f;			m32 = 0.0f;			m33 = fZfar/(fZnear-fZfar);				m34 = -1.0f;
	m41 = 0.0f;			m42 = 0.0f;			m43 = fZnear * fZfar/(fZnear - fZfar);	m44 =  0.0f;
}

void GtMatrix4::OrthoLH(const float fWidth,const float fHeight,const float fZnear,const float fZfar)
{
	m11 = 2.0f/fWidth;	m12 = 0.0f;			m13 = 0.0f;						m14 = 0.0f;
	m21 = 0.0f;			m22 = 2.0f/fHeight;	m23 = 0.0f;						m24 = 0.0f;
	m31 = 0.0f;			m32 = 0.0f;			m33 = 1.0f/(fZfar - fZnear);	m34 = 0.0f;
	m41 = 0.0f;			m42 = 0.0f;			m43 = fZnear/(fZnear-fZfar);	m44 = 1.0f;
}

void GtMatrix4::OrthoRH(const float fWidth,const float fHeight,const float fZnear,const float fZfar)
{
	m11 = 2.0f/fWidth;	m12 = 0.0f;			m13 = 0.0f;						m14 = 0.0f;
	m21 = 0.0f;			m22 = 2.0f/fHeight;	m23 = 0.0f;						m24 = 0.0f;
	m31 = 0.0f;			m32 = 0.0f;			m33 = 1.0f/(fZnear - fZfar);	m34 = 0.0f;
	m41 = 0.0f;			m42 = 0.0f;			m43 = -fZnear/(fZfar-fZnear);	m44 = 1.0f;
}

GtMatrix4 GtMatrix4::operator	+	(const GtMatrix4& mat) const
{
	GtMatrix4 result = *this;
	result.m[0][0] += mat.m[0][0];
	result.m[0][1] += mat.m[0][1];
	result.m[0][2] += mat.m[0][2];
	result.m[0][3] += mat.m[0][3];
	result.m[1][0] += mat.m[1][0];
	result.m[1][1] += mat.m[1][1];
	result.m[1][2] += mat.m[1][2];
	result.m[1][3] += mat.m[1][3];
	result.m[2][0] += mat.m[2][0];
	result.m[2][1] += mat.m[2][1];
	result.m[2][2] += mat.m[2][2];
	result.m[2][3] += mat.m[2][3];
	result.m[3][0] += mat.m[3][0];
	result.m[3][1] += mat.m[3][1];
	result.m[3][2] += mat.m[3][2];
	result.m[3][3] += mat.m[3][3];
	return result;
}

GtMatrix4 GtMatrix4::operator	-	(const GtMatrix4& mat) const
{
	GtMatrix4 result = *this;
	result.m[0][0] -= mat.m[0][0];
	result.m[0][1] -= mat.m[0][1];
	result.m[0][2] -= mat.m[0][2];
	result.m[0][3] -= mat.m[0][3];
	result.m[1][0] -= mat.m[1][0];
	result.m[1][1] -= mat.m[1][1];
	result.m[1][2] -= mat.m[1][2];
	result.m[1][3] -= mat.m[1][3];
	result.m[2][0] -= mat.m[2][0];
	result.m[2][1] -= mat.m[2][1];
	result.m[2][2] -= mat.m[2][2];
	result.m[2][3] -= mat.m[2][3];
	result.m[3][0] -= mat.m[3][0];
	result.m[3][1] -= mat.m[3][1];
	result.m[3][2] -= mat.m[3][2];
	result.m[3][3] -= mat.m[3][3];
	return result;
}

GtMatrix4 GtMatrix4::operator *(const float fValue) const
{
	GtMatrix4 result = *this;
	result.m[0][0] *= fValue;
	result.m[0][1] *= fValue;
	result.m[0][2] *= fValue;
	result.m[0][3] *= fValue;
	result.m[1][0] *= fValue;
	result.m[1][1] *= fValue;
	result.m[1][2] *= fValue;
	result.m[1][3] *= fValue;
	result.m[2][0] *= fValue;
	result.m[2][1] *= fValue;
	result.m[2][2] *= fValue;
	result.m[2][3] *= fValue;
	result.m[3][0] *= fValue;
	result.m[3][1] *= fValue;
	result.m[3][2] *= fValue;
	result.m[3][3] *= fValue;
	return result;
}

GtMatrix4 GtMatrix4::operator *(const GtMatrix4& mat) const
{
	GtMatrix4 result;
	result.m11 = m11 * mat.m11 + m12 * mat.m21 + m13 * mat.m31 + m14 * mat.m41;
	result.m12 = m11 * mat.m12 + m12 * mat.m22 + m13 * mat.m32 + m14 * mat.m42;
	result.m13 = m11 * mat.m13 + m12 * mat.m23 + m13 * mat.m33 + m14 * mat.m43;
	result.m14 = m11 * mat.m14 + m12 * mat.m24 + m13 * mat.m34 + m14 * mat.m44;

	result.m21 = m21 * mat.m11 + m22 * mat.m21 + m23 * mat.m31 + m24 * mat.m41;
	result.m22 = m21 * mat.m12 + m22 * mat.m22 + m23 * mat.m32 + m24 * mat.m42;
	result.m23 = m21 * mat.m13 + m22 * mat.m23 + m23 * mat.m33 + m24 * mat.m43;
	result.m24 = m21 * mat.m14 + m22 * mat.m24 + m23 * mat.m34 + m24 * mat.m44;

	result.m31 = m31 * mat.m11 + m32 * mat.m21 + m33 * mat.m31 + m34 * mat.m41;
	result.m32 = m31 * mat.m12 + m32 * mat.m22 + m33 * mat.m32 + m34 * mat.m42;
	result.m33 = m31 * mat.m13 + m32 * mat.m23 + m33 * mat.m33 + m34 * mat.m43;
	result.m34 = m31 * mat.m14 + m32 * mat.m24 + m33 * mat.m34 + m34 * mat.m44;

	result.m41 = m41 * mat.m11 + m41 * mat.m21 + m43 * mat.m31 + m44 * mat.m41;
	result.m42 = m41 * mat.m12 + m42 * mat.m22 + m43 * mat.m32 + m44 * mat.m42;
	result.m43 = m41 * mat.m13 + m42 * mat.m23 + m43 * mat.m33 + m44 * mat.m43;
	result.m44 = m41 * mat.m14 + m42 * mat.m24 + m43 * mat.m34 + m44 * mat.m44;
	return result;
}

GtMatrix4 GtMatrix4::operator /(const GtMatrix4& mat) const
{
//	GtMatrix4 _mat = mat.MakeInverse();
	return (*this) * mat.MakeInverse();
}

GtMatrix4 GtMatrix4::operator +=(const GtMatrix4& mat)
{
	m[0][0] += mat.m[0][0];
	m[0][1] += mat.m[0][1];
	m[0][2] += mat.m[0][2];
	m[0][3] += mat.m[0][3];
	m[1][0] += mat.m[1][0];
	m[1][1] += mat.m[1][1];
	m[1][2] += mat.m[1][2];
	m[1][3] += mat.m[1][3];
	m[2][0] += mat.m[2][0];
	m[2][1] += mat.m[2][1];
	m[2][2] += mat.m[2][2];
	m[2][3] += mat.m[2][3];
	m[3][0] += mat.m[3][0];
	m[3][1] += mat.m[3][1];
	m[3][2] += mat.m[3][2];
	m[3][3] += mat.m[3][3];

	return *this;
}

GtMatrix4 GtMatrix4::operator -=(const GtMatrix4& mat)
{
	m[0][0] -= mat.m[0][0];
	m[0][1] -= mat.m[0][1];
	m[0][2] -= mat.m[0][2];
	m[0][3] -= mat.m[0][3];
	m[1][0] -= mat.m[1][0];
	m[1][1] -= mat.m[1][1];
	m[1][2] -= mat.m[1][2];
	m[1][3] -= mat.m[1][3];
	m[2][0] -= mat.m[2][0];
	m[2][1] -= mat.m[2][1];
	m[2][2] -= mat.m[2][2];
	m[2][3] -= mat.m[2][3];
	m[3][0] -= mat.m[3][0];
	m[3][1] -= mat.m[3][1];
	m[3][2] -= mat.m[3][2];
	m[3][3] -= mat.m[3][3];

	return *this;
}

GtMatrix4 GtMatrix4::operator	*=	(const float fValue)
{
	m[0][0] *= fValue;
	m[0][1] *= fValue;
	m[0][2] *= fValue;
	m[0][3] *= fValue;
	m[1][0] *= fValue;
	m[1][1] *= fValue;
	m[1][2] *= fValue;
	m[1][3] *= fValue;
	m[2][0] *= fValue;
	m[2][1] *= fValue;
	m[2][2] *= fValue;
	m[2][3] *= fValue;
	m[3][0] *= fValue;
	m[3][1] *= fValue;
	m[3][2] *= fValue;
	m[3][3] *= fValue;

	return *this;
}

GtMatrix4 GtMatrix4::operator *=(const GtMatrix4& mat)
{
	GtMatrix4 result = *this;
	m11 = result.m11 * mat.m11 + result.m12 * mat.m21 + result.m13 * mat.m31 + result.m14 * mat.m41;
	m12 = result.m11 * mat.m12 + result.m12 * mat.m22 + result.m13 * mat.m32 + result.m14 * mat.m42;
	m13 = result.m11 * mat.m13 + result.m12 * mat.m23 + result.m13 * mat.m33 + result.m14 * mat.m43;
	m14 = result.m11 * mat.m14 + result.m12 * mat.m24 + result.m13 * mat.m34 + result.m14 * mat.m44;
																			   
	m21 = result.m21 * mat.m11 + result.m22 * mat.m21 + result.m23 * mat.m31 + result.m24 * mat.m41;
	m22 = result.m21 * mat.m12 + result.m22 * mat.m22 + result.m23 * mat.m32 + result.m24 * mat.m42;
	m23 = result.m21 * mat.m13 + result.m22 * mat.m23 + result.m23 * mat.m33 + result.m24 * mat.m43;
	m24 = result.m21 * mat.m14 + result.m22 * mat.m24 + result.m23 * mat.m34 + result.m24 * mat.m44;
																			   
	m31 = result.m31 * mat.m11 + result.m32 * mat.m21 + result.m33 * mat.m31 + result.m34 * mat.m41;
	m32 = result.m31 * mat.m12 + result.m32 * mat.m22 + result.m33 * mat.m32 + result.m34 * mat.m42;
	m33 = result.m31 * mat.m13 + result.m32 * mat.m23 + result.m33 * mat.m33 + result.m34 * mat.m43;
	m34 = result.m31 * mat.m14 + result.m32 * mat.m24 + result.m33 * mat.m34 + result.m34 * mat.m44;
																			   
	m41 = result.m41 * mat.m11 + result.m41 * mat.m21 + result.m43 * mat.m31 + result.m44 * mat.m41;
	m42 = result.m41 * mat.m12 + result.m42 * mat.m22 + result.m43 * mat.m32 + result.m44 * mat.m42;
	m43 = result.m41 * mat.m13 + result.m42 * mat.m23 + result.m43 * mat.m33 + result.m44 * mat.m43;
	m44 = result.m41 * mat.m14 + result.m42 * mat.m24 + result.m43 * mat.m34 + result.m44 * mat.m44;
	return result;
}

GtMatrix4 GtMatrix4::operator /=(const GtMatrix4& mat)
{
	GtMatrix4 result = *this;
	result = result * mat.MakeInverse();
	return result;
}

GtMatrix4 GtMatrix4::operator -() const
{
	GtMatrix4 result;
	result.m[0][0] = -m[0][0];
	result.m[0][1] = -m[0][1];
	result.m[0][2] = -m[0][2];
	result.m[0][3] = -m[0][3];
	result.m[1][0] = -m[1][0];
	result.m[1][1] = -m[1][1];
	result.m[1][2] = -m[1][2];
	result.m[1][3] = -m[1][3];
	result.m[2][0] = -m[2][0];
	result.m[2][1] = -m[2][1];
	result.m[2][2] = -m[2][2];
	result.m[2][3] = -m[2][3];
	result.m[3][0] = -m[3][0];
	result.m[3][1] = -m[3][1];
	result.m[3][2] = -m[3][2];
	result.m[3][3] = -m[3][3];
	return result;
}

bool GtMatrix4::operator ==(const GtMatrix4& mat) const
{
	return (m[0][0] == mat.m[0][0] && m[0][1] == mat.m[0][1] && m[0][2] == mat.m[0][2] && m[0][3] == mat.m[0][3] &&
			m[1][0] == mat.m[1][0] && m[1][1] == mat.m[1][1] && m[1][2] == mat.m[1][2] && m[1][3] == mat.m[1][3] &&
			m[2][0] == mat.m[2][0] && m[2][1] == mat.m[2][1] && m[2][2] == mat.m[2][2] && m[2][3] == mat.m[2][3] &&
			m[3][0] == mat.m[3][0] && m[3][1] == mat.m[3][1] && m[3][2] == mat.m[3][2] && m[3][3] == mat.m[3][3]);
}

bool GtMatrix4::operator !=(const GtMatrix4& mat) const
{
	return !(*this == mat);
}

GtMatrix4 &GtMatrix4::operator =(const GtMatrix4 &mat)
{
	m[0][0] = mat.m[0][0];
	m[0][1] = mat.m[0][1];
	m[0][2] = mat.m[0][2];
	m[0][3] = mat.m[0][3];
	m[1][0] = mat.m[1][0];
	m[1][1] = mat.m[1][1];
	m[1][2] = mat.m[1][2];
	m[1][3] = mat.m[1][3];
	m[2][0] = mat.m[2][0];
	m[2][1] = mat.m[2][1];
	m[2][2] = mat.m[2][2];
	m[2][3] = mat.m[2][3];
	m[3][0] = mat.m[3][0];
	m[3][1] = mat.m[3][1];
	m[3][2] = mat.m[3][2];
	m[3][3] = mat.m[3][3];
	return *this;
}