#include "GtNode.h"
#include "Macro.h"

GtNode::GtNode(void)
: m_pParent(NULL)
, m_bRunning(false)
{
#ifdef		MULTIBYTE
	m_strName = "Entity";
#else
	m_strName = L"Entity";
#endif
	memset(&m_vScale, 0, sizeof(GtPoint3f));
	memset(&m_vPosition, 0, sizeof(GtPoint3f));
	m_matTM.MakeIdentity();
	m_matRot.MakeIdentity();
	m_lsChild.clear();
}

GtNode::GtNode(const GtString& strName)
: m_pParent(NULL)
, m_bRunning(false)
{
	m_strName = strName;
	memset(&m_vScale, 0, sizeof(GtPoint3f));
	memset(&m_vPosition, 0, sizeof(GtPoint3f));
	m_matTM.MakeIdentity();
	m_matRot.MakeIdentity();
	m_lsChild.clear();
}

GtNode::~GtNode(void)
{
	m_pParent = NULL;
	std::list<GtNode*> ::iterator it;
	for(it = m_lsChild.begin(); it != m_lsChild.end();)
	{
		if((*it))
		{
			SAFE_DELETE((*it));
			it = m_lsChild.erase(it);
		}
		it++;
	}
}

const GtString	GtNode::GetName(void) const
{
	return m_strName;
}

GtNode*	GtNode::GetParent(void)
{
	return m_pParent;
}

void GtNode::SetPosition(const GtPoint3f &pos)
{
//	m_vPosition.x = pos.x;
//	m_vPosition.y = pos.y;
//	m_vPosition.z = pos.z;
	m_vPosition = pos;
}

void GtNode::SetPosition(float x, float y, float z)
{
	m_vPosition.x = x;
	m_vPosition.y = y;
	m_vPosition.z = z;
}

const GtPoint3f &GtNode::GetPosition(void) const
{
	return m_vPosition;
}

void GtNode::SetScale(const GtPoint3f &scale)
{
	m_vScale = scale;
}

void GtNode::SetScale(const float value)
{
	m_vScale.SetScale(value);
}

void GtNode::SetScale(float x, float y, float z)
{
	m_vScale.SetScale(x, y, z);
}

const GtPoint3f &GtNode::GetScale(void) const
{
	return m_vScale;
}

void GtNode::SetTranslate(float x, float y, float z, eGT_TransformSpace relativeTo)
{
	switch(relativeTo)
	{
	case eTS_LOCAL:
		m_matTM.MakeTranslation(x, y, z);
		m_vPosition = GtPoint3f(x, y, z);
		break;
	case eTS_PARENT:
		break;
	case eTS_WORLD:
		break;
	}
}

void GtNode::SetTranslate(const GtPoint3f &pos, GtNode::eGT_TransformSpace relativeTo)
{
	switch(relativeTo)
	{
	case eTS_LOCAL:
		m_matTM.MakeTranslation(pos.x, pos.y, pos.z);
		m_vPosition = pos;
		break;
	case eTS_PARENT:
		break;
	case eTS_WORLD:
		break;
	}
}

void GtNode::RotationRoll(const float angle, GtNode::eGT_TransformSpace relativeTo)
{
	switch(relativeTo)
	{
	case eTS_LOCAL:
		m_matRot.MakeZRotation(angle);
		m_matTM = m_matTM * m_matRot;
		break;
	case eTS_PARENT:
		break;
	case eTS_WORLD:
		break;
	}
}

void GtNode::RotationPitch(const float angle, GtNode::eGT_TransformSpace relativeTo)
{
	switch(relativeTo)
	{
	case eTS_LOCAL:
		m_matRot.MakeXRotation(angle);
		m_matTM = m_matTM * m_matRot;
		break;
	case eTS_PARENT:
		break;
	case eTS_WORLD:
		break;
	}
}

void GtNode::RotationYaw(const float angle, GtNode::eGT_TransformSpace relativeTo)
{
	switch(relativeTo)
	{
	case eTS_LOCAL:
		m_matRot.MakeYRotation(angle);
		m_matTM = m_matTM * m_matRot;
		break;
	case eTS_PARENT:
		break;
	case eTS_WORLD:
		break;
	}
}

void GtNode::RotationYawPitchRoll(const GtPoint3f& pos, eGT_TransformSpace relativeTo)
{
	GtMatrix4 mtX;
	mtX.MakeIdentity();
	mtX.MakeXRotation(pos.x);
	GtMatrix4 mtY;
	mtY.MakeIdentity();
	mtY.MakeYRotation(pos.y);
	GtMatrix4 mtZ;
	mtZ.MakeIdentity();
	mtZ.MakeZRotation(pos.z);
	m_matRot = mtX * mtY * mtZ;
	m_matTM = m_matTM * m_matRot;
}

void GtNode::RotationYawPitchRoll(const float x, const float y, const float z, eGT_TransformSpace relativeTo)
{
	GtMatrix4 mtX;
	mtX.MakeIdentity();
	mtX.MakeXRotation(x);
	GtMatrix4 mtY;
	mtY.MakeIdentity();
	mtY.MakeYRotation(y);
	GtMatrix4 mtZ;
	mtZ.MakeIdentity();
	mtZ.MakeZRotation(z);
	m_matRot = mtX * mtY * mtZ;
	m_matTM = m_matTM * m_matRot;
}

void GtNode::SetRotate(const GtPoint3f& axis, const float angle, GtNode::eGT_TransformSpace relativeTo)
{
	switch(relativeTo)
	{
	case eTS_LOCAL:
		m_matRot.MakeRotation(angle, axis.x, axis.y, axis.z);
		m_matTM = m_matTM * m_matRot;
		break;
	case eTS_PARENT:
		break;
	case eTS_WORLD:
		break;
	}
}

GtMatrix4 &GtNode::GetMatrix(GtNode::eGT_TransformSpace relativeTo)
{
	switch(relativeTo)
	{
	case eTS_LOCAL:
		return m_matTM;
		break;
	case eTS_PARENT:
		break;
	case eTS_WORLD:
		break;
	}
	return m_matTM;
}

void GtNode::SetParent(GtNode* pParent)
{
	m_pParent = pParent;
}

void GtNode::AddChild(GtNode* pChild)
{
	m_lsChild.push_back(pChild);
	pChild->SetParent(this);
}

unsigned int GtNode::GetChildNum() const
{
	return m_lsChild.size();
}

GtNode* GtNode::GetChild(unsigned int uiIdx)
{
	size_t n = 0;
	if(n > GetChildNum())
		return NULL;
	std::list<GtNode*> ::iterator it;
	for(it = m_lsChild.begin(); it != m_lsChild.end();)
	{
		if(uiIdx == n)
			return (*it);
		n++;
	}
	return NULL;
}

GtNode* GtNode::GetChild(const GtString name)
{
	int n = 0;
	std::list<GtNode*> ::iterator it;
	for(it = m_lsChild.begin(); it != m_lsChild.end();)
	{
		if((*it)->m_strName == name)
			return (*it);
	}
	return NULL;
}

bool GtNode::RemoveChild(GtNode* pChild)
{
	int n = 0;
	std::list<GtNode*> ::iterator it;
	for(it = m_lsChild.begin(); it != m_lsChild.end();)
	{
		if( (*it) == pChild)
		{
			SAFE_DELETE((*it));
			m_lsChild.erase(it);
			return true;
		}
	}
	return false;
}

bool GtNode::RemoveChild(const GtString &name)
{
	int n = 0;
	std::list<GtNode*> ::iterator it;
	for(it = m_lsChild.begin(); it != m_lsChild.end();)
	{
		if( (*it)->GetName() == name)
		{
			SAFE_DELETE((*it));
			m_lsChild.erase(it);
			return true;
		}
	}
	return false;
}

bool GtNode::RemoveChild(unsigned int uiIdx)
{
	size_t n = 0;
	if(n > GetChildNum())
		return false;
	std::list<GtNode*> ::iterator it;
	for(it = m_lsChild.begin(); it != m_lsChild.end();)
	{
		if(uiIdx == n)
		{
			SAFE_DELETE((*it));
			m_lsChild.erase(it);
			return true;
		}
		n++;
	}
	return false;
}

void GtNode::Render(void)
{
	std::list<GtNode*> ::iterator it;
	for(it = m_lsChild.begin(); it != m_lsChild.end();)
	{
		if((*it))
			(*it)->Update();
	}
}

void GtNode::Update(void)
{
	std::list<GtNode*> ::iterator it;
	for(it = m_lsChild.begin(); it != m_lsChild.end();)
	{
		if((*it))
			(*it)->Update();
	}
}

void GtNode::NeedUpdate(void)
{
	std::list<GtNode*> ::iterator it;
	for(it = m_lsChild.begin(); it != m_lsChild.end();)
	{
		if((*it))
			(*it)->NeedUpdate();
	}
}