#pragma once

#include "GtPoint2.h"
#include "GtPoint3.h"
#include "GtPoint4.h"
#include "GtQuaternion.h"
#include "GtMatrix4.h"

#include <string>
#include <list>

#define MULTIBYTE 0

// Ƽ Ʈ ϶ string
//  ܿ ڵ
#ifdef		MULTIBYTE
	typedef		std::string		GtString;
#else
	typedef		std::wstring	GtString;
#endif

class GtNode
{
public:
	enum eGT_TransformSpace
	{
		eTS_LOCAL,
		eTS_PARENT,
		eTS_WORLD,

		eTS_END
	};
protected:
	GtString				m_strName;					// Node ̸

	GtPoint3f				m_vScale;					// Node ũ
	GtPoint3f				m_vPosition;				// Node ġ

	GtMatrix4				m_matTM;					// Node  
	GtMatrix4				m_matRot;					// Node  ȸ 

	GtNode					*m_pParent;					// Node θ  
	std::list<GtNode*>		m_lsChild;					// Child Node list ( ⶧)

	bool					m_bRunning;					// Node Update 
public:
	GtNode(void);
	GtNode(const GtString& strName);
	virtual	~GtNode(void);

	const	GtString			GetName(void) const;
	virtual GtNode*				GetParent(void);

	// ġ Լ
	virtual			void		SetPosition(const GtPoint3f& pos);
	virtual			void		SetPosition(float x, float y, float z);
	virtual const	GtPoint3f &	GetPosition(void) const;
	virtual			void		SetTranslate(float x, float y, float z, eGT_TransformSpace relativeTo = eTS_PARENT);
	virtual			void		SetTranslate(const GtPoint3f& pos, eGT_TransformSpace relativeTo = eTS_PARENT);


	// Scale Լ
	virtual			void		SetScale(const GtPoint3f& scale);
	virtual			void		SetScale(const float value);
	virtual			void		SetScale(float x, float y, float z);
	virtual	const	GtPoint3f &	GetScale(void) const;

	// ȸ Լ
	virtual			void		RotationRoll(const float angle, eGT_TransformSpace relativeTo = eTS_LOCAL);
	virtual			void		RotationPitch(const float angle, eGT_TransformSpace relativeTo = eTS_LOCAL);
	virtual			void		RotationYaw(const float angle, eGT_TransformSpace relativeTo = eTS_LOCAL);
	virtual			void		RotationYawPitchRoll(const GtPoint3f& pos, eGT_TransformSpace relativeTo = eTS_LOCAL);
	virtual			void		RotationYawPitchRoll(const float x, const float y, const float z, eGT_TransformSpace relativeTo = eTS_LOCAL);
	virtual			void		SetRotate(const GtPoint3f& axis, const float angle, eGT_TransformSpace relativeTo = eTS_LOCAL);

	//   
	virtual			GtMatrix4 &	GetMatrix(eGT_TransformSpace relativeTo = eTS_LOCAL);

	// Node Parent , Child Ʈ
	void						SetParent(GtNode*	pParent);
	virtual			void		AddChild(GtNode*	pChild);
	virtual unsigned int		GetChildNum(void) const;
	virtual			GtNode*		GetChild(unsigned int uiIdx);
	virtual			GtNode*		GetChild(const GtString name);

	virtual			bool		RemoveChild(GtNode* pChild);
	virtual			bool		RemoveChild(const GtString& name);
	virtual			bool		RemoveChild(unsigned int uiIdx);

	virtual			void		Update(void);
	virtual			void		Render(void);
	virtual			void		NeedUpdate(void);
};