#include "GtPoint2.h"
//------------------------------------------------------------------------------------
template<typename T>
inline GtPoint2<T>::GtPoint2(T tX, T tY)
{
	x = tX;
	y = tY;
}
//------------------------------------------------------------------------------------

//------------------------------------------------------------------------------------
template<typename T>
inline bool GtPoint2<T>::operator == (const GtPoint2<T>& pt) const
{
	return (x == pt.x && y == pt.y);
}
//------------------------------------------------------------------------------------

//------------------------------------------------------------------------------------
template<typename T>
inline bool GtPoint2<T>::operator != (const GtPoint2<T>& pt) const
{
	return !operator==(pt);
}
//------------------------------------------------------------------------------------

//------------------------------------------------------------------------------------
template<typename T>
inline GtPoint2<T> GtPoint2<T>::operator +(const GtPoint2<T> &pt) const
{
	GtPoint2<T> result = *this;
	result.x += pt.x;
	result.y += pt.y;

	return result;
}
//------------------------------------------------------------------------------------

//------------------------------------------------------------------------------------
template<typename T>
inline GtPoint2<T> GtPoint2<T>::operator -(const GtPoint2<T> &pt) const
{
	GtPoint2<T> result = *this;
	result.x -= pt.x;
	result.y -= pt.y;

	return result;
}
//------------------------------------------------------------------------------------

//------------------------------------------------------------------------------------
template<typename T>
inline T GtPoint2<T>::operator *(const GtPoint2<T> &pt) const
{
	return (x*pt.x) + (y * pt.y);
}
//------------------------------------------------------------------------------------

//------------------------------------------------------------------------------------
template<typename T>
inline GtPoint2<T> GtPoint2<T>::operator *(T tScalar) const
{
	GtPoint2<T> result = *this;
	result.x *= tScalar;
	result.y *= tScalar;
	return result;
}
//------------------------------------------------------------------------------------

//------------------------------------------------------------------------------------
template<typename T>
inline GtPoint2<T> GtPoint2<T>::operator /(T tScalar) const
{
	GtPoint2<T> result = *this;
	result.x /= tScalar;
	result.y /= tScalar;

	return result;
}
//------------------------------------------------------------------------------------

//------------------------------------------------------------------------------------
template<typename T>
inline GtPoint2<T> GtPoint2<T>::operator -() const
{
	return GtPoint2<T>(-x, -y);
}
//------------------------------------------------------------------------------------

//------------------------------------------------------------------------------------
template<typename T>
inline GtPoint2<T> GtPoint2<T>:: operator += (const GtPoint2<T> &pt)
{
	x += pt.x;
	y += pt.y;

	return *this;
}
//------------------------------------------------------------------------------------

//------------------------------------------------------------------------------------
template<typename T>
inline GtPoint2<T> GtPoint2<T>:: operator -= (const GtPoint2<T> &pt)
{
	x -= pt.x;
	y -= pt.y;

	return *this;
}
//------------------------------------------------------------------------------------

//------------------------------------------------------------------------------------
template<typename T>
inline GtPoint2<T> GtPoint2<T>:: operator *= (T tScalar)
{
	x *= tScalar;
	y *= tScalar;

	return *this;
}
//------------------------------------------------------------------------------------

//------------------------------------------------------------------------------------
template<typename T>
inline GtPoint2<T> GtPoint2<T>:: operator /= (T tScalar)
{
	x /= tScalar;
	y /= tScalar;

	return *this;
}
//------------------------------------------------------------------------------------

//------------------------------------------------------------------------------------
template<typename T>
inline GtPoint2<T> GtPoint2<T>::ComponentProduct(const GtPoint2<T>& p0, const GtPoint2<T>& p1)
{
	return GtPoint2<T>(p0.x * p1.x, p0.y * p1.y);
}
//------------------------------------------------------------------------------------

//------------------------------------------------------------------------------------
template<typename T>
inline float GtPoint2<T>::Length() const
{
	return GtSqrt((float)(x*x + y*y));
}
//------------------------------------------------------------------------------------

//------------------------------------------------------------------------------------
template<typename T>
inline float GtPoint2<T>::SqrLength() const
{
	return (float)(x*x + y*y);
}
//------------------------------------------------------------------------------------

//------------------------------------------------------------------------------------
template<typename T>
inline float GtPoint2<T>::Dot(const GtPoint2<T> &pt) const
{
	return (float)(x * pt.x + y * pt.y);
}
//------------------------------------------------------------------------------------

//------------------------------------------------------------------------------------
template<typename T>
inline float GtPoint2<T>::Unitize()
{
	float fLength = Length();

	if( fLength > 1e-06f )
	{
		float fRecip = 1.0f / fLength;
		x *= (T)fRecip;
		y *= (T)fRecip;
	}
	else
	{
		x = 0;
		y = 0;
		fLength = 0;
	}

	return fLength;
}
//------------------------------------------------------------------------------------