#include "GtQuadTree.h"
#include "GtD3DRenderer.h"

GtQuadTree::GtQuadTree(int iX, int iY)
{
	//  ʱȭ
	m_fRadius = 0.0f;
	m_spSphere.m_fRadius = 0.0f;
	m_spSphere.v = GtPoint3f(0.0f, 0.0f, 0.0f);
	m_bCulled = false;
	for(int iLoop = 0; iLoop < 4; ++iLoop)
		m_pChild[iLoop] = m_pNeighbor[iLoop] = NULL;

	//ڳ ε 
	m_nCorner[eCN_TL]	= 0;				//  ε
	m_nCorner[eCN_TR]	= iX - 1;			//  ε
	m_nCorner[eCN_BL]	= iX * (iY - 1);	// ϴ ε
	m_nCorner[eCN_BR]	= iX * iY - 1;		// ϴ ε

	//豸 ߽ ε ()
	m_nCenter = CENTER_INDEX(m_nCorner[eCN_TL], m_nCorner[eCN_TR], m_nCorner[eCN_BL], m_nCorner[eCN_BR]);
}

GtQuadTree::GtQuadTree(void)
{
	m_nCenter  = 0;
	m_fRadius = 0.0f;
	m_spSphere.m_fRadius = 0.0f;
	m_spSphere.v = GtPoint3f(0.0f, 0.0f, 0.0f);
	m_bCulled = false;
	for(int iLoop = 0; iLoop < 4; ++iLoop)
	{
		m_pChild[iLoop] = m_pNeighbor[iLoop] = NULL;
		m_nCorner[iLoop] = 0;
	}
}

GtQuadTree::~GtQuadTree(void)
{
	for(int i=0; i<4; i++)
	{
		if(m_pChild[i])
		{
			delete m_pChild[i];
			m_pChild[i] = NULL;
		}
	}
}

GtQuadTree* GtQuadTree::AddChild(int iCornerTL, int iCornerTR, int iCornerBL, int iCornerBR)
{
	GtQuadTree *pChild = new GtQuadTree;
	pChild->SetCorners(iCornerTL, iCornerTR, iCornerBL, iCornerBR);
	return pChild;
}

void GtQuadTree::SetCorners(int iCornerTL, int iCornerTR, int iCornerBL, int iCornerBR)
{
	//ڳε
	m_nCorner[eCN_TL] = iCornerTL;
	m_nCorner[eCN_TR] = iCornerTR;
	m_nCorner[eCN_BL] = iCornerBL;
	m_nCorner[eCN_BR] = iCornerBR;

	//豸 ߽ ε ()
	m_spSphere.m_fRadius = 0.0f;
	m_nCenter = CENTER_INDEX(m_nCorner[eCN_TL], m_nCorner[eCN_TR], m_nCorner[eCN_BL], m_nCorner[eCN_BR]);
}

// Ʈ 
void GtQuadTree::Build(LPGT_CUSTOMVERTEX pHeightMap)
{
	BuildQuadTree(pHeightMap);
	BuildNeighborNode(this, pHeightMap, m_nCorner[eCN_TR] + 1);
}

// ڽ  
void GtQuadTree::BuildQuadTree(LPGT_CUSTOMVERTEX pHeightMap)
{
	if(SubDivide())
	{
		D3DXVECTOR3 vDiameter = pHeightMap[ m_nCorner[eCN_TL] ].m_vPos - pHeightMap[ m_nCorner[eCN_BR] ].m_vPos;
		m_fRadius = D3DXVec3Length(&vDiameter) / 2.0f;
		m_spSphere.m_fRadius = m_fRadius;
		m_spSphere.v = GtPoint3f(vDiameter.x, vDiameter.y, vDiameter.z);
		m_pChild[eCN_TL]->BuildQuadTree( pHeightMap );
		m_pChild[eCN_TR]->BuildQuadTree( pHeightMap );
		m_pChild[eCN_BL]->BuildQuadTree( pHeightMap );
		m_pChild[eCN_BR]->BuildQuadTree( pHeightMap );
	}
}

void GtQuadTree::BuildNeighborNode(GtQuadTree* pRoot, LPGT_CUSTOMVERTEX pHeightMap, int iIndexRow)
{
	int iCorner = 0;
	int iNCorner[4] = { 0, };
	for(int iLoop = 0; iLoop < 4; ++iLoop)
	{
		iNCorner[0] = m_nCorner[0];
		iNCorner[1] = m_nCorner[1];
		iNCorner[2] = m_nCorner[2];
		iNCorner[3] = m_nCorner[3];
		iCorner = GetNeighborIndex(iLoop, iIndexRow, iNCorner[0], iNCorner[1], iNCorner[2], iNCorner[3]);

		if( -1 < iCorner )
			m_pNeighbor[iLoop] = pRoot->FindNode(pHeightMap, iNCorner[0], iNCorner[1], iNCorner[2], iNCorner[3]);
	}
	if( m_pChild[0] )
	{
		m_pChild[0]->BuildNeighborNode( pRoot, pHeightMap, iIndexRow );
		m_pChild[1]->BuildNeighborNode( pRoot, pHeightMap, iIndexRow );
		m_pChild[2]->BuildNeighborNode( pRoot, pHeightMap, iIndexRow );
		m_pChild[3]->BuildNeighborNode( pRoot, pHeightMap, iIndexRow );
	}
}

//  
bool GtQuadTree::SubDivide(void)
{
	// ߽
	int nTopEdgeCenter		= ( m_nCorner[eCN_TL] + m_nCorner[eCN_TR] ) / 2;
	//ϴ ߽
	int nBottomEdgeCenter	= ( m_nCorner[eCN_BL] + m_nCorner[eCN_BR] ) / 2;
	// ߽
	int nLeftEdgeCenter		= ( m_nCorner[eCN_TL] + m_nCorner[eCN_BL] ) / 2;
	// ߽
	int nRightEdgeCenter	= ( m_nCorner[eCN_TR] + m_nCorner[eCN_BR] ) / 2;
	//߽
	int nCentralPoint		= CENTER_INDEX(m_nCorner[eCN_TL], m_nCorner[eCN_TR], m_nCorner[eCN_BL], m_nCorner[eCN_BR]);

	//Ҽִ Ȯ
	if( 1 < (m_nCorner[eCN_TR] - m_nCorner[eCN_TL]) )
	{
		m_pChild[eCN_TL] = AddChild( m_nCorner[eCN_TL], nTopEdgeCenter, nLeftEdgeCenter, nCentralPoint );
		m_pChild[eCN_TR] = AddChild( nTopEdgeCenter, m_nCorner[eCN_TR], nCentralPoint, nRightEdgeCenter );
		m_pChild[eCN_BL] = AddChild( nLeftEdgeCenter, nCentralPoint, m_nCorner[eCN_BL], nBottomEdgeCenter );
		m_pChild[eCN_BR] = AddChild( nCentralPoint, nRightEdgeCenter, nBottomEdgeCenter, m_nCorner[eCN_BR] );

		return true;
	}
	return false;
}

// ̿ ε 
int GtQuadTree::GetNeighborIndex(int iCourse, int iIndexRow, int& iCorner_0, int& iCorner_1, int& iCorner_2, int& iCorner_3)
{
	//  /  ϱ
	int iGap = iCorner_1 - iCorner_0;

	int iNCorner[4];
	iNCorner[0]	= iCorner_0;		iNCorner[1]	= iCorner_1;
	iNCorner[2]	= iCorner_2;		iNCorner[3]	= iCorner_3;

	switch(iCourse)
	{
		case eCS_UP://  ̿ ε
			iCorner_0 = iNCorner[0] - iIndexRow * iGap;
			iCorner_1 = iNCorner[1] - iIndexRow * iGap;
			iCorner_2 = iNCorner[0];
			iCorner_3 = iNCorner[1];
			break;
		case eCS_DN://Ʒ  ̿ ε
			iCorner_0 = iNCorner[2];
			iCorner_1 = iNCorner[3];
			iCorner_2 = iNCorner[2] + iIndexRow * iGap;
			iCorner_3 = iNCorner[3] + iIndexRow * iGap;
			break;
		case eCS_LT:	//  ̿ ε
			iCorner_0 = iNCorner[0] - iGap;
			iCorner_1 = iNCorner[0];
			iCorner_2 = iNCorner[2] - iGap;
			iCorner_3 = iNCorner[2];
			break;
		case eCS_RT:	//  ̿ ε
			iCorner_0 = iNCorner[1];
			iCorner_1 = iNCorner[1] + iGap;
			iCorner_2 = iNCorner[3];
			iCorner_3 = iNCorner[3] + iGap;
			break;
	}

	//߽ε ϱ
	int iCenter = CENTER_INDEX(iCorner_0, iCorner_1, iCorner_2, iCorner_3);
	if( (0 > iCenter) || (iCenter > (iIndexRow * iIndexRow - 1)) ) return -1;
	return iCenter;
}

//ڳʰ ġϴ ã
GtQuadTree* GtQuadTree::FindNode(LPGT_CUSTOMVERTEX pHeightMap, int iCorner_0, int iCorner_1, int iCorner_2, int iCorner_3)
{
	//ġϴ Ȯ
	if( (m_nCorner[0] == iCorner_0) && (m_nCorner[1] == iCorner_1) && (m_nCorner[2] == iCorner_2) && (m_nCorner[3] == iCorner_3) )
	{	return this;	}

	if( m_pChild[0] )
	{
		int iCenter = CENTER_INDEX(iCorner_0, iCorner_1, iCorner_2, iCorner_3);

		for(int iLoop=0; iLoop<4; ++iLoop)
		{
			if(	pHeightMap[iCenter].m_vPos.x >= pHeightMap[ m_pChild[iLoop]->m_nCorner[eCN_TL] ].m_vPos.x && 
				pHeightMap[iCenter].m_vPos.x <= pHeightMap[ m_pChild[iLoop]->m_nCorner[eCN_BR] ].m_vPos.x &&
				pHeightMap[iCenter].m_vPos.z >= pHeightMap[ m_pChild[iLoop]->m_nCorner[eCN_BR] ].m_vPos.z && 
				pHeightMap[iCenter].m_vPos.z <= pHeightMap[ m_pChild[iLoop]->m_nCorner[eCN_TL] ].m_vPos.z)
			{	return m_pChild[iLoop]->FindNode( pHeightMap, iCorner_0, iCorner_1, iCorner_2, iCorner_3 );		}
		}
	}
	return NULL;
}

// LOD ó |   Ǵ
bool GtQuadTree::IsVisible(LPGT_CUSTOMVERTEX pHeightMap, GtPoint3f *pCamera, double fLODRatio)
{
	return (m_nCorner[eCN_TR] - m_nCorner[eCN_TL]) <= GetLODLevel(pHeightMap, pCamera, fLODRatio);
}

// LOD  Ȯ
int GtQuadTree::GetLODLevel(LPGT_CUSTOMVERTEX pHeightMap, GtPoint3f *pCamera, double fLODRatio)
{
	GtPoint3f vPos(0.0f, 0.0f, 0.0f);
	ConvertGtPoint3fFromD3DVector(&vPos, &pHeightMap[m_nCenter].m_vPos);
	int iDest = int(GetDistance(&vPos, pCamera) *fLODRatio);
	return max(iDest, 1);
}

// 2  Ÿ üũ
float GtQuadTree::GetDistance(GtPoint3f *pV1, GtPoint3f *pV2)
{
	GtPoint3f vLength = *pV2 - *pV1;
	return vLength.Length();
}

// ü ø
void GtQuadTree::FrustumCull(LPGT_CUSTOMVERTEX pHeightMap, GtFrustum *pFrustum)
{
	switch(IsInFrustum(pHeightMap, pFrustum))
	{
	case eFS_COMPLETELY_IN:
		m_bCulled = false;		//  
		return;
	case eFS_PARTIALLY_IN:		// Ϻ
		m_bCulled = false;
		break;
	case eFS_OUT:				//  ƿ
		m_bCulled = true;
		return;
	}
	if(m_pChild[0]) m_pChild[0]->FrustumCull(pHeightMap, pFrustum);
	if(m_pChild[1]) m_pChild[1]->FrustumCull(pHeightMap, pFrustum);
	if(m_pChild[2]) m_pChild[2]->FrustumCull(pHeightMap, pFrustum);
	if(m_pChild[3]) m_pChild[3]->FrustumCull(pHeightMap, pFrustum);
}

//ü ø üũ
int GtQuadTree::IsInFrustum(LPGT_CUSTOMVERTEX pHeightMap, GtFrustum *pFrustum)
{
	GtPoint3f vPosCenter;
	ConvertGtPoint3fFromD3DVector(&vPosCenter, &pHeightMap[ m_nCenter ].m_vPos);
	m_spSphere.v = vPosCenter;
	if(pFrustum->StateSphereFrustum(m_spSphere) == 0)
		return eFS_OUT;

	//豸ü ԵǾٸ 簢   øüũ
	GtPoint3f vPos[4];
	ConvertGtPoint3fFromD3DVector(&vPos[0], &pHeightMap[ m_nCorner[ eCN_TL ] ].m_vPos);
	ConvertGtPoint3fFromD3DVector(&vPos[1], &pHeightMap[ m_nCorner[ eCN_TR ] ].m_vPos);
	ConvertGtPoint3fFromD3DVector(&vPos[2], &pHeightMap[ m_nCorner[ eCN_BL ] ].m_vPos);
	ConvertGtPoint3fFromD3DVector(&vPos[3], &pHeightMap[ m_nCorner[ eCN_BR ] ].m_vPos);
	int b_0 = pFrustum->IsInPoint( &vPos[0] );
	int b_1 = pFrustum->IsInPoint( &vPos[1] );
	int b_2 = pFrustum->IsInPoint( &vPos[2] );
	int b_3 = pFrustum->IsInPoint( &vPos[3] );

	// 4 ü ȿ 
	if( 4 == (b_0 + b_1 + b_2 + b_3) ) return eFS_COMPLETELY_IN;

	// Ϻκ ҿ ִ 
	return eFS_PARTIALLY_IN;
}

//¿ ε
int	GtQuadTree::GenerateIndex(LPVOID pIndex, LPGT_CUSTOMVERTEX pHeightMap, GtFrustum *pFrustum, double fLODRatio )
{
	//и麰 üø
	FrustumCull(pHeightMap, pFrustum);

	//εƮ
	return GenTriIndex(0, pIndex, pHeightMap, pFrustum, fLODRatio);
}

//¿ ε ۼ
int GtQuadTree::GenTriIndex(int nTris, LPVOID pIndex, LPGT_CUSTOMVERTEX pHeightMap, GtFrustum* pFrustum, double fLODRatio)
{
	//ø Ȯ
	if( m_bCulled )
	{
		m_bCulled = false;
		return nTris;
	}
	if( IsVisible(pHeightMap, &pFrustum->GetPosition(), fLODRatio) )
	{
		LPDWORD pIdx = ((LPDWORD)pIndex) + nTris * 3;

		bool bDraw[4] = {true, true, true, true};
		int iNewCenter;

		//̿ Ȯ(°ɿ)
		if( m_pNeighbor[eCS_UP] ) bDraw[eCS_UP] = m_pNeighbor[eCS_UP]->IsVisible( pHeightMap, &pFrustum->GetPosition(), fLODRatio );
		if( m_pNeighbor[eCS_DN] ) bDraw[eCS_DN] = m_pNeighbor[eCS_DN]->IsVisible( pHeightMap, &pFrustum->GetPosition(), fLODRatio );
		if( m_pNeighbor[eCS_LT] ) bDraw[eCS_LT] = m_pNeighbor[eCS_LT]->IsVisible( pHeightMap, &pFrustum->GetPosition(), fLODRatio );
		if( m_pNeighbor[eCS_RT] ) bDraw[eCS_RT] = m_pNeighbor[eCS_RT]->IsVisible( pHeightMap, &pFrustum->GetPosition(), fLODRatio );

		//  Ѱ  
		if( bDraw[eCS_UP] && bDraw[eCS_DN] && bDraw[eCS_LT] && bDraw[eCS_RT] )
		{
			*pIdx++ = m_nCorner[0];		*pIdx++ = m_nCorner[1];		*pIdx++ = m_nCorner[2];
			nTris++;
			*pIdx++ = m_nCorner[2];		*pIdx++ = m_nCorner[1];		*pIdx++ = m_nCorner[3];
			nTris++;

			return nTris;
		}

		//ܺȮ
		if( !bDraw[eCS_UP] )
		{
			iNewCenter = ( m_nCorner[eCN_TL] + m_nCorner[eCN_TR] ) / 2;
			*pIdx++ = m_nCenter;	*pIdx++ = m_nCorner[eCN_TL];		*pIdx++ = iNewCenter;
			nTris++;
			*pIdx++ = m_nCenter;	*pIdx++ = iNewCenter;				*pIdx++ = m_nCorner[eCN_TR];
			nTris++;
		}
		else
		{ 
			*pIdx++ = m_nCenter;	*pIdx++ = m_nCorner[eCN_TL];		*pIdx++ = m_nCorner[eCN_TR];
			nTris++;
		}

		//ϴܺȮ
		if( !bDraw[eCS_DN] )
		{
			iNewCenter = ( m_nCorner[eCN_BL] + m_nCorner[eCN_BR] ) / 2;
			*pIdx++ = m_nCenter;	*pIdx++ = m_nCorner[eCN_BR];		*pIdx++ = iNewCenter; 
			nTris++;
			*pIdx++ = m_nCenter;	*pIdx++ = iNewCenter;				*pIdx++ = m_nCorner[eCN_BL]; 
			nTris++;
		}
		else
		{
			*pIdx++ = m_nCenter;	*pIdx++ = m_nCorner[eCN_BR];		*pIdx++ = m_nCorner[eCN_BL]; 
			nTris++;
		}

		//Ȯ
		if( !bDraw[eCS_LT] )
		{
			iNewCenter = ( m_nCorner[eCN_TL] + m_nCorner[eCN_BL] ) / 2;
			*pIdx++ = m_nCenter;	*pIdx++ = m_nCorner[eCN_BL];		*pIdx++ = iNewCenter; 
			nTris++;
			*pIdx++ = m_nCenter;	*pIdx++ = iNewCenter;				*pIdx++ = m_nCorner[eCN_TL]; 
			nTris++;
		}
		else
		{
			*pIdx++ = m_nCenter;	*pIdx++ = m_nCorner[eCN_BL];		*pIdx++ = m_nCorner[eCN_TL];
			nTris++;
		}

		//Ȯ
		if( !bDraw[eCS_RT] )
		{
			iNewCenter = ( m_nCorner[eCN_TR] + m_nCorner[eCN_BR] ) / 2;
			*pIdx++ = m_nCenter;	*pIdx++ = m_nCorner[eCN_TR];		*pIdx++ = iNewCenter;
			nTris++;
			*pIdx++ = m_nCenter;	*pIdx++ = iNewCenter;				*pIdx++ = m_nCorner[eCN_BR];
			nTris++;
		}
		else
		{ 
			*pIdx++ = m_nCenter;	*pIdx++ = m_nCorner[eCN_TR];		*pIdx++ = m_nCorner[eCN_BR]; 
			nTris++; 
		}

		//  Ȯ(տʿ)
		if(m_nCorner[eCN_TR] - m_nCorner[eCN_TL] <= 1)
		{
			*pIdx++ = m_nCorner[0];			*pIdx++ = m_nCorner[1];		*pIdx++ = m_nCorner[2];
			nTris++;
			*pIdx++ = m_nCorner[2];			*pIdx++ = m_nCorner[1];		*pIdx++ = m_nCorner[3];
			nTris++;

			return nTris;
		}
	}

	//ڽĳ ˻
	if( m_pChild[eCN_TL] ) nTris = m_pChild[eCN_TL]->GenTriIndex( nTris, pIndex, pHeightMap, pFrustum, fLODRatio );
	if( m_pChild[eCN_TR] ) nTris = m_pChild[eCN_TR]->GenTriIndex( nTris, pIndex, pHeightMap, pFrustum, fLODRatio );
	if( m_pChild[eCN_BL] ) nTris = m_pChild[eCN_BL]->GenTriIndex( nTris, pIndex, pHeightMap, pFrustum, fLODRatio );
	if( m_pChild[eCN_BR] ) nTris = m_pChild[eCN_BR]->GenTriIndex( nTris, pIndex, pHeightMap, pFrustum, fLODRatio );

	return nTris;
}
