#include "GtSoftwareRenderDevice.h"
#include<iostream>


GtSoftwareRenderDevice::GtSoftwareRenderDevice(void) : 
m_hWnd(NULL), 
m_hMainBuffer(NULL),
m_hBackBuffer(NULL),
m_hBitmap(NULL),
m_hDefaultBitmap(NULL),
m_pBit(NULL),
m_iWidth(0),
m_iHeight(0),
m_nPitch(0),
m_pZBuffer16(NULL),
m_pZBuffer32(NULL)
{
	for(int i=0; i<TEX_STAGE_NUM; i++)
		m_pTexture[i] = NULL;
}

GtSoftwareRenderDevice::~GtSoftwareRenderDevice(void)
{
	Destroy();
	SAFE_DELETE(m_pZBuffer16);
	SAFE_DELETE(m_pZBuffer32);
}

void GtSoftwareRenderDevice::Destroy(void)
{
	SelectObject(m_hBackBuffer, m_hDefaultBitmap);
	DeleteDC(m_hBackBuffer);
	DeleteObject(m_hDefaultBitmap);
	DeleteObject(m_hBitmap);
	ReleaseDC(m_hWnd, m_hMainBuffer);
}

void GtSoftwareRenderDevice::Create(HWND hWnd, int iWidth, int iHeight)
{
	if(!hWnd)
		return ;

	m_hWnd				= hWnd;
	m_hMainBuffer		= GetDC(hWnd);
	m_hBackBuffer		= CreateCompatibleDC(m_hMainBuffer);
	m_hBitmap			= CreateBitmap(m_hMainBuffer, m_iWidth = iWidth, m_iHeight = iHeight, &m_pBit);
	
	m_hDefaultBitmap	= (HBITMAP)SelectObject(m_hBackBuffer, m_hBitmap);

	// Ʈ ⺻ //////////////////////////////////////////////////////////////////
	GtViewPort ViewPort;
	ViewPort.Height		= m_iHeight;
	ViewPort.Width		= m_iWidth;
	ViewPort.MinZ		= 0;
	ViewPort.MaxZ		= 1;
	ViewPort.x			= 0;
	ViewPort.y			= 0;
	SetViewPort(ViewPort);
	////////////////////////////////////////////////////////////////////////////////////

	// Ŭ ⺻ /////////////////////////////////////////////////////////////////
	GtClipping Clip;
	Clip.ClipHeight		= 2;
	Clip.ClipWidth		= 2;
	Clip.ClipX			= -1;
	Clip.ClipY			= 1;
	Clip.MinZ			= 0;
	Clip.MaxZ			= 1;
	CreateMatrixClipping(Clip);
	/////////////////////////////////////////////////////////////////////////////////////

	// ؽ ////////////////////////////////////////////////////////////////////
	memset(m_pTexture, 8, sizeof(LPGT_TEXTURE));	// 8 
	//////////////////////////////////////////////////////////////////////////////////

	//⺻ GtLight   ////////////////////////////////////////////////////////////////////
	GtLight t, t2;
	t.m_Ambient  = GtColorValue(1.0f, 1.0f, 1.0f);
	t.m_Diffuse  = GtColorValue(1.0f, 1.0f, 1.0f);
	t.m_Specular = GtColorValue(0.0f, 0.0f, 0.0f);
	t.m_Direction = GtPoint3f(0.0f, 1.0f, 0.0f);
	SetLight(0, t);
	/////////////////////////////////////////////////////////////////////////////////////////

	//⺻ GtMaterial  ////////////////////////////////////////////////////////////////////
	m_Material.m_Ambient  = GtColorValue(1.0f, 1.0f, 1.0f);
	m_Material.m_Diffuse  = GtColorValue(1.0f, 1.0f, 1.0f);
	m_Material.m_Specular = GtColorValue(1.0f, 1.0f, 1.0f);
	m_Material.m_Emissive = GtColorValue(0.0f, 0.0f, 0.0f, 1.0f);
	m_Material.m_fPower = 1.0f;
	SetMaterial(m_Material);
	//////////////////////////////////////////////////////////////////////////////////////////////////////

	// RenderState/////////////////////////////////////////////////////////////////////////////////////
	SetRenderState(GT_RS_SPECULARENABLE,false);
	SetRenderState(GT_RS_AMBIENT,GT_RGB(0,0,0));
	LightEnable(0,true);
	SetRenderState(GT_RS_SHADEMODE,GT_SHADE_GOURAUD);
	SetRenderState(GT_RS_LIGHTING,true);
	SetRenderState(GT_RS_CLIPPING,true);
	//////////////////////////////////////////////////////////////////////////////////////////////////////

	CreateZBuffer(GT_ZBUFF_16);
}

void GtSoftwareRenderDevice::Swap()
{
	BitBlt(m_hMainBuffer, 0, 0, m_iWidth, m_iHeight, m_hBackBuffer, 0, 0, SRCCOPY);
}

bool GtSoftwareRenderDevice::Clear(unsigned long ulCount, const GtRect* pRects, unsigned long ulFlags, unsigned int uiColor, float fZ, unsigned long ulStencil)
{
	static int	bkColor;
	static BYTE	a1[SCR_W * SCR_H * 3];

	if(GT_GET_RED(uiColor)   == GT_GET_GREEN(uiColor) && 
	   GT_GET_GREEN(uiColor) == GT_GET_BLUE(uiColor) && 
	   GT_GET_BLUE(uiColor)  == GT_GET_RED(uiColor))
		memset( m_pBit, uiColor, sizeof(BYTE) * 3 * m_iWidth * m_iHeight);
	else if(bkColor != uiColor)
	{
		bkColor = uiColor;

		for(int i=0; i<m_iWidth * m_iHeight * 3; i+=3)
		{
			a1[i  ] = GT_GET_BLUE(uiColor);
			a1[i+1] = GT_GET_GREEN(uiColor);
			a1[i+2] = GT_GET_RED(uiColor);
		}
	}
	else
		memcpy(m_pBit, a1, sizeof(BYTE) * 3 * m_iWidth * m_iHeight);

	// Z  Ŭ
	if(m_ZBufferState == GT_ZBUFF_16)
		memset(m_pZBuffer16, (int)((fZ * (float)m_uiZBufferSize) + 0.5f), sizeof(unsigned short) * m_iHeight * m_iWidth);
	else
		memset(m_pZBuffer32, (int)((fZ * (float)m_uiZBufferSize) + 0.5f), sizeof(unsigned int) * m_iHeight * m_iWidth);

	return true;
}

void GtSoftwareRenderDevice::Update()
{
}

/************************************************************************/
/* FPS                                                              */
/************************************************************************/
void GtSoftwareRenderDevice::DrawFPS(void)
{
	static char  str[64];
	static DWORD dwDelay;
	static int   nFrame,nTotalFrame;

	++nFrame; //  

	if(dwDelay < GetTickCount()) //  īƮ dwDelay ũٸ
	{
		nTotalFrame = nFrame; //   Total Frame 
		nFrame = 1; //ٽ 1 ʱȭ...
		dwDelay = GetTickCount() + 1000; //  ...

		ZeroMemory(str,sizeof(str));
		std::sprintf(str,"FPS : %d",nTotalFrame); //   ڿ 
	}
	WCHAR wcStr[256];
//	MultiByteToWideChar( CP_ACP, 0, str, strlen(str)+1, wcStr, sizeof(wcStr)/sizeof(wcStr[0]));
//	TextOut(m_hBackBuffer,0, 0, wcStr, (int)strlen(str));
}

void GtSoftwareRenderDevice::SetPixel(int x, int y, int color)
{
	int i = y * m_nPitch + x * 3;
	
	m_pBit[i]		=		GT_GET_BLUE(color);
	m_pBit[i+1]		=		GT_GET_GREEN(color);
	m_pBit[i+2]		=		GT_GET_RED(color);
}

HBITMAP GtSoftwareRenderDevice::CreateBitmap(HDC hDC, int iWidth, int iHeight, BYTE **ppBit)
{
	BITMAPINFO BitInfo;

	ZeroMemory(&BitInfo, sizeof(BitInfo));
	BitInfo.bmiHeader.biSize		= sizeof (BITMAPINFOHEADER);
	BitInfo.bmiHeader.biWidth		= iWidth;
	BitInfo.bmiHeader.biHeight		= -iHeight;
	BitInfo.bmiHeader.biPlanes		= 1;
	BitInfo.bmiHeader.biBitCount	= 24;
	BitInfo.bmiHeader.biCompression	= BI_RGB;

	m_nPitch						= (m_iWidth * (BitInfo.bmiHeader.biBitCount >> 3) + 3) & ~3;

	return CreateDIBSection(hDC, (BITMAPINFO*)&BitInfo, DIB_RGB_COLORS, (VOID**)ppBit, NULL, 0);
}

void GtSoftwareRenderDevice::DrawLine(int x0, int y0, int x1, int y1, int color)
{
	if(!ClipLine(x0,y0,x1,y1,0,m_iWidth-1,0,m_iHeight-1)) return;

	int dx,dy,d,x,y,incX,incY;

	if(x0 > x1)
	{
		dx = x0 - x1;
		incX = -1;
	}
	else
	{
		dx = x1 - x0;
		incX = 1;
	}

	if(y0 > y1)
	{
		dy = y0 - y1;
		incY = -1;
	}
	else
	{
		dy = y1 - y0;
		incY = 1;
	}

	x = x0; y = y0;

	if(dx > dy) // -1 < m < 1   X 
	{
		dy <<= 1;
		d = dy - dx; // ʱ  dy - dx
		dx <<= 1;

		while (x != x1)
		{
			SetPixel(x,y,color);

			if (d >= 0) // NE κ 
			{
				//   (dy-dx) ε
				y += incY;
				d -= dx; // -dx  ϰ
			}
			d += dy; // dy ϰڴٴ 
			x += incX;
		}
	}
	else // |m| > 1   Y 
	{
		dx <<= 1;
		d = dy - dx;
		dy <<= 1;

		while(y != y1)
		{
			SetPixel(x,y,color);
			if (d < 0) // NE κ 
			{
				x += incX;
				d += dy;
			}
			d -= dx;

			y += incY;
		}
	}

	SetPixel(x,y,color);
}

bool GtSoftwareRenderDevice::ClipLine(int& x0, int& y0, int& x1, int &y1, int xmin, int xmax, int ymin, int ymax)
{
	int nCheck[2] = {0,0};
	int tx[2] = {x0,x1};
	int ty[2] = {y0,y1};

	for(int i = 0; i < 2; i++)
	{
		if     (ymin > ty[i]) nCheck[i] |= 8;
		else if(ymax < ty[i]) nCheck[i] |= 4;
		if     (xmin > tx[i]) nCheck[i] |= 1;
		else if(xmax < tx[i]) nCheck[i] |= 2;
	}

	if(!(nCheck[0] | nCheck[1])) return true; //  ȿ 
	if(nCheck[0] & nCheck[1])    return false;//  ȭ ۿ ־ ׸ʿ䰡 

	int sel = 0;

	while((nCheck[0] | nCheck[1]))
	{
		(nCheck[0] != 0) ? sel = 0 : sel = 1;

		if(nCheck[sel] & 8)
		{
			tx[sel] = (int)((((float)(tx[1] - tx[0]) / (ty[1] - ty[0])) * (ymin - ty[sel])) + tx[sel] + 0.5f);
			ty[sel] = ymin;

			nCheck[sel] -= 8;
		}
		else if(nCheck[sel] & 4)
		{
			tx[sel] = (int)((((float)(tx[1] - tx[0]) / (ty[1] - ty[0])) * (ymax - ty[sel])) + tx[sel] + 0.5f);
			ty[sel] = ymax;

			nCheck[sel] -= 4;
		}
		else if(nCheck[sel] & 2)
		{
			ty[sel] = (int)((((float)(ty[1] - ty[0]) / (tx[1] - tx[0])) * (xmax - tx[sel])) + ty[sel] + 0.5f);
			tx[sel] = xmax;

			nCheck[sel] -= 2;
		}
		else if(nCheck[sel] & 1)
		{
			ty[sel] = (int)((((float)(ty[1] - ty[0]) / (tx[1] - tx[0])) * (xmin - tx[sel])) + ty[sel] + 0.5f);
			tx[sel] = xmin;

			nCheck[sel] -= 1;
		}

		if     (ymin > ty[sel]) nCheck[sel] |= 8;
		else if(ymax < ty[sel]) nCheck[sel] |= 4;
		if     (xmin > tx[sel]) nCheck[sel] |= 1;
		else if(xmax < tx[sel]) nCheck[sel] |= 2;

		if(nCheck[0] & nCheck[1]) return false;//  ȭ ۿ ־ ׸ʿ䰡 
	}

	x0 = tx[0]; y0 = ty[0];
	x1 = tx[1]; y1 = ty[1];

	return true;
}

void GtSoftwareRenderDevice::Line_Horizon(int x1, int x2, int y, int color)
{
	if(y < 0 || y >= m_iHeight) return; // ׸ʿ䰡 .

	if(x1 > x2) std::swap(x1,x2);
	// Ŭ
	if(x1 < 0) x1 = 0;
	if(x2 >= m_iWidth) x2 = m_iWidth - 1;

	// x2 < 0 x1 >= m_lWidth  ˻ ʾƵ ȴ.
	//   Ʒ for ڵ false ó ޴´.

	int idx;
	for(int i = x1; i <= x2; i++)
	{
		//SetPixel(i,y,color);
		idx = y *  m_nPitch + i * 3;

		m_pBit[idx]     = GT_GET_BLUE(color);
		m_pBit[idx + 1] = GT_GET_GREEN(color);
		m_pBit[idx + 2] = GT_GET_RED(color);
	}	
}

void GtSoftwareRenderDevice::Line_Vertical(int x, int y1, int y2, int color)
{
	if(y1 > y2) std::swap(y1,y2);

	for(int i = y1; i <= y2; i++)
		SetPixel(x,y1++,color);
}

void GtSoftwareRenderDevice::Line_Horizon_Gouraud(int x1, int x2, int y, int color1, int color2)
{
	if(y < 0 || y >= m_iHeight) return; // ׸ʿ䰡 .

	if(x1 > x2)
	{
		std::swap(x1,x2);
		std::swap(color1,color2);
	}
	// Ŭ
	if(x1 < 0) x1 = 0;
	if(x2 >= m_iWidth) x2 = m_iWidth - 1;

	// x2 < 0 x1 >= m_iWidth  ˻ ʾƵ ȴ.
	//   Ʒ for ڵ false ó ޴´.

	int Length = x2 - x1;
	int mR;
	int mG;
	int mB;

	if(!Length)
	{
		mR = mG = mB = 0;
	}
	else
	{
		mR = (((int) GT_GET_RED(color2)   - GT_GET_RED(color1)) << 16) / Length;
		mG = (((int) GT_GET_GREEN(color2) - GT_GET_GREEN(color1)) << 16)  / Length;
		mB = (((int) GT_GET_BLUE(color2)  - GT_GET_BLUE(color1)) << 16) / Length;
	}

	int red   = GT_GET_RED(color1) << 16;
	int green = GT_GET_GREEN(color1) << 16;
	int blue  = GT_GET_BLUE(color1) << 16;

	static const int r = (1 << 15); // 0.5 .   

	int idx;
	for(int i = x1; i <= x2; i++)
	{
		//SetPixel(i,y,color);
		idx = y *  m_nPitch + i * 3;

		m_pBit[idx]     = (blue + r) >> 16;
		m_pBit[idx + 1] = (green + r) >> 16;
		m_pBit[idx + 2] = (red + r) >> 16;

		red   += mR;
		green += mG;
		blue  += mB;
	}
}

//   ﰢ  ̵ (TopTriangle)
void GtSoftwareRenderDevice::GouraudShadingTopTriangle(int x1, int y1, int x2, int y2, int x3, int y3, int color1, int color2, int color3)
{
	if(y2 == y3 || y1 == y3)
		return;

	const int Height = y3 - y2;
	const int mLeft_Inv  = ((x3-x2) << 16) / Height;
	const int mRight_Inv = ((x3-x1) << 16) / Height;
	static const int r = (1 << 15); // 0.5 .   

	const int Length = y3 - y2;
	const int mLR = (((int) GT_GET_RED(color3)   - GT_GET_RED(color2)) << 16) / Length;
	const int mLG = (((int) GT_GET_GREEN(color3) - GT_GET_GREEN(color2)) << 16)  / Length;
	const int mLB = (((int) GT_GET_BLUE(color3)  - GT_GET_BLUE(color2)) << 16) / Length;

	const int mRR = (((int) GT_GET_RED(color3)   - GT_GET_RED(color1)) << 16) / Length;
	const int mRG = (((int) GT_GET_GREEN(color3) - GT_GET_GREEN(color1)) << 16)  / Length;
	const int mRB = (((int) GT_GET_BLUE(color3)  - GT_GET_BLUE(color1)) << 16) / Length;

	int LColor = color2;
	int LColorR = GT_GET_RED(color2) << 16;
	int LColorG = GT_GET_GREEN(color2) << 16;
	int LColorB = GT_GET_BLUE(color2) << 16;
	int RColor = color1;
	int RColorR = GT_GET_RED(color1) << 16;
	int RColorG = GT_GET_GREEN(color1) << 16;
	int RColorB = GT_GET_BLUE(color1) << 16;

	int xLeft = x2 << 16,xRight = x1 << 16;

	for(int i = y2; i <= y3; i++)
	{
		LColor = GT_RGB((LColorR + r) >> 16,(LColorG + r) >> 16,(LColorB + r) >> 16);
		RColor = GT_RGB((RColorR + r) >> 16,(RColorG + r) >> 16,(RColorB + r) >> 16);
		Line_Horizon_Gouraud((xLeft + r) >> 16,(xRight + r) >> 16,i,LColor,RColor);

		xLeft  += mLeft_Inv;
		xRight += mRight_Inv;

		LColorR += mLR;
		LColorG += mLG;
		LColorB += mLB;

		RColorR += mRR;
		RColorG += mRG;
		RColorB += mRB;
	}
}

// Ʒ  ﰢ  ̵(BottomTriangle)
void GtSoftwareRenderDevice::GouraudShadingBottomTriangle(int x1, int y1, int x2, int y2, int x3, int y3, int color1, int color2, int color3)
{
	if(y1 == y2 || y1 == y3)
		return;

	const int Height = y2 - y1;
	const int mLeft_Inv  = ((x2-x1) << 16) / Height;
	const int mRight_Inv = ((x3-x1) << 16) / Height;
	static const int r = (1 << 15); // 0.5 .   

	const int Length = y2 - y1;
	const int mLR = (((int) GT_GET_RED(color2)   - GT_GET_RED(color1)) << 16) / Length;
	const int mLG = (((int) GT_GET_GREEN(color2) - GT_GET_GREEN(color1)) << 16)  / Length;
	const int mLB = (((int) GT_GET_BLUE(color2)  - GT_GET_BLUE(color1)) << 16) / Length;

	const int mRR = (((int) GT_GET_RED(color3)   - GT_GET_RED(color1)) << 16) / Length;
	const int mRG = (((int) GT_GET_GREEN(color3) - GT_GET_GREEN(color1)) << 16)  / Length;
	const int mRB = (((int) GT_GET_BLUE(color3)  - GT_GET_BLUE(color1)) << 16) / Length;

	int LColor = color1;
	int LColorR = GT_GET_RED(color1) << 16;
	int LColorG = GT_GET_GREEN(color1) << 16;
	int LColorB = GT_GET_BLUE(color1) << 16;
	int RColor = LColor;
	int RColorR = LColorR;
	int RColorG = LColorG;
	int RColorB = LColorB;


	int xLeft = x1 << 16,xRight = xLeft;

	for(int i = y1; i <= y2; i++)
	{
		LColor = GT_RGB((LColorR + r) >> 16,(LColorG + r) >> 16,(LColorB + r) >> 16);
		RColor = GT_RGB((RColorR + r) >> 16,(RColorG + r) >> 16,(RColorB + r) >> 16);

		Line_Horizon_Gouraud((xLeft + r) >> 16,(xRight + r) >> 16,i,LColor,RColor);

		xLeft  += mLeft_Inv;
		xRight += mRight_Inv;

		LColorR += mLR;
		LColorG += mLG;
		LColorB += mLB;

		RColorR += mRR;
		RColorG += mRG;
		RColorB += mRB;
	}
}

//   ̵(ð ݴ LH)
void GtSoftwareRenderDevice::GouraudShadingTriangle(int x1,int y1,int x2,int y2,int x3,int y3,int color1,int color2,int color3)
{
	if(y1 > y2)
	{
		std::swap(y1,y2);
		std::swap(x1,x2);
		std::swap(color1,color2);
	}
	if(y1 > y3)
	{
		std::swap(y1,y3);
		std::swap(x1,x3);
		std::swap(color1,color3);
	}
	if(y2 > y3)
	{
		std::swap(y2,y3);
		std::swap(x2,x3);
		std::swap(color2,color3);
	}

	if(y1 == y2) //   
	{
		GouraudShadingTopTriangle(x1,y1,x2,y2,x3,y3,color1,color2,color3);
	}
	else if(y2 == y3) // Ʒ  
	{
		GouraudShadingBottomTriangle(x1,y1,x2,y2,x3,y3,color1,color2,color3);
	}
	else // ,Ʒ  äϴ 
	{
		static const int r = (1 << 15); // 0.5 .   
		const int YLength = y2 - y1;
		const int NewX = (((x3-x1) << 16) / (y3-y1) * YLength + (x1 << 16) + r) >> 16;
		const int ColorLength = (y3 - y1);
		const int mNR = ((((int) GT_GET_RED(color3)   - GT_GET_RED(color1)) << 16)   / ColorLength) * YLength;
		const int mNG = ((((int) GT_GET_GREEN(color3) - GT_GET_GREEN(color1)) << 16) / ColorLength) * YLength;
		const int mNB = ((((int) GT_GET_BLUE(color3)  - GT_GET_BLUE(color1)) << 16)  / ColorLength) * YLength;
		const int NewColor = GT_RGB(((GT_GET_RED(color1)   << 16) + mNR + r) >> 16,
							        ((GT_GET_GREEN(color1) << 16) + mNG + r) >> 16,
									((GT_GET_BLUE(color1)  << 16) + mNB + r) >> 16);

		GouraudShadingBottomTriangle(x1,y1,x2,y2,NewX,y2,color1,color2,NewColor);
		GouraudShadingTopTriangle(NewX,y2,x2,y2,x3,y3,NewColor,color2,color3);
	}
}

// Ʈ 
// D3DDevice 
void GtSoftwareRenderDevice::SetTransform(const GT_RENDERING_PIPELINE eState, const GtMatrix4 &mat)
{
	switch(eState)
	{
	case GT_WORLD:
		m_RenderPipeLine.matWorld = mat;
		break;
	case GT_VIEW:
		{
			GtMatrix4 RCa(mat);
			RCa.m41 = RCa.m42 = RCa.m43 = 0;
			RCa = mat / RCa;
			m_RenderPipeLine.ptCameraEye.x = -RCa.m41;
			m_RenderPipeLine.ptCameraEye.y = -RCa.m42;
			m_RenderPipeLine.ptCameraEye.z = -RCa.m43;
			m_RenderPipeLine.matView = mat;
		}
		break;
	case GT_PROJECTION:
		m_RenderPipeLine.matProj = mat;
		break;
	}
}

bool GtSoftwareRenderDevice::SetViewPort(const sGT_ViewPort &pViewPort)
{
	m_RenderPipeLine.matViewPort.MakeIdentity();

	const float nCalcWidth  = (float)(pViewPort.Width  >> 1);
	const float nCalcHeight = (float)(pViewPort.Height >> 1);

	m_RenderPipeLine.matViewPort.m11 =  nCalcWidth;
	m_RenderPipeLine.matViewPort.m22 = -nCalcHeight;
	m_RenderPipeLine.matViewPort.m33 = pViewPort.MaxZ - pViewPort.MinZ;
	m_RenderPipeLine.matViewPort.m41 = pViewPort.x + nCalcWidth;
	m_RenderPipeLine.matViewPort.m42 = pViewPort.y + nCalcHeight;;
	m_RenderPipeLine.matViewPort.m43 = pViewPort.MinZ;

	return true;
}

bool GtSoftwareRenderDevice::GetViewPort(sGT_ViewPort &pOut)
{
	pOut.Width  = (unsigned long) (  m_RenderPipeLine.matViewPort.m11  * 2.f);
	pOut.Height = (unsigned long) ((-m_RenderPipeLine.matViewPort.m22) * 2.f);
	pOut.MinZ   = m_RenderPipeLine.matViewPort.m43;
	pOut.MaxZ   = m_RenderPipeLine.matViewPort.m33 + pOut.MinZ;
	pOut.x      = (unsigned long) (m_RenderPipeLine.matViewPort.m41 - m_RenderPipeLine.matViewPort.m11);
	pOut.y      = (unsigned long) (m_RenderPipeLine.matViewPort.m42 + m_RenderPipeLine.matViewPort.m22);

	return true;
}

bool GtSoftwareRenderDevice::CreateMatrixClipping(const sGT_Clipping &pClip)
{
	m_RenderPipeLine.matClip.MakeIdentity();

	m_RenderPipeLine.matClip.m11 = 2.f / pClip.ClipWidth;
	m_RenderPipeLine.matClip.m22 = 2.f / pClip.ClipHeight;
	m_RenderPipeLine.matClip.m33 = 1.f / (pClip.MaxZ - pClip.MinZ);
	m_RenderPipeLine.matClip.m41 = -1 - 2 * pClip.ClipX / pClip.ClipWidth;
	m_RenderPipeLine.matClip.m42 =  1 - 2 * pClip.ClipY / pClip.ClipHeight;
	m_RenderPipeLine.matClip.m43 = -pClip.MinZ / (pClip.MaxZ - pClip.MinZ);

	return true;
}

void GtSoftwareRenderDevice::DrawPrimitive(GT_PRIMITIVETYPE ePrimitiveState, unsigned int iStartVertex, unsigned int iPrimitiveCount)
{
	RenderingPipeLine();
}

bool GtSoftwareRenderDevice::BackFaceCullingLH(const GtPoint3f &p1, const GtPoint3f &p2, const GtPoint3f &p3)
{
	static GtPoint3f n,l;
	n = (p3 - p1).Cross(p2 - p1); // v cross u

	l.x = m_RenderPipeLine.ptCameraEye.x - p1.x;
	l.y = m_RenderPipeLine.ptCameraEye.y - p1.y;
	l.z = m_RenderPipeLine.ptCameraEye.z - p1.z;

	if(n.Dot(l) < 0.f)
		return false;

	return true;
}

bool GtSoftwareRenderDevice::BackFaceCullingRH(const GtPoint3f &p1, const GtPoint3f &p2, const GtPoint3f &p3)
{
	static GtPoint3f n,l;
	n = (p2 - p1).Cross(p3 - p1); // u cross v

	l.x = m_RenderPipeLine.ptCameraEye.x - p1.x;
	l.y = m_RenderPipeLine.ptCameraEye.y - p1.y;
	l.z = m_RenderPipeLine.ptCameraEye.z - p1.z;

	if(l.Dot(n) < 0.f)
		return false;

	return true;
}

const int GtSoftwareRenderDevice::CalcLighting(GtPoint3f pt)
{
	if(!m_RenderStateFlag.bLighting) return 0;// 

	GT_COLORVALUE Color(GT_GET_RED(m_RenderStateFlag.lAmbient)/255.f,
						GT_GET_GREEN(m_RenderStateFlag.lAmbient)/255.f,
						GT_GET_BLUE(m_RenderStateFlag.lAmbient)/255.f,1.f);

	static GtLight TotalLightColor;
	static GtColorValue Ambient,Diffuse,Specular;

	SetMaterialColor(Ambient,Diffuse,Specular);

	memset(&TotalLightColor,0,sizeof(GtLight));

	mapLightSet::iterator iter = m_Light.begin();
	for(; iter != m_Light.end(); ++iter)
	{
		if(!(*iter).second.m_bEnable) continue;
		TotalLightColor.m_Ambient += (*iter).second.m_Light.m_Ambient;
		float temp = max(0,pt.Dot((*iter).second.m_Light.m_Direction));
		TotalLightColor.m_Diffuse += Diffuse * (*iter).second.m_Light.m_Diffuse * (max(0,pt.Dot((*iter).second.m_Light.m_Direction)));
		
		if(m_RenderStateFlag.bSpecularEnable)
		{
			GtPoint3f E(m_RenderPipeLine.matView.m13,m_RenderPipeLine.matView.m23,m_RenderPipeLine.matView.m33);
			GtPoint3f H = (E+(*iter).second.m_Light.m_Direction)/(E+(*iter).second.m_Light.m_Direction).Length();
			TotalLightColor.m_Specular += (*iter).second.m_Light.m_Specular * pow(max(pt.Dot(H),0),m_Material.m_fPower);
		}
	}

	//ȯ汤
	// I = Ia * Ka
	Color  = Ambient * (Color + TotalLightColor.m_Ambient);

	//ݻ籤
	// I = Id * Kd * (N dot L)
	Color += TotalLightColor.m_Diffuse;

	//ݻ籤
	//I = Is * Ks * (N * H)^power
	if(m_RenderStateFlag.bSpecularEnable)
		Color += Specular * TotalLightColor.m_Specular;

	//emission
	Color += m_Material.m_Emissive;

	return GT_RGB(Color.r * 255.f/Color.a,Color.g * 255.f/Color.a,Color.b * 255.f/Color.a);
}

bool GtSoftwareRenderDevice::RenderingPipeLine()
{
	// PipeLine : Local -> World -> Camera -> Projection -> Clipping

	// FVF_XYZRHW  ƴҶ    óش.
	//  RHW ̹  Ʈ  ģ Ʈ̱ ̴.
	if(!(m_FVF.IsFVF(GT_FVF_XYZRHW)))
	{
		Local_To_World_Object();
		if(!World_To_Camera_Object())
			return false;
		Camera_To_Proj_Object();
	}
	// RHW Ǿ Clipping  ؾ Ѵ.
	if(m_RenderStateFlag.bClip)
		Proj_Object_To_Clipping(); //  Ʈȯ(ȭ )

	Clipping_To_ViewPort();
	Fill_Rasterize();

	return true;
}

bool GtSoftwareRenderDevice::Local_To_World_Object(void)
{
	//m_RenderObj.VtxBuff->m_vWorld.TransformCoordRow(m_RendPipe.matWorld);
	m_RenderPipeLine.matTotal = m_RenderPipeLine.matWorld;

	// ĸ 
	void* t = m_RenderObj.VertexBuffer->m_pVB;

	if(m_FVF.IsFVF(GT_FVF_XYZ))
	{
	//	static char* pVoid;
		GtPoint3f t_xyz[3];
	//	pVoid = (char*)t;
		m_FVF.ResetPtr();

		m_RenderObj.bVisible.resize(m_RenderObj.VertexCount/3);
		for(unsigned int i = 0; i < m_RenderObj.VertexCount/3; i++)
		{
			for(unsigned int j = 0; j < 3; j++)
			{
				t_xyz[j] = *(GtPoint3f*) m_FVF.NextValue(GT_FVF_NONE,GT_FVF_XYZ);
				m_FVF.NextNode();
			}

			/// test
			/// Vector * 
			t_xyz[0].TransformCoordRow(m_RenderPipeLine.matTotal);
			t_xyz[1].TransformCoordRow(m_RenderPipeLine.matTotal);
			t_xyz[2].TransformCoordRow(m_RenderPipeLine.matTotal);
			///

			m_RenderObj.bVisible[i] = BackFaceCullingRH(t_xyz[0],t_xyz[1],t_xyz[2]);
		}
	}
	return true;
}


bool GtSoftwareRenderDevice::World_To_Camera_Object(void)
{
	m_RenderPipeLine.matTotal *= m_RenderPipeLine.matView;
	m_RenderObj.VertexBuffer->m_Sphere.v = GtPoint3f(0,0,0);
	m_RenderObj.VertexBuffer->m_Sphere.v.TransformCoordRow(m_RenderPipeLine.matWorld);

	// 
	unsigned long bLight;
	GetRenderState(GT_RS_LIGHTING,&bLight);
	if(bLight)
	{
		mapLightSet::iterator iter = m_Light.begin();

		for(; iter != m_Light.end(); ++iter)
		{
			if(!(*iter).second.m_bEnable) continue;

			(*iter).second.m_Light.m_Direction = (*iter).second.m_Org_Direction;

			//(*iter).second.Light.Direction.TransformCoordRow(m_RendPipe.matView);
			//(*iter).second.Light.Direction.Normalize();
			(*iter).second.m_Light.m_Direction = -(*iter).second.m_Light.m_Direction;
		}
	}

	// Ʈ Ŭ
	m_Frustum.MakeFrustum(m_RenderPipeLine.matView,m_RenderPipeLine.matProj);

	int State = m_Frustum.StateSphereFrustum(m_RenderObj.VertexBuffer->m_Sphere);

	// State  : 0 -  ü  .   Ʈ ׸ ʿ䰡 .
	//            1 -  ü 浹 Ͽ.
	//            2 -  ü ο ִ.
	switch(State)
	{
	case 0: // ׸ʿ䰡 Ƿ ⼭ .
		return false;
	case 1: // ü ̿ Ʈ Ƿ и Ѵ.
		{
			// ĸ 
			void* t = m_RenderObj.VertexBuffer->m_pVB;

			if(m_FVF.IsFVF(GT_FVF_XYZ))//IsFVF(GT_FVF_XYZ))
			{
				static GtPoint3f t_xyz[3];
				m_FVF.ResetPtr();

				for(unsigned int i = 0; i < m_RenderObj.VertexCount/3; i++)
				{
					for(unsigned int j = 0; j < 3; j++)
					{
						t_xyz[j] = *(GtPoint3f*) m_FVF.NextValue(GT_FVF_NONE,GT_FVF_XYZ);
						m_FVF.NextNode();
					}

					if(!m_RenderObj.bVisible[i]) continue; // Ⱥ̴° ǵ帱 ʿ䰡 .

					/// test
					t_xyz[0].TransformCoordRow(m_RenderPipeLine.matWorld);
					t_xyz[1].TransformCoordRow(m_RenderPipeLine.matWorld);
					t_xyz[2].TransformCoordRow(m_RenderPipeLine.matWorld);
					///

					m_RenderObj.bVisible[i] = m_Frustum.StatePolygonFrustum(t_xyz[0],t_xyz[1],t_xyz[2]);
				}
			}
		}
		break;
	case 2: // ̻ ǵ帱  ׳   ȴ.
		break;
	default:
		assert(0);
		break;
	}
	return true;
}

inline bool GtSoftwareRenderDevice::Camera_To_Proj_Object(void)
{
	m_RenderPipeLine.matTotal *= m_RenderPipeLine.matProj;
	return true;
}

bool GtSoftwareRenderDevice::Proj_Object_To_Clipping(void)
{
	m_RenderPipeLine.matTotal *= m_RenderPipeLine.matClip;
	return true;
}

inline bool GtSoftwareRenderDevice::Clipping_To_ViewPort(void)
{
	m_RenderPipeLine.matTotal *= m_RenderPipeLine.matViewPort;
	return true;
}

bool GtSoftwareRenderDevice::Fill_Rasterize(void)
{
	static unsigned long lShadeMode;
	static unsigned int i, j;

	GetRenderState(GT_RS_SHADEMODE, &lShadeMode);

	GtPoint3f	v3_xyz[3];
	GtPoint3f	v3_normal[3];
	sGT_FVF_DIFFUSE diffuse[3];
	GtPoint3f	v3_uv[3];

	m_FVF.ResetPtr();

	switch(lShadeMode)
	{
	case GT_SHADE_FLAT:
		break;
	case GT_SHADE_GOURAUD:
		{
			if(m_FVF.IsFVF(GT_FVF_XYZ))
			{
				for(i=0; i<m_RenderObj.VertexCount/3; i++)
				{
					for(j = 0; j < 3; j++)
					{
						v3_xyz[j] = *(GtPoint3f*)m_FVF.NextValue(GT_FVF_NONE, GT_FVF_XYZ);

						if(m_FVF.IsFVF(GT_FVF_NORMAL))
						{
							v3_normal[j] = *(GtPoint3f*) m_FVF.NextValue(GT_FVF_XYZ,GT_FVF_NORMAL);

							if(m_FVF.IsFVF(GT_FVF_TEX1))
							{
								v3_uv[j] = *(GtPoint3f*) m_FVF.NextValue(GT_FVF_NORMAL,GT_FVF_TEX1);
								v3_uv[j].z = 1.f;
							}
						}
						else if(m_FVF.IsFVF(GT_FVF_DIFFUSE))
						{
							diffuse[j] = *(sGT_FVF_DIFFUSE*) m_FVF.NextValue(GT_FVF_XYZ,GT_FVF_DIFFUSE);

							if(m_FVF.IsFVF(GT_FVF_TEX1))
							{
								v3_uv[j] = *(GtPoint3f*) m_FVF.NextValue(GT_FVF_DIFFUSE,GT_FVF_TEX1);
								v3_uv[j].z = 1.f;
							}
						}
						m_FVF.NextNode();
					}
					if(m_FVF.IsFVF(GT_FVF_NORMAL))
					{
						if(m_RenderObj.bVisible[i])
						{
							float z1,z2,z3;
                            z1 = v3_xyz[0].TransformCoordRow(m_RenderPipeLine.matTotal);
							z2 = v3_xyz[1].TransformCoordRow(m_RenderPipeLine.matTotal);
							z3 = v3_xyz[2].TransformCoordRow(m_RenderPipeLine.matTotal);

							GtMatrix4 b = m_RenderPipeLine.matWorld;
							b = b.MakeInverse();
							b = b.MakeTranspose();

							v3_normal[0].TransformNormalRow(b);
							v3_normal[1].TransformNormalRow(b);
							v3_normal[2].TransformNormalRow(b);

							if(m_RenderStateFlag.bZEnable)
							{
								GouraudShadingTriangleZ(
									((int)(v3_xyz[0].x + 0.5f)),((int)(v3_xyz[0].y + 0.5f)),(v3_xyz[0].z),
									((int)(v3_xyz[1].x + 0.5f)),((int)(v3_xyz[1].y + 0.5f)),(v3_xyz[1].z),
									((int)(v3_xyz[2].x + 0.5f)),((int)(v3_xyz[2].y + 0.5f)),(v3_xyz[2].z),
									CalcLighting(v3_normal[0].Normalized()),
									CalcLighting(v3_normal[1].Normalized()),
									CalcLighting(v3_normal[2].Normalized()),
									v3_uv[0].x/z1,v3_uv[0].y/z1,v3_uv[1].x/z2,v3_uv[1].y/z2,v3_uv[2].x/z3,v3_uv[2].y/z3,
									1/z1,1/z2,1/z3
									);
							}
							else
							{
								GouraudShadingTriangle(
									((int)(v3_xyz[0].x + 0.5f)),((int)(v3_xyz[0].y + 0.5f)),
									((int)(v3_xyz[1].x + 0.5f)),((int)(v3_xyz[1].y + 0.5f)),
									((int)(v3_xyz[2].x + 0.5f)),((int)(v3_xyz[2].y + 0.5f)),
									CalcLighting(v3_normal[0]),
									CalcLighting(v3_normal[1]),
									CalcLighting(v3_normal[2])
									);
							}
						}
					}
					else if(m_FVF.IsFVF(GT_FVF_DIFFUSE))
					{
						if(m_RenderObj.bVisible[i])
						{
							GouraudShadingTriangle(
								((int)(v3_xyz[0].x + 0.5f)),((int)(v3_xyz[0].y + 0.5f)),
								((int)(v3_xyz[1].x + 0.5f)),((int)(v3_xyz[1].y + 0.5f)),
								((int)(v3_xyz[2].x + 0.5f)),((int)(v3_xyz[2].y + 0.5f)),
								diffuse[0].ul[0],diffuse[1].ul[0],diffuse[2].ul[0]);
						}
					}
				}
			}
			else if(m_FVF.IsFVF(GT_FVF_XYZRHW))
			{

			}
		}
		break;
	case GT_SHADE_PHONG:
		break;

	}

	return true;
}

bool GtSoftwareRenderDevice::SetRenderState(GT_RENDERSTATE eState, unsigned long ulValue)
{
	switch(eState)
	{
	case GT_RS_ZENABLE:
		m_RenderStateFlag.bZEnable = (ulValue) ? 1 : 0;
		return true;
	case GT_RS_SHADEMODE:
		m_RenderStateFlag.ShadeMode = ulValue;
	return true;
	case GT_RS_SPECULARENABLE : // ulValue -> bool
		m_RenderStateFlag.bSpecularEnable = (ulValue) ? 1 : 0;
	return true;
	case GT_RS_CLIPPING:
		m_RenderStateFlag.bClip = (ulValue) ? 1 : 0;
	return true;
	case GT_RS_LIGHTING : // ulValue -> bool
		m_RenderStateFlag.bLighting = (ulValue) ? 1 : 0;
	return true;
	case GT_RS_AMBIENT : // ulValue -> int
		m_RenderStateFlag.lAmbient = ulValue;
	return true;
	/*
	D3DRS_NORMALIZENORMALS 
	  ڵ ȭ ȿϰ Ϸ TRUE, ȿ Ϸ FALSE  Ѵ. 
	Ʈ FALSE ̴.   ȿϰ ϸ, 
	 ī޶  ȯ     ȭ, 꿡 ð ɸ. 
	*/
	case GT_RS_NORMALIZENORMALS : // ulValue -> bool
		m_RenderStateFlag.bNormalizeNormals = (ulValue) ? 1 : 0;
	return true;
	case GT_RS_DIFFUSEMATERIALSOURCE :
		m_RenderStateFlag.DiffuseMaterialSource = ulValue;
	return true;
	case GT_RS_SPECULARMATERIALSOURCE :
		m_RenderStateFlag.SpecularMaterialSource = ulValue;
	return true;
	case GT_RS_AMBIENTMATERIALSOURCE :
		m_RenderStateFlag.AmbientMaterialSource = ulValue;
	return true;
	case GT_RS_EMISSIVEMATERIALSOURCE :
		m_RenderStateFlag.EmissiveMaterialSource = ulValue;
	return true;
	}
	return false;
}

bool GtSoftwareRenderDevice::GetRenderState(GT_RENDERSTATE eState, unsigned long *pValue)
{
	switch(eState)
	{
	case GT_RS_ZENABLE:
		*pValue = m_RenderStateFlag.bZEnable;
		return true;
	case GT_RS_SHADEMODE:
		*pValue = m_RenderStateFlag.ShadeMode;
		return true;
	case GT_RS_SPECULARENABLE : // _Value -> bool
		*pValue = m_RenderStateFlag.bSpecularEnable;
		return true;
	case GT_RS_CLIPPING:
		*pValue = m_RenderStateFlag.bClip;
		return true;
	case GT_RS_LIGHTING : // _Value -> bool
		*pValue = m_RenderStateFlag.bLighting;
		return true;
	case GT_RS_AMBIENT : // _Value -> int
		*pValue = m_RenderStateFlag.lAmbient;
		return true;
	case GT_RS_NORMALIZENORMALS : // _Value -> bool
		*pValue = m_RenderStateFlag.bNormalizeNormals;
		return true;
	case GT_RS_DIFFUSEMATERIALSOURCE :
		*pValue = m_RenderStateFlag.DiffuseMaterialSource;
		return true;
	case GT_RS_SPECULARMATERIALSOURCE :
		*pValue = m_RenderStateFlag.SpecularMaterialSource;
		return true;
	case GT_RS_AMBIENTMATERIALSOURCE :
		*pValue = m_RenderStateFlag.AmbientMaterialSource;
		return true;
	case GT_RS_EMISSIVEMATERIALSOURCE :
		*pValue = m_RenderStateFlag.EmissiveMaterialSource;
		return true;
	}

	return false;
}

void GtSoftwareRenderDevice::SetLight(unsigned long ulIndex, const GtLight &in)
{
	m_Light[ulIndex].m_Light = in;
	m_Light[ulIndex].m_Org_Direction = in.m_Direction;
}

bool GtSoftwareRenderDevice::LightEnable(unsigned long ulIndex, bool bEnable)
{
	mapLightSet::iterator it = m_Light.find(ulIndex);

	if(it == m_Light.end())
		return false;

	(*it).second.m_bEnable = bEnable;

	return true;
}

void GtSoftwareRenderDevice::SetMaterial(const GtMaterial &in)
{
	m_Material = in;
}

void GtSoftwareRenderDevice::SetMaterialColor(GtColorValue &Ambient, GtColorValue &Diffuse, GtColorValue &Specular)
{
	switch(m_RenderStateFlag.AmbientMaterialSource)
	{
	case GT_MCS_MATERIAL:
		Ambient = m_Material.m_Ambient;
		break;
	case GT_MCS_COLOR1:
		break;
	case GT_MCS_COLOR2:
		break;
	}

	switch(m_RenderStateFlag.DiffuseMaterialSource)
	{
	case GT_MCS_MATERIAL:
		Diffuse = m_Material.m_Diffuse;
		break;
	case GT_MCS_COLOR1:
		break;
	case GT_MCS_COLOR2:
		break;
	}
	switch(m_RenderStateFlag.SpecularMaterialSource)
	{
	case GT_MCS_MATERIAL:
		Specular = m_Material.m_Specular;
		break;
	case GT_MCS_COLOR1:
		break;
	case GT_MCS_COLOR2:
		break;
	}
}


bool GtSoftwareRenderDevice::SetFVF(unsigned long ulFVF)
{
	m_FVF.ReMakeFVF(ulFVF);
	return true;
}
bool GtSoftwareRenderDevice::GetFVF(unsigned long *ulFVF)
{
	if(!ulFVF)
		return false;
	*ulFVF = m_FVF.GetFVF();

	return true;
}

bool GtSoftwareRenderDevice::CreateVertexBuffer(unsigned int uiLength, unsigned long ulUsage, unsigned long ulFVF, GT_POOL ePool, GtVertexBuffer **ppVertexBuffer, void **pSharedHandle)
{
	m_FVF.ReMakeFVF(ulFVF);
	*ppVertexBuffer = new GtVertexBuffer(uiLength);

	return true;
}

bool GtSoftwareRenderDevice::SetStreamSource(unsigned int uiStreamNumber, GtVertexBuffer *pStreamData, unsigned int uiOffsetInBytes, unsigned int uiStride)
{
	if(!pStreamData)
		return false;

	m_RenderObj.VertexBuffer = pStreamData;
	m_RenderObj.VertexCount = pStreamData->m_uiLength / uiStride;

	m_FVF.ReMakePtr(m_RenderObj.VertexBuffer->m_pVB);

	if(m_FVF.IsFVF(GT_FVF_XYZ))
	{
		GtPoint3f xyz;
		float fBig = 0.0f;
		float fLength = 0.0f;
		m_FVF.ResetPtr();

		for(unsigned int i = 0; i<m_RenderObj.VertexCount; i++)
		{
			xyz = *(GtPoint3f*) m_FVF.NextValue(GT_FVF_NONE, GT_FVF_XYZ);
			m_FVF.NextNode();

			fLength = (xyz - GtPoint3f(0.0f, 0.0f, 0.0f)).Length();
			if(fLength > 0)
				fBig = fLength;
		}
		m_RenderObj.VertexBuffer->m_Sphere.m_fRadius = fBig;
	}

	return true;
}

void GtSoftwareRenderDevice::CreateZBuffer(GTZBUFFERBIT eZBufferBit)
{
	if(eZBufferBit == GT_ZBUFF_16)
	{
		m_ZBufferState = GT_ZBUFF_16;
		m_pZBuffer16 = new unsigned short[m_iWidth * m_iHeight];
		m_uiZBufferSize = (unsigned int)GtPow(2, sizeof(unsigned short) * 8) -1;
		memset(m_pZBuffer16, m_uiZBufferSize, sizeof(unsigned short) * m_iWidth * m_iHeight);

	}
	else
	{
		m_ZBufferState = GT_ZBUFF_32;
		m_pZBuffer32 = new unsigned int[m_iWidth * m_iHeight];
		m_uiZBufferSize = (unsigned int)GtPow(2, sizeof(unsigned short) * 8) -1;
		memset(m_pZBuffer32, m_uiZBufferSize, sizeof(unsigned short) * m_iWidth * m_iHeight);
	}
}

bool GtSoftwareRenderDevice::WriteZBuffer(int x, int y, unsigned int uiZValue)
{
	if(x < 0 || x >= m_iWidth) return false;
	if(y < 0 || y >= m_iHeight) return false;

	if(m_ZBufferState == GT_ZBUFF_16)
	{
		if(m_pZBuffer16[y * m_iWidth + x] >= uiZValue)
		{
			m_pZBuffer16[y * m_iWidth + x] = uiZValue;
			return true;
		}
	}
	else // 32 Bit
	{
		if(m_pZBuffer32[y * m_iWidth + x] >= uiZValue)
		{
			m_pZBuffer32[y * m_iWidth + x] = uiZValue;
			return true;
		}
	}

	return false;
}

bool GtSoftwareRenderDevice::CreateTextureFromFile(const char *pName, LPGT_TEXTURE *ppTexture)
{
	*ppTexture = new GtTexture(pName, m_pBit, 3);
	if(!ppTexture)
		return false;
	return true;
}

bool GtSoftwareRenderDevice::SetTexture(unsigned long ulStage, LPGT_TEXTURE pTexture)
{
	// ؽ ִ밡 
	if(ulStage >= TEX_STAGE_NUM)
		return false;
	m_pTexture[ulStage] = pTexture;

	if(!pTexture)
		return false;

	return true;
}


//   ﰢ  ̵
void GtSoftwareRenderDevice::GouraudShadingTopTriangleZ(int x1,int y1,float z1,int x2,int y2,float z2,int x3,int y3,float z3,int color1,int color2,int color3,
											  float u1,float v1,float u2,float v2,float u3,float v3,float invZ1,float invZ2,float invZ3)
{
	if(y2 == y3 || y1 == y3) return;

	const int Height = y3 - y2;
	const int mLeft_Inv  = ((x3-x2) << 16) / Height;
	const int mRight_Inv = ((x3-x1) << 16) / Height;

	const float mZL_Inv = (((float)z3-z2)) / Height;
	const float mZR_Inv = (((float)z3-z1)) / Height;

	const float minvZL_Inv = (((float)invZ3-invZ2)) / Height;
	const float minvZR_Inv = (((float)invZ3-invZ1)) / Height;

	const float m_UL_Inv = (((float)u3-u2)) / Height;
	const float m_UR_Inv = (((float)u3-u1)) / Height;

	const float m_VL_Inv = (((float)v3-v2)) / Height;
	const float m_VR_Inv = (((float)v3-v1)) / Height;

	static const int r = (1 << 15); // 0.5 .   

	const int Length = y3 - y2;
	const int mLR = (((int) GT_GET_RED(color3)   - GT_GET_RED(color2)) << 16) / Length;
	const int mLG = (((int) GT_GET_GREEN(color3) - GT_GET_GREEN(color2)) << 16)  / Length;
	const int mLB = (((int) GT_GET_BLUE(color3)  - GT_GET_BLUE(color2)) << 16) / Length;

	const int mRR = (((int) GT_GET_RED(color3)   - GT_GET_RED(color1)) << 16) / Length;
	const int mRG = (((int) GT_GET_GREEN(color3) - GT_GET_GREEN(color1)) << 16)  / Length;
	const int mRB = (((int) GT_GET_BLUE(color3)  - GT_GET_BLUE(color1)) << 16) / Length;

	int LColor = color2;
	int LColorR = GT_GET_RED(color2) << 16;
	int LColorG = GT_GET_GREEN(color2) << 16;
	int LColorB = GT_GET_BLUE(color2) << 16;
	int RColor = color1;
	int RColorR = GT_GET_RED(color1) << 16;
	int RColorG = GT_GET_GREEN(color1) << 16;
	int RColorB = GT_GET_BLUE(color1) << 16;

	int xLeft = x2 << 16,xRight = x1 << 16;
	float zLeft = z2,		zRight = z1;
	float uLeft = u2,		uRight = u1;
	float vLeft = v2,		vRight = v1;
	float invZLeft = invZ2,	invZRight = invZ1;

	for(int i = y2; i <= y3; i++)
	{
		LColor = GT_RGB((LColorR + r) >> 16,(LColorG + r) >> 16,(LColorB + r) >> 16);
		RColor = GT_RGB((RColorR + r) >> 16,(RColorG + r) >> 16,(RColorB + r) >> 16);
		Line_Horizon_GouraudZ((xLeft + r) >> 16,(xRight + r) >> 16,i,
							  (zLeft ),(zRight),LColor,RColor,uLeft,vLeft,uRight,vRight,invZLeft,invZRight);

		xLeft  += mLeft_Inv;
		xRight += mRight_Inv;

		zLeft  += mZL_Inv;
		zRight += mZR_Inv;

		invZLeft += minvZL_Inv;
		invZRight += minvZR_Inv;

		uLeft += m_UL_Inv;
		uRight+= m_UR_Inv;

		vLeft += m_VL_Inv;
		vRight+= m_VR_Inv;

		LColorR += mLR;
		LColorG += mLG;
		LColorB += mLB;

		RColorR += mRR;
		RColorG += mRG;
		RColorB += mRB;
	}
}


// Ʒ  ﰢ  ̵
void GtSoftwareRenderDevice::GouraudShadingBottomTriangleZ(int x1,int y1,float z1,int x2,int y2,float z2,int x3,int y3,float z3,int color1,int color2,int color3,
												 float u1,float v1,float u2,float v2,float u3,float v3,float invZ1,float invZ2,float invZ3)
{
	if(y1 == y2 || y1 == y3) return;

	const int Height = y2 - y1;
	const int mLeft_Inv  = ((x2-x1) << 16) / Height;
	const int mRight_Inv = ((x3-x1) << 16) / Height;

	const float m_ZL_Inv = (((float)z2-z1)) / Height;
	const float m_ZR_Inv = (((float)z3-z1)) / Height;

	const float m_UL_Inv = (((float)u2-u1)) / Height;
	const float m_UR_Inv = (((float)u3-u1)) / Height;

	const float m_VL_Inv = (((float)v2-v1)) / Height;
	const float m_VR_Inv = (((float)v3-v1)) / Height;

	const float m_invZL_Inv = (((float)invZ2-invZ1)) / Height;
	const float m_invZR_Inv = (((float)invZ3-invZ1)) / Height;

	static const int r = (1 << 15); // 0.5 .   

	const int Length = y2 - y1;
	const int mLR = (((int) GT_GET_RED(color2)   - GT_GET_RED(color1)) << 16) / Length;
	const int mLG = (((int) GT_GET_GREEN(color2) - GT_GET_GREEN(color1)) << 16)  / Length;
	const int mLB = (((int) GT_GET_BLUE(color2)  - GT_GET_BLUE(color1)) << 16) / Length;

	const int mRR = (((int) GT_GET_RED(color3)   - GT_GET_RED(color1)) << 16) / Length;
	const int mRG = (((int) GT_GET_GREEN(color3) - GT_GET_GREEN(color1)) << 16)  / Length;
	const int mRB = (((int) GT_GET_BLUE(color3)  - GT_GET_BLUE(color1)) << 16) / Length;

	int LColor = color1;
	int LColorR = GT_GET_RED(color1) << 16;
	int LColorG = GT_GET_GREEN(color1) << 16;
	int LColorB = GT_GET_BLUE(color1) << 16;
	int RColor = LColor;
	int RColorR = LColorR;
	int RColorG = LColorG;
	int RColorB = LColorB;


	int xLeft = x1 << 16,xRight = xLeft;
	float zLeft = z1,zRight = zLeft;
	float uLeft = u1,uRight = uLeft;
	float vLeft = v1,vRight = vLeft;
	float invZLeft = invZ1,invZRight = invZLeft;

	for(int i = y1; i <= y2; i++)
	{
		LColor = GT_RGB((LColorR + r) >> 16,(LColorG + r) >> 16,(LColorB + r) >> 16);
		RColor = GT_RGB((RColorR + r) >> 16,(RColorG + r) >> 16,(RColorB + r) >> 16);

		Line_Horizon_GouraudZ((xLeft + r) >> 16,(xRight + r) >> 16,i,
			                  (zLeft),(zRight),LColor,RColor,uLeft,vLeft,uRight,vRight,invZLeft,invZRight);

		xLeft  += mLeft_Inv;
		xRight += mRight_Inv;

		zLeft  += m_ZL_Inv;
		zRight += m_ZR_Inv;

		invZLeft  += m_invZL_Inv;
		invZRight += m_invZR_Inv;

		uLeft += m_UL_Inv;
		uRight+= m_UR_Inv;

		vLeft += m_VL_Inv;
		vRight+= m_VR_Inv;

		LColorR += mLR;
		LColorG += mLG;
		LColorB += mLB;

		RColorR += mRR;
		RColorG += mRG;
		RColorB += mRB;
	}
}

//   ̵ (ð ݴ  LH)
void GtSoftwareRenderDevice::GouraudShadingTriangleZ(int x1,int y1,float z1,int x2,int y2,float z2,int x3,int y3,float z3,int color1,int color2,int color3,
										   float u1,float v1,float u2,float v2,float u3,float v3,float invZ1,float invZ2,float invZ3)
{
	if(y1 > y2)
	{
		std::swap(y1,y2);
		std::swap(x1,x2);
		std::swap(z1,z2);
		std::swap(color1,color2);
		std::swap(u1,u2);
		std::swap(v1,v2);
		std::swap(invZ1,invZ2);
	}
	if(y1 > y3)
	{
		std::swap(y1,y3);
		std::swap(x1,x3);
		std::swap(z1,z3);
		std::swap(color1,color3);
		std::swap(u1,u3);
		std::swap(v1,v3);
		std::swap(invZ1,invZ3);
	}
	if(y2 > y3)
	{
		std::swap(y2,y3);
		std::swap(x2,x3);
		std::swap(z2,z3);
		std::swap(color2,color3);
		std::swap(u2,u3);
		std::swap(v2,v3);
		std::swap(invZ2,invZ3);
	}

	if(y1 == y2) //   
	{
		GouraudShadingTopTriangleZ(x1,y1,z1,x2,y2,z2,x3,y3,z3,color1,color2,color3,u1,v1,u2,v2,u3,v3,invZ1,invZ2,invZ3);
	}
	else if(y2 == y3) // Ʒ  
	{
		GouraudShadingBottomTriangleZ(x1,y1,z1,x2,y2,z2,x3,y3,z3,color1,color2,color3,u1,v1,u2,v2,u3,v3,invZ1,invZ2,invZ3);
	}
	else // ,Ʒ  äϴ 
	{
		static const int r = (1 << 15); // 0.5 .   
		const int YLength = y2 - y1;
		const int NewX = (((x3-x1) << 16) / (y3-y1) * YLength + (x1 << 16) + r) >> 16;
		const float NewZ = ((((float)z3-z1)) / ((float)y3-y1) * YLength + z1);
		const float NewInvZ = ((((float)invZ3-invZ1)) / ((float)y3-y1) * YLength + invZ1);
		const float NewU = ((((float)u3-u1)) / ((float)y3-y1) * YLength + u1);
		const float NewV = ((((float)v3-v1)) / ((float)y3-y1) * YLength + v1);
		const int ColorLength = (y3 - y1);
		const int mNR = ((((int) GT_GET_RED(color3)   - GT_GET_RED(color1)) << 16)   / ColorLength) * YLength;
		const int mNG = ((((int) GT_GET_GREEN(color3) - GT_GET_GREEN(color1)) << 16) / ColorLength) * YLength;
		const int mNB = ((((int) GT_GET_BLUE(color3)  - GT_GET_BLUE(color1)) << 16)  / ColorLength) * YLength;
		const int NewColor = GT_RGB(((GT_GET_RED(color1) << 16) + mNR + r) >> 16,
									((GT_GET_GREEN(color1) << 16) + mNG + r) >> 16,
									((GT_GET_BLUE(color1)  << 16) + mNB + r) >> 16);

		GouraudShadingBottomTriangleZ(x1,y1,z1,x2,y2,z2,NewX,y2,NewZ,color1,color2,NewColor,u1,v1,u2,v2,NewU,NewV,invZ1,invZ2,NewInvZ);
		GouraudShadingTopTriangleZ(NewX,y2,NewZ,x2,y2,z2,x3,y3,z3,NewColor,color2,color3,NewU,NewV,u2,v2,u3,v3,NewInvZ,invZ2,invZ3);
	}
}
void GtSoftwareRenderDevice::Line_Horizon_GouraudZ(int x1,int x2,int y,float z1,float z2,int color1,int color2,
										 float u1,float v1,float u2,float v2,float invZ1,float invZ2)
{
	if(y < 0 || y >= m_iHeight) return; // ׸ʿ䰡 .

	if(x1 > x2)
	{
		std::swap(x1,x2);
		std::swap(z1,z2);
		std::swap(color1,color2);
		std::swap(u1,u2);
		std::swap(v1,v2);
		std::swap(invZ1,invZ2);
	}

	int Length = x2 - x1;
	int mR,mG,mB;

	float mZ,mU,mV,minvZ;

	if(!Length)
	{
		mR = mG = mB = 0;
		mU = mV = mZ = 0;
		minvZ = 0;
	}
	else
	{
		mR = (((int) GT_GET_RED(color2)   - GT_GET_RED(color1)) << 16) / Length;
		mG = (((int) GT_GET_GREEN(color2) - GT_GET_GREEN(color1)) << 16)  / Length;
		mB = (((int) GT_GET_BLUE(color2)  - GT_GET_BLUE(color1)) << 16) / Length;
		mZ = (((float)z2 - z1)) / Length;
		mU = (((float)u2 - u1)) / Length;
		mV = (((float)v2 - v1)) / Length;
		minvZ = (((float)invZ2 - invZ1)) / Length;
	}

	int red     = (GT_GET_RED(color1) << 16);
	int green   = (GT_GET_GREEN(color1) << 16);
	int blue    = (GT_GET_BLUE(color1) << 16);
	float ZBuff = z1;
	float U     = u1;
	float V     = v1;
	float Zinv = invZ1;
	
	static const int r = (1 << 15); // 0.5 .   

	static int CutX1 = 0;
	// Ŭ
	if(x1 < 0)
	{
		CutX1 = -x1;
		x1 = 0;
		ZBuff += mZ * CutX1;
		red   += mR * CutX1;
		green += mG * CutX1;
		blue  += mB * CutX1;
		U     += mU * CutX1;
		V     += mV * CutX1;
		Zinv += minvZ * CutX1;
	}
	if(x2 >= m_iWidth) x2 = m_iWidth - 1; // ̰ x2 Ŭθ ϸ ȴ ̻ ׸ ʿ  ̴.

	// x2 < 0 x1 >= m_iWidth  ˻ ʾƵ ȴ.
	//   Ʒ for ڵ false ó ޴´.

	int idx;
	for(int i = x1; i <= x2; i++)
	{
		//SetPixel(i,y,color);
		if(WriteZBuffer(i,y,(unsigned int)((ZBuff) * m_uiZBufferSize + 0.5f)))
		{
			idx = y *  m_nPitch + i * 3;

			for(int j = 0; j < TEX_STAGE_NUM; j++)
			{
				if(!m_pTexture[j]) continue;

				int ct = m_pTexture[j]->GetUVPixel((U * (1/Zinv)),(V * (1/Zinv)));
				static const float inv = 1.f/256.f;

				m_pBit[idx]     = (int)((GT_GET_BLUE(ct)  * (((blue + r) >> 16)  * inv)) + 0.5f);
				m_pBit[idx + 1] = (int)((GT_GET_GREEN(ct) * (((green + r) >> 16) * inv)) + 0.5f);
				m_pBit[idx + 2] = (int)((GT_GET_RED(ct)   * (((red + r) >> 16)   * inv)) + 0.5f);
			}
		}

		ZBuff += mZ;
		Zinv  += minvZ;
		red   += mR;
		green += mG;
		blue  += mB;
		U     += mU;
		V     += mV;
	}
}