/************************************************************************/
/* GtSoftwareRenderDevice  				                                */
/*                                                                      */
/*							 Language :                          C/C++  */
/*				  		   Programmer :				   Hwang Dae Woong  */
/*						  Last Update :				        2010.02.16  */
/*					  Program Version :							 1.00a  */
/*                                                                      */
/*         Copyright (c) Hwang Dae Woong. All rights reserved.          */
/************************************************************************/
//////////////////////////////////////////////////////////////////////////
#pragma once

#include "GtFVF.h"
#include "GtSoftwareRender_Type.h"
#include "GtFrustum.h"
#include "GtLight.h"
#include "GtMaterial.h"
#include "GtTexture.h"
#include <windows.h>
#include <map>

//////////////////////////////////////////////////////////////////////////
/** 
@class	  GtSoftwareRenderDevice
@brief	  RenderDevice̴. Direct3D ̽ ſ  ϰ ۼ Ͽ.
@date     2010-02-16
@author   ghostkyow@naver.com 
**/
//////////////////////////////////////////////////////////////////////////
class GtSoftwareRenderDevice
{
public:
	typedef std::map<int, GtLightSet> mapLightSet;
private:
	HWND			m_hWnd;
	HDC				m_hMainBuffer;
	HDC				m_hBackBuffer;
	HBITMAP			m_hBitmap;
	HBITMAP			m_hDefaultBitmap;
	BYTE*			m_pBit;
	int				m_iWidth;
	int				m_iHeight;
	int				m_nPitch;

private:
	GT_RenderPipeLine	m_RenderPipeLine;			// Rendering Pipe Line  ִ ü
	GtFVF				m_FVF;						// Flexible vertex format bits : FVF 
	sGtRenderStateFlag	m_RenderStateFlag;			// RenderState Flag
	GtRenderObj			m_RenderObj;				// Render Obj
	GtFrustum			m_Frustum;					// ü
	
	// Light
	mapLightSet			m_Light;
	GtMaterial			m_Material;
	unsigned long		m_ulGlobal_Ambient;
	
	// ؽ
	LPGT_TEXTURE		m_pTexture[TEX_STAGE_NUM];		

	// ZBuffer
	GTZBUFFERBIT		m_ZBufferState;
	unsigned int		m_uiZBufferSize;
	unsigned short		*m_pZBuffer16;
	unsigned int		*m_pZBuffer32;
public:
	GtSoftwareRenderDevice(void);
	~GtSoftwareRenderDevice(void);

public:
	void			Create(HWND hWnd, int iWidth, int iHeight);
	void			Destroy(void);
	void			Swap(void);
	bool			Clear(unsigned long ulCount, const GtRect* pRects, unsigned long ulFlags, unsigned int uiColor, float fZ, unsigned long ulStencil); 

	void			Update(void);
	HBITMAP			CreateBitmap(HDC hDC, int iWidth, int iHeight, BYTE** ppBit);
	void			DrawFPS(void);

public:
	void			SetPixel(int x, int y, int color);
	void			DrawLine(int x0, int y0, int x1, int y1, int color);	// Breseham 
	bool			ClipLine(int& x0, int& y0, int& x1, int &y1, int xmin, int xmax, int ymin, int ymax);	//	Cohen Sutherland 
	void			Line_Horizon(int x1, int x2, int y, int color);		// ܻ
	void			Line_Vertical(int x, int y1, int y2, int color);	// ܻ

public:
	void			Line_Horizon_Gouraud(int x1,int x2,int y,int color1,int color2); //
	void			GouraudShadingTopTriangle(int x1,int y1,int x2,int y2,int x3,int y3,int color1,int color2,int color3);
	void			GouraudShadingBottomTriangle(int x1,int y1,int x2,int y2,int x3,int y3,int color1,int color2,int color3);
	void			GouraudShadingTriangle(int x1,int y1,int x2,int y2,int x3,int y3,int color1,int color2,int color3);
public:
	//  ̵ Z  
	void			Line_Horizon_GouraudZ(int x1,int x2,int y,float z1,float z2,int color1,int color2,float u1,float v1,float u2,float v2,float invZ1,float invZ2);
	void			GouraudShadingTopTriangleZ(int x1,int y1,float z1,int x2,int y2,float z2,int x3,int y3,float z3,int color1,int color2,int color3,float u1,float v1,float u2,float v2,float u3,float v3,float invZ1,float invZ2,float invZ3);
	void			GouraudShadingBottomTriangleZ(int x1,int y1,float z1,int x2,int y2,float z2,int x3,int y3,float z3,int color1,int color2,int color3,float u1,float v1,float u2,float v2,float u3,float v3,float invZ1,float invZ2,float invZ3);
	void			GouraudShadingTriangleZ(int x1,int y1,float z1,int x2,int y2,float z2,int x3,int y3,float z3,int color1,int color2,int color3,float u1,float v1,float u2,float v2,float u3,float v3,float invZ1,float invZ2,float invZ3);
public:
	// , ,   ޾ƿ 
	void			SetTransform(const GT_RENDERING_PIPELINE eState, const GtMatrix4& mat);
	void			DrawPrimitive(GT_PRIMITIVETYPE ePrimitiveState, unsigned int iStartVertex, unsigned int iPrimitiveCount);
	void			DrawPrimitiveUp(GT_PRIMITIVETYPE ePrimitiveState, unsigned int iPrimivieCount, const void* pVtxStreamZeroData, unsigned int iVtxStreamZeroStride);
	// Ʈ   ´.   Ʈ    ü̴.
	bool			SetViewPort(const sGT_ViewPort& pViewPort);
	bool			GetViewPort(sGT_ViewPort& pOut);

	// Ŭ  .
	bool			CreateMatrixClipping(const GtClipping& pClip);

	// ĸ
	bool			BackFaceCullingLH(const GtPoint3f& p1, const GtPoint3f& p2, const GtPoint3f& p3);
	bool			BackFaceCullingRH(const GtPoint3f& p1, const GtPoint3f& p2, const GtPoint3f& p3);

	// Rendering PiPeLine 
	bool			RenderingPipeLine(void);
	bool			Local_To_World_Object(void);		//  
	bool			World_To_Camera_Object(void);		// 忡 ī޶
	bool			Camera_To_Proj_Object(void);		// ī޶󿡼 
	bool			Proj_Object_To_Clipping(void);		//  Ŭ
	bool			Clipping_To_ViewPort(void);			//  Ʈ ȯ(ȭ )
	bool			Fill_Rasterize(void);				//  ȭ  ﰢ ä.

	// Light 
	void			SetLight(unsigned long ulIndex, const GtLight& in);
	bool			LightEnable(unsigned long ulIndex, bool bEnable);

	//  (Material)
	void			SetMaterial(const GtMaterial& in);
	void			SetMaterialColor(GtColorValue& Ambient, GtColorValue& Diffuse, GtColorValue& Specular);
	const int		CalcLighting(GtPoint3f pt);


	// RenderState 
	bool			SetRenderState(GT_RENDERSTATE	eState, unsigned long ulValue);
	bool			GetRenderState(GT_RENDERSTATE	eState, unsigned long *pValue);


	// FVF
	bool			SetFVF(unsigned long ulFVF);
	bool			GetFVF(unsigned long *ulFVF);

	// Vertex 
	bool			SetStreamSource(unsigned int uiStreamNumber, GtVertexBuffer* pStreamData,
									unsigned int uiOffsetInBytes, unsigned int uiStride);
	bool			CreateVertexBuffer(unsigned int uiLength, unsigned long ulUsage, unsigned long ulFVF, GT_POOL ePool, 
										GtVertexBuffer** ppVertexBuffer, void ** pSharedHandle = NULL);
	// Texture 
	bool			CreateTextureFromFile(const char* pName, LPGT_TEXTURE* ppTexture);
	bool			SetTexture(unsigned long ulStage, LPGT_TEXTURE pTexture);

	// ZBuffer
	void			CreateZBuffer(GTZBUFFERBIT eZBufferBit);
	bool			WriteZBuffer(int x, int y, unsigned int uiZValue);

};