#include "GtTerrain.h"
#include "GtQuadTree.h"
#include "GtD3DRenderer.h"

void sGtHeightMap::SetHeightMap(int iNumVertsPerRow, int iNumVertsPerCol, GtPoint3f vCellSpacing, float fHeightScale)
{
	m_iNumVertsPerRow = iNumVertsPerRow;
	m_iNumVertsPerCol = iNumVertsPerCol;
	m_vCellSpacing.x = vCellSpacing.x;
	m_vCellSpacing.y = vCellSpacing.y;
	m_vCellSpacing.z = vCellSpacing.z;

	m_iNumCellsPerRow = iNumVertsPerRow - 1;
	m_iNumCellsPerCol = iNumVertsPerCol - 1;
	m_iWidth = int(iNumVertsPerRow * vCellSpacing.x);
	m_iDepth = int(iNumVertsPerCol * vCellSpacing.z);
	m_iNumVertices = iNumVertsPerRow * iNumVertsPerCol;
	m_iNumTriangles = m_iNumCellsPerRow * m_iNumCellsPerCol * 2;
}

GtTerrain::GtTerrain(void) : GtNode()
{
//	m_iNumTex				= 0;
//	m_iBaseTex				= 0;
//	m_iSplateIndex			= 0;
//	m_fBaseTexScale			= 10.0f;
	m_pVBPicking				= NULL;
	m_pVBMap					= NULL;
	m_pIBMap					= NULL;
	m_pQuadTree					= NULL;
	m_pVMap						= NULL;
	m_pBaseTexture				= NULL;
	m_pPicking					= NULL;
	m_eFillMode					= D3DFILL_SOLID;
	m_sParameter.m_iBigCircle	= 100;
	m_sParameter.m_iSmallCircle = 50;
	m_bMapSize					= false;
}

GtTerrain::GtTerrain(const GtString& strName) : GtNode(strName)
{
	m_pVBPicking				= NULL;
	m_pVBMap					= NULL;
	m_pIBMap					= NULL;
	m_pQuadTree					= NULL;
	m_pVMap						= NULL;
	m_pBaseTexture				= NULL;
	m_pPicking					= NULL;
	m_eFillMode					= D3DFILL_SOLID;
	m_sParameter.m_iBigCircle	= 100;
	m_sParameter.m_iSmallCircle = 50;
	m_bMapSize					= false;
}

GtTerrain::~GtTerrain(void)
{
	m_pDev = NULL;
	m_pPicking = NULL;

	std::vector<LPGT_TERRAINTEXTURE> ::iterator it;
	for(it = m_vTerrainTextureTable.begin(); it != m_vTerrainTextureTable.end(); )
	{
		if(*it)
		{
			delete *it;
			it = m_vTerrainTextureTable.erase(it);
			continue;
		}
		it++;
	}
	
	for(it = m_vAlphaMapTable.begin(); it != m_vAlphaMapTable.end(); )
	{
		if(*it)
		{
			delete *it;
			it = m_vAlphaMapTable.erase(it);
			continue;
		}
		it++;
	}

	std::vector<sGT_TERRAINTEXTUREINFO>	::iterator it2;
	for(it2 = m_vSplatingTextureInfo.begin(); it2 != m_vSplatingTextureInfo.end();)
	{
		it2 = m_vSplatingTextureInfo.erase(it2);
		continue;
	}


	if(m_pVBPicking)	m_pVBPicking->Release();
	if(m_pVBMap)		m_pVBMap->Release();
	if(m_pIBMap)		m_pIBMap->Release();
	if(m_pVMap)			delete [] m_pVMap;
	if(m_pQuadTree)		delete m_pQuadTree;
	if(m_pBaseTexture)	delete m_pBaseTexture;
	m_pVMap = NULL;
	m_pQuadTree = NULL;
	m_pVBPicking = NULL;
	m_pIBMap = NULL;
	m_pBaseTexture = NULL;
}

//------------------------------------------------------------------------
// Լ  : void SetDevice(LPDIRECT3DDEVICE9 pDev)
//     : ̽ 
//     : m_pDev : IndexBuffer  ﰢ ù 
//     : ڷ  ̽  ο   ֵ 
//------------------------------------------------------------------------
void GtTerrain::SetDevice(LPDIRECT3DDEVICE9 pDev)
{
	m_pDev = pDev;
}

//------------------------------------------------------------------------
// Լ  : bool CreateHeightMap(int iWidth, int iHeight, int iCellSpacing)
//     : ̸ 
//     : m_pDev : IndexBuffer  ﰢ ù 
//     : ڷ  ̽  ο   ֵ 
//------------------------------------------------------------------------
bool GtTerrain::CreateHeightMap(int iWidth, int iHeight, int iCellSpacing)
{
	//̸
	m_sMapHeader.SetHeightMap(iWidth, iHeight, GtPoint3f(float(iCellSpacing), float(iCellSpacing), float(iCellSpacing)), 10.0f);

	//  
	if( !CreateVBMap() )
		return false;

	//ε  
	if(m_pIBMap)
		m_pIBMap->Release();
    if( FAILED(m_pDev->CreateIndexBuffer(m_sMapHeader.m_iNumTriangles * sizeof(sGT_MAPINDEX), 0, D3DFMT_INDEX32, D3DPOOL_DEFAULT, &m_pIBMap, NULL)) )
    	return false;

	// ŷ  
	if(!SetPickCircle(true))
		return false;

	return true;
}

//------------------------------------------------------------------------
// Լ  : bool LoadHeightMap(char* pFileName)
//     : ̸  ͷ  ε
//     : pFileName  ̸
//     : ̸   Ǿִ ͷκ ε
//------------------------------------------------------------------------
bool GtTerrain::LoadHeightMap(char* pFileName)
{
	FILE *fp = fopen(pFileName, "rb");

	int iWidth, iHeight;
	GtPoint3f vCellSpacing;
	fread(&iWidth, sizeof(int), 1, fp);
	fread(&iHeight, sizeof(int), 1, fp);
	fread(&vCellSpacing, sizeof(GtPoint3f), 1, fp);
	m_sMapHeader.SetHeightMap(iWidth, iHeight, vCellSpacing, 10.0f);

	int iVertexSize = m_sMapHeader.m_iNumVertices * sizeof(sGT_CUSTOMVERTEX);

	//   
	if(m_pVBMap)
		m_pVBMap->Release();
	if(FAILED(m_pDev->CreateVertexBuffer(iVertexSize, 0, D3DFVF_TERRAIN, D3DPOOL_DEFAULT, &m_pVBMap, NULL)) )
		return false;

	//     ̺ 
	if(m_pVMap)
		delete [] m_pVMap;
	m_pVMap = new sGT_CUSTOMVERTEX[m_sMapHeader.m_iNumVertices];
	LPGT_CUSTOMVERTEX vP = m_pVMap;
//	fread(m_pVMap, m_sMapHeader.m_iNumVertices * sizeof(sGT_CUSTOMVERTEX), 1, fp);

	// ۱
	sGT_CUSTOMVERTEX v;
	LPGT_CUSTOMVERTEX pVertex;
	if(FAILED(m_pVBMap->Lock(0, iVertexSize, (void**)&pVertex, 0)))
		return false;

	for(int iLoopZ = 0; iLoopZ < m_sMapHeader.m_iNumVertsPerCol; ++iLoopZ)
	{
		for(int iLoopX = 0; iLoopX < m_sMapHeader.m_iNumVertsPerRow; ++iLoopX)
		{
			fread(&v, sizeof(sGT_CUSTOMVERTEX), 1, fp);
			// õ   ۿ 
			*pVertex ++ = v;
			*vP++ = v;
		}
	}

	m_pVBMap->Unlock();

	fclose(fp);

	//̸ʻüũ(2^n+1϶ Ʈ )
	m_bMapSize = (CheckMapSize(m_sMapHeader.m_iNumVertsPerRow) && CheckMapSize(m_sMapHeader.m_iNumVertsPerCol) );

	// ʻ  ɶ  Ʈ 
	if(m_bMapSize)
	{
		if(m_pQuadTree) delete m_pQuadTree;
		m_pQuadTree = new GtQuadTree(m_sMapHeader.m_iNumVertsPerRow, m_sMapHeader.m_iNumVertsPerCol);
		m_pQuadTree->Build(m_pVMap);
	}

	//ε  
	if(m_pIBMap)
		m_pIBMap->Release();
    if( FAILED(m_pDev->CreateIndexBuffer(m_sMapHeader.m_iNumTriangles * sizeof(sGT_MAPINDEX), 0, D3DFMT_INDEX32, D3DPOOL_DEFAULT, &m_pIBMap, NULL)) )
    	return false;

	return true;
}

//------------------------------------------------------------------------
// Լ  : bool CreateVBMap(void)
//     : ε   쿡 ο ؽ ۸ 
//     : ̸ʿ   Ͱ ÿ  ϱ  Լ
//------------------------------------------------------------------------
bool GtTerrain::CreateVBMap(void)
{
	int iVertexSize = m_sMapHeader.m_iNumVertices * sizeof(sGT_CUSTOMVERTEX);

	//   
	if(m_pVBMap)
		m_pVBMap->Release();
	if(FAILED(m_pDev->CreateVertexBuffer(iVertexSize, 0, D3DFVF_TERRAIN, D3DPOOL_DEFAULT, &m_pVBMap, NULL)) )
		return false;

	//     ̺ 
	if(m_pVMap)
		delete [] m_pVMap;
	m_pVMap = new sGT_CUSTOMVERTEX[m_sMapHeader.m_iNumVertices];
	LPGT_CUSTOMVERTEX vP = m_pVMap;

	// ۱
	sGT_CUSTOMVERTEX v;
	LPGT_CUSTOMVERTEX pVertex;
	if(FAILED(m_pVBMap->Lock(0, iVertexSize, (void**)&pVertex, 0)))
		return false;

	//  () *  ؽ ŭ ݺ ϸ鼭   
	for(int iLoopZ = 0; iLoopZ < m_sMapHeader.m_iNumVertsPerCol; ++iLoopZ)
	{
		for(int iLoopX = 0; iLoopX < m_sMapHeader.m_iNumVertsPerRow; ++iLoopX)
		{
			// x z  ǥ ߽ ̵, y ؽ       ϹǷ 0.0f
			v.m_vPos.x =  ((iLoopX) - (m_sMapHeader.m_iNumVertsPerRow) / 2.0f) * m_sMapHeader.m_vCellSpacing.x;
			v.m_vPos.z = -((iLoopZ) - (m_sMapHeader.m_iNumVertsPerCol) / 2.0f) * m_sMapHeader.m_vCellSpacing.z;
			v.m_vPos.y = 0.0f;

			//   
			D3DXVec3Normalize( &v.m_vNormal, &v.m_vPos );

			//  uv 
			v.u = iLoopX / (m_sMapHeader.m_iNumVertsPerRow - 1.0f);
			v.v = iLoopZ / (m_sMapHeader.m_iNumVertsPerCol - 1.0f);

			// õ   ۿ 
			*pVertex ++ = v;
			*vP++ = v;
		}
	}
	m_pVBMap->Unlock();

	//̸ʻüũ(2^n+1϶ Ʈ )
	m_bMapSize = (CheckMapSize(m_sMapHeader.m_iNumVertsPerRow) && CheckMapSize(m_sMapHeader.m_iNumVertsPerCol) );

	// ʻ  ɶ  Ʈ 
	if(m_bMapSize)
	{
		if(m_pQuadTree) delete m_pQuadTree;
		m_pQuadTree = new GtQuadTree(m_sMapHeader.m_iNumVertsPerRow, m_sMapHeader.m_iNumVertsPerCol);
		m_pQuadTree->Build(m_pVMap);
	}
	return true;
}

//------------------------------------------------------------------------
// Լ  : bool SetVBMap(sGT_MAPINDEX* pIndexBuffer)
//     :  ؽ ۸  
// 	   : pIndexBuffer  ε  ̺
//     : ڷ  ε  ͸   ѵ
//			    
//------------------------------------------------------------------------
bool GtTerrain::SetVBMap(sGT_MAPINDEX* pIndexBuffer)
{
	//ֺ 
	D3DXVECTOR3 vA, vB, vC;
	for(int iLoop=0; iLoop<m_iTriangles; ++iLoop)
	{
		vA = m_pVMap[ pIndexBuffer[iLoop]._1 ].m_vPos - m_pVMap[ pIndexBuffer[iLoop]._0 ].m_vPos;
		vB = m_pVMap[ pIndexBuffer[iLoop]._2 ].m_vPos - m_pVMap[ pIndexBuffer[iLoop]._0 ].m_vPos;
		D3DXVec3Cross(&vC, &vA, &vB);		D3DXVec3Normalize( &vC, &vC );
		m_pVMap[ pIndexBuffer[iLoop]._0 ].m_vNormal += vC;
		m_pVMap[ pIndexBuffer[iLoop]._1 ].m_vNormal += vC;
		m_pVMap[ pIndexBuffer[iLoop]._2 ].m_vNormal += vC;
	}

	//ֺ ȭ
	for(int iLoop=0; iLoop<m_iTriangles; ++iLoop)
	{
		D3DXVec3Normalize( &m_pVMap[pIndexBuffer[iLoop]._0].m_vNormal, &m_pVMap[pIndexBuffer[iLoop]._0].m_vNormal );
		D3DXVec3Normalize( &m_pVMap[pIndexBuffer[iLoop]._1].m_vNormal, &m_pVMap[pIndexBuffer[iLoop]._1].m_vNormal );
		D3DXVec3Normalize( &m_pVMap[pIndexBuffer[iLoop]._2].m_vNormal, &m_pVMap[pIndexBuffer[iLoop]._2].m_vNormal );
	}

	//  VB ؽ  
	LPGT_CUSTOMVERTEX	pVertex;
	if( FAILED(m_pVBMap->Lock( 0, m_sMapHeader.m_iNumVertices * sizeof(LPGT_CUSTOMVERTEX), (void**)&pVertex, 0)) )
		return false;

	memcpy(pVertex, m_pVMap, sizeof(sGT_CUSTOMVERTEX) * m_sMapHeader.m_iNumVertices);

	m_pVBMap->Unlock();

	return true;
}

//------------------------------------------------------------------------
// Լ  : void ControlHeightMapUpAndDown(void)
//     :  ؽ ۸ ŷ    øų ų ϱ ؼ
//     :   ִ m_pPicking  ŷ ġκ  ؽ ̸ 
//------------------------------------------------------------------------
void GtTerrain::ControlHeightMapUpAndDown(void)
{
	int iTemp, iSRow, iERow, iSCol, iECol;

	// TerrainControl   value .
	// value   -   Ʒ +  
	float value = 0.0f;
	if(m_sParameter.m_eTerrainControlState == eTS_UP)
		value = 1.0f;
	else if(m_sParameter.m_eTerrainControlState == eTS_DOWN)
		value = -1.0f;
	else if(m_sParameter.m_eTerrainControlState == eTS_NONE)
		return;

	//ū  :    ŷã
	GetHeightMapIndex(m_pPicking->m_vPickPos.x - m_sParameter.m_iBigCircle, m_pPicking->m_vPickPos.z, &iSRow, &iTemp);
	GetHeightMapIndex(m_pPicking->m_vPickPos.x + m_sParameter.m_iBigCircle, m_pPicking->m_vPickPos.z, &iERow, &iTemp);
	GetHeightMapIndex(m_pPicking->m_vPickPos.x, m_pPicking->m_vPickPos.z + m_sParameter.m_iBigCircle, &iTemp, &iSCol);
	GetHeightMapIndex(m_pPicking->m_vPickPos.x, m_pPicking->m_vPickPos.z - m_sParameter.m_iBigCircle, &iTemp, &iECol);

	//̰
	for(int iLoopZ=iSCol; iLoopZ<=iECol; ++iLoopZ)
	{
		for(int iLoopX=iSRow; iLoopX<=iERow; ++iLoopX)
		{
			if(m_sParameter.m_eTerrainControlState != eTS_FLATLAND)
				SetHeightMapY(iLoopX, iLoopZ, value * m_sParameter.m_fPower * 5.0f);
			else
				SetHeightMapY(iLoopX, iLoopZ, 0.0f);
		}
	}

	//  :    ŷã
	GetHeightMapIndex(m_pPicking->m_vPickPos.x - m_sParameter.m_iSmallCircle, m_pPicking->m_vPickPos.z, &iSRow, &iTemp);
	GetHeightMapIndex(m_pPicking->m_vPickPos.x + m_sParameter.m_iSmallCircle, m_pPicking->m_vPickPos.z, &iERow, &iTemp);
	GetHeightMapIndex(m_pPicking->m_vPickPos.x, m_pPicking->m_vPickPos.z + m_sParameter.m_iSmallCircle, &iTemp, &iSCol);
	GetHeightMapIndex(m_pPicking->m_vPickPos.x, m_pPicking->m_vPickPos.z - m_sParameter.m_iSmallCircle, &iTemp, &iECol);

	//̰
	for(int iLoopZ=iSCol; iLoopZ <= iECol; ++iLoopZ)
	{
		for(int iLoopX=iSRow; iLoopX <= iERow; ++iLoopX)
		{
			if(m_sParameter.m_eTerrainControlState != eTS_FLATLAND)
				SetHeightMapY(iLoopX, iLoopZ, value * m_sParameter.m_fPower * 15.0f);
			else
				SetHeightMapY(iLoopX, iLoopZ, 0.0f);
		}
	}
}

//------------------------------------------------------------------------
// Լ  : void SetFillMode(D3DFILLMODE eFillMode)
//     : ʸ 
// 	   : eFillMode : Mode enum 
//     : ʸ带 Wire  Solid   
//------------------------------------------------------------------------
void GtTerrain::SetFillMode(D3DFILLMODE eFillMode)
{
	m_eFillMode = eFillMode;
}

//------------------------------------------------------------------------
// Լ  : LPGT_CUSTOMVERTEX GetVertexMap(void)
//     : ؽ  
//------------------------------------------------------------------------
LPGT_CUSTOMVERTEX GtTerrain::GetVertexMap(void)
{
	return m_pVMap;
}

//------------------------------------------------------------------------
// Լ  : sGT_HeightMap* GetHeigtMapHeader(void)
//     : ̸ () 
//------------------------------------------------------------------------
sGT_HeightMap* GtTerrain::GetHeigtMapHeader(void)
{
	return &m_sMapHeader;
}

//------------------------------------------------------------------------
// Լ  : bool LoadTexture(char* pFileName)
//     : ؽ ε
// 	   : pFileName :  ̸
//     : ڷ  ؽ ̸ ؽĸ ε
//------------------------------------------------------------------------
bool GtTerrain::LoadTexture(wchar_t* pFileName)
{
	if(m_pBaseTexture)
		delete m_pBaseTexture;
	m_pBaseTexture = new sGT_TERRAINTEXTURE;
	if( FAILED( D3DXCreateTextureFromFile(m_pDev, pFileName, &m_pBaseTexture->m_pTexture)))
		return false;
	return true;
}

//------------------------------------------------------------------------
// Լ  : bool CreateAlphaMaps(void)
//     : ĸ 
//     : ĸ  ؽĸ . ( )
//------------------------------------------------------------------------
bool GtTerrain::CreateAlphaMaps(void)
{
	if(m_pAlphaMap)
		delete m_pAlphaMap;
	if(!m_pAlphaMap)
		m_pAlphaMap = new sGT_TERRAINTEXTURE;
	LPBYTE pData;	LPDWORD pDWord;
	D3DLOCKED_RECT AlphaMap_Locked;
	if( m_pAlphaMap->m_pTexture )
		m_pAlphaMap->m_pTexture->Release();
	if( FAILED(m_pDev->CreateTexture( 1024, 1024, 1, 0, D3DFMT_A8R8G8B8, D3DPOOL_MANAGED, &m_pAlphaMap->m_pTexture, NULL )) )
		return false;

	memset( &AlphaMap_Locked, 0, sizeof(D3DLOCKED_RECT) );
	if( FAILED(m_pAlphaMap->m_pTexture->LockRect(0, &AlphaMap_Locked, NULL, 0)) )
		return false;

	pData = (LPBYTE)AlphaMap_Locked.pBits;
	for(int iLoopY=0; iLoopY<1024; ++iLoopY)
	{
		pDWord = LPDWORD(pData + iLoopY * AlphaMap_Locked.Pitch);
		for(int iLoopX=0; iLoopX<1024; ++iLoopX)
			*(pDWord + iLoopX) = 0x00000000;
	}

	if( FAILED(m_pAlphaMap->m_pTexture->UnlockRect(0)) )	
		return false;
	return true;
}

//------------------------------------------------------------------------
// Լ  : bool AddTexture(wchar_t *pFileName)
//     : ؽ ̺ ߰
// 	   : pFileName : ؽ  
//------------------------------------------------------------------------
bool GtTerrain::AddTexture(wchar_t *pFileName)
{
	LPGT_TERRAINTEXTURE pTexture;
	pTexture = new sGT_TERRAINTEXTURE;
	if( FAILED( D3DXCreateTextureFromFile(m_pDev, pFileName, &pTexture->m_pTexture)))
		return false;
	m_vTerrainTextureTable.push_back(pTexture);
//	m_iCurrentTerrainTexIdx = m_vTerrainTextureTable.size() - 1;
	m_sTerrainTextureInfo.m_iTextureIndex = m_vTerrainTextureTable.size() - 1;
	return true;
}

//------------------------------------------------------------------------
// Լ  : bool RemoveTexture(int iTextureIndex)
//     : ؽ ̺ ִ ؽ ε 
// 	   : iTextureIndex : ؽ ̺ ε
//------------------------------------------------------------------------
bool GtTerrain::RemoveTexture(int iTextureIndex)
{
	std::vector<LPGT_TERRAINTEXTURE> ::iterator it;
	int n = 0;
	for(it = m_vTerrainTextureTable.begin(); it != m_vTerrainTextureTable.end(); )
	{
		if(n == iTextureIndex)
		{
			if(*it)
				delete *it;
			m_vTerrainTextureTable.erase(it);
			if(m_vTerrainTextureTable.size() <= m_sTerrainTextureInfo.m_iTextureIndex)
				m_sTerrainTextureInfo.m_iTextureIndex -= 1;
			return true;
		}
		it++;
		n++;
	}
	return false;
}

//------------------------------------------------------------------------
// Լ  : bool SetCurrentTerrainTexIndex(int iCurrentTerrainTexIndex)
//     : ؽ ̺ ε 
// 	   : iCurrentTerrainTexIndex : ؽ ̺ ε
//------------------------------------------------------------------------
bool GtTerrain::SetCurrentTerrainTexIndex(int iCurrentTerrainTexIndex)
{
	if(iCurrentTerrainTexIndex >= m_vTerrainTextureTable.size())
		return false;
	m_sTerrainTextureInfo.m_iTextureIndex = iCurrentTerrainTexIndex;
	return true;
}

//------------------------------------------------------------------------
// Լ  : int GetCurrentTerrainTexIndex(void)
//     : ؽ ̺ ε 
//------------------------------------------------------------------------
int GtTerrain::GetCurrentTerrainTexIndex(void)
{
	return m_sTerrainTextureInfo.m_iTextureIndex;
}

//------------------------------------------------------------------------
// Լ  : bool SetTerrainTextureScale(int iTerrainTexIndex, float fScale)
//     :  ؽ  
//------------------------------------------------------------------------
bool GtTerrain::SetTerrainTextureScale(float fScale)
{
	m_sTerrainTextureInfo.m_fScale = fScale;

	return true;
}

//------------------------------------------------------------------------
// Լ  : float GetTerrainTextureScale(int iTerrainTexIndex)
//     :  ؽ  
//------------------------------------------------------------------------
float GtTerrain::GetTerrainTextureScale(void)
{
	return m_sTerrainTextureInfo.m_fScale;
}

//------------------------------------------------------------------------
// Լ  : bool AddAlphaMap(void)
//     : ĸ  ̺ ߰
//------------------------------------------------------------------------
bool GtTerrain::AddAlphaMap(void)
{	
	LPGT_TERRAINTEXTURE pAlphaMap;
	pAlphaMap = new sGT_TERRAINTEXTURE;

	LPBYTE pData;	LPDWORD pDWord;
	D3DLOCKED_RECT AlphaMap_Locked;
	if( pAlphaMap->m_pTexture )
		pAlphaMap->m_pTexture->Release();
	if( FAILED(m_pDev->CreateTexture( 1024, 1024, 1, 0, D3DFMT_A8R8G8B8, D3DPOOL_MANAGED, &pAlphaMap->m_pTexture, NULL )) )
		return false;

	memset( &AlphaMap_Locked, 0, sizeof(D3DLOCKED_RECT) );
	if( FAILED(pAlphaMap->m_pTexture->LockRect(0, &AlphaMap_Locked, NULL, 0)) )
		return false;

	pData = (LPBYTE)AlphaMap_Locked.pBits;
	for(int iLoopY=0; iLoopY<1024; ++iLoopY)
	{
		pDWord = LPDWORD(pData + iLoopY * AlphaMap_Locked.Pitch);
		for(int iLoopX=0; iLoopX<1024; ++iLoopX)
			*(pDWord + iLoopX) = 0x00000000;
	}

	if( FAILED(pAlphaMap->m_pTexture->UnlockRect(0)) )	
		return false;

	m_vAlphaMapTable.push_back(pAlphaMap);
	return true;
}

//------------------------------------------------------------------------
// Լ  : bool RemoveAlphaMap(int iAlphaMapIndex)
//     : ĸ 
// 	   : iAlphaMapIndex :  ϴ ε
//------------------------------------------------------------------------
bool GtTerrain::RemoveAlphaMap(int iAlphaMapIndex)
{
	if(iAlphaMapIndex >= m_vAlphaMapTable.size())
		return false;
	int n = 0;
	std::vector<LPGT_TERRAINTEXTURE> ::iterator it;
	for(it = m_vAlphaMapTable.begin(); it != m_vAlphaMapTable.end();)
	{
		if(n == iAlphaMapIndex)
		{
			if(*it)
			{
				delete *it;
				it = m_vAlphaMapTable.erase(it);
				return true;
			}
		}
		n++;
		it++;
	}
	return true;
}

//------------------------------------------------------------------------
// Լ  : bool DrawAlphaMap(GtPicking *pPicking, int iIndex)
//     : ĸ 
// 	   : pPicking : ŷ ü , iIndex : ؽ ε
//     : pPicking iIndex  ؽ ĸ ϴ 
//------------------------------------------------------------------------
bool GtTerrain::DrawAlphaMap(GtPicking *pPicking, int iIndex)
{
	if(iIndex == -1 || iIndex >= m_vAlphaMapTable.size())
		return false;
	int iRow, iCol;
	GetHeightMap(pPicking->m_vPickPos.x, pPicking->m_vPickPos.y);

	float fPixSize = (float)m_sMapHeader.m_iWidth / 1024.0f;

	// 귯 
	float fHalfBrushSize = float(m_sParameter.m_iBigCircle);
	float fHalfSmoothSize = m_sParameter.m_fSmoothSize;

	float fEditSize = fHalfBrushSize / fPixSize;

	float fPosU = float(pPicking->m_vPickPos.x + m_sMapHeader.m_iWidth/2.0f) / float(m_sMapHeader.m_iWidth);
	float fPosV = 1.0f - float(pPicking->m_vPickPos.z + m_sMapHeader.m_iDepth/2.0f) / float(m_sMapHeader.m_iDepth);

	m_iAlphaMapX = int(fPosU * 1024.0f + (m_sMapHeader.m_iWidth * 0.0004f));
	m_iAlphaMapY = int(fPosV * 1024.0f + (m_sMapHeader.m_iDepth * 0.0004f));

	// 귯 
	int iStartPosX = int( ((m_iAlphaMapX - fEditSize) < 0) ? 0 : m_iAlphaMapX - fEditSize);
	int iStartPosY = int( ((m_iAlphaMapY - fEditSize) < 0) ? 0 : m_iAlphaMapY - fEditSize);
	int iEndPosX   = int( ((m_iAlphaMapX + fEditSize) >= 1024) ? 1024 - 1: m_iAlphaMapX + fEditSize);
	int iEndPosY   = int( ((m_iAlphaMapY + fEditSize) >= 1024) ? 1024 - 1: m_iAlphaMapY + fEditSize);

	float fVecLength, fSmooth;
	DWORD dwColor, dwChangeColor;
	D3DLOCKED_RECT rcAlphaMapLocked;
	memset( &rcAlphaMapLocked, 0, sizeof(D3DLOCKED_RECT) );

	// ؽ Ʈ  
	if(FAILED(m_vAlphaMapTable[iIndex]->m_pTexture->LockRect(0, &rcAlphaMapLocked, NULL, 0)))
		return false;

	// ŷ ġ   Ʈ 
	LPBYTE pDst = LPBYTE(rcAlphaMapLocked.pBits);
	for(int iLoopZ = iStartPosY; iLoopZ <= iEndPosY; ++iLoopZ)
	{
		LPDWORD dwDst = LPDWORD(pDst + iLoopZ * rcAlphaMapLocked.Pitch);
		for(int iLoopX = iStartPosX; iLoopX <= iEndPosX; ++iLoopX)
		{
			D3DXVECTOR3 v = D3DXVECTOR3(iLoopX * fPixSize, 0.0f, iLoopZ * fPixSize)
				- D3DXVECTOR3(m_iAlphaMapX * fPixSize, 0.0f, m_iAlphaMapY * fPixSize);

			fVecLength = float(D3DXVec3Length(&v));

			if(m_sParameter.m_iSplateControl)
			{
				if(fVecLength <= fHalfSmoothSize)
				{
					dwChangeColor = 0xffffffff;		// A : ff , G : ff , B : ff , A : ff
				}
				else if(fVecLength <= fHalfBrushSize)
				{
					fVecLength -= fHalfSmoothSize;
					fSmooth = fHalfBrushSize - fHalfSmoothSize;
					dwChangeColor = DWORD((fSmooth - fVecLength) / fSmooth * 0xff000000);
					dwColor = (dwChangeColor & 0xff000000) >> 24;
					dwChangeColor = (dwColor << 24) | (dwColor << 16) | (dwColor << 8) | dwColor;
				}
				else
					continue;

				dwDst[iLoopX] = (dwDst[iLoopX] < dwChangeColor ) ? dwChangeColor : dwDst[iLoopX];
			}
			else
			{
				if(fHalfSmoothSize >= fVecLength || fVecLength <= fHalfBrushSize)
					dwDst[iLoopX] = 0x00000000;
			}
		}
	}

	if(FAILED(m_vAlphaMapTable[iIndex]->m_pTexture->UnlockRect(0)))
		return false;

	return true;
}

//------------------------------------------------------------------------
// Լ  : bool SetCurrentDrawAlphaMapIndex(int iDrawAlphaMapIndex)
//     :  ĸ ϱ
// 	   : iDrawAlphaMapIndex ĸ ε
//------------------------------------------------------------------------
bool GtTerrain::SetCurrentDrawAlphaMapIndex(int iDrawAlphaMapIndex)
{
	if(m_vAlphaMapTable.size() <= iDrawAlphaMapIndex)
		return false;
	m_iCurrentDrawAlphaMapIdx = iDrawAlphaMapIndex;
	return true;
}

//------------------------------------------------------------------------
// Լ  : int GetCurrentDrawAlphaMapIndex(void)
//     :  ϰ ִ ĸ ε 
// 	   : iDrawAlphaMapIndex ĸ ε
int GtTerrain::GetCurrentDrawAlphaMapIndex(void)
{
	return m_iCurrentDrawAlphaMapIdx;
}

//------------------------------------------------------------------------
// Լ  : bool AddSplatingTextureInfo(int iSplatingTextureInfoIndex)
//     : ÷ ؽ  
// 	   : iSplatingTextureInfoIndex ÷ ؽ ε
//------------------------------------------------------------------------
bool GtTerrain::AddSplatingTextureInfo(int iSplatingTextureInfoIndex)
{
	if(iSplatingTextureInfoIndex >= m_vTerrainTextureTable.size())
		return false;
	sGT_TERRAINTEXTUREINFO sSplatingTextureInfo;
	sSplatingTextureInfo.m_iTextureIndex = iSplatingTextureInfoIndex;
	sSplatingTextureInfo.m_fScale = 1.0f;
	m_vSplatingTextureInfo.push_back(sSplatingTextureInfo);
	AddAlphaMap();
	return true;
}

//------------------------------------------------------------------------
// Լ  : bool RemoveSplatingTextureInfo(int iSplatingTextureInfoIndex)
//     : ÷ ؽ  
// 	   : iSplatingTextureInfoIndex ÷ ؽ ε
//------------------------------------------------------------------------
bool GtTerrain::RemoveSplatingTextureInfo(int iSplatingTextureInfoIndex)
{
	if(iSplatingTextureInfoIndex >= m_vTerrainTextureTable.size())
		return false;
	int n = 0;
	std::vector<sGT_TERRAINTEXTUREINFO> ::iterator it;
	for(it = m_vSplatingTextureInfo.begin(); it != m_vSplatingTextureInfo.end();)
	{
		if(n == iSplatingTextureInfoIndex)
		{
			it = m_vSplatingTextureInfo.erase(it);
			RemoveAlphaMap(iSplatingTextureInfoIndex);
			m_iCurrentDrawAlphaMapIdx = -1;
			return true;
		}
		n++;
		it++;
	}
	return false;
}

//------------------------------------------------------------------------
// Լ  : bool SetCurrentSplatingTextureScale(float fScale)
//     : õǾִ ÷ ؽ ũ 
// 	   : fScale : ϰ ϴ 
//------------------------------------------------------------------------
bool GtTerrain::SetCurrentSplatingTextureScale(float fScale)
{
	if(GetCurrentDrawAlphaMapIndex() < 0 || GetCurrentDrawAlphaMapIndex() >= m_vAlphaMapTable.size())
		return false;
	m_vSplatingTextureInfo[GetCurrentDrawAlphaMapIndex()].m_fScale = fScale;

	return true;
}

//------------------------------------------------------------------------
// Լ  : float GetCurrentSplatingTextureScale(void)
//     :  ÷ ؽ    
//------------------------------------------------------------------------
float GtTerrain::GetCurrentSplatingTextureScale(void)
{
	return m_vSplatingTextureInfo[GetCurrentDrawAlphaMapIndex()].m_fScale;
}

//------------------------------------------------------------------------
// Լ  : bool SwapLayer(int iLayer1, int iLayer2)
//     : ÷ ؽ ̾ ٲٱ
//------------------------------------------------------------------------
bool GtTerrain::SwapLayer(int iLayer1, int iLayer2)
{
	if(iLayer2 >= m_vSplatingTextureInfo.size() || iLayer2 < 0)
		return false;
	sGT_TERRAINTEXTUREINFO sTemp = m_vSplatingTextureInfo[iLayer1];
	m_vSplatingTextureInfo[iLayer1] = m_vSplatingTextureInfo[iLayer2];
	m_vSplatingTextureInfo[iLayer2] = sTemp;

	LPGT_TERRAINTEXTURE pTemp = m_vAlphaMapTable[iLayer1];
	m_vAlphaMapTable[iLayer1] = m_vAlphaMapTable[iLayer2];
	m_vAlphaMapTable[iLayer2] = pTemp;
	return true;
}

//------------------------------------------------------------------------
// Լ  : bool FrustumCull(LPGT_MAPINDEX pIBIdx)
//     : ü ø
// 	   : pIndexBuffer : ε 
//     : ڷ  pIndexBuffer ü ø
//------------------------------------------------------------------------
bool GtTerrain::FrustumCull(LPGT_MAPINDEX pIndexBuffer)
{
	DWORD			iTempIndex[4];
	int				bCullCheck[4];
	sGT_MAPINDEX	sIdx;
	
	for(int iLoopZ = 0; iLoopZ < m_sMapHeader.m_iNumCellsPerCol; ++iLoopZ)
	{
		for(int iLoopX = 0; iLoopX < m_sMapHeader.m_iNumCellsPerRow; ++iLoopX)
		{
			iTempIndex[0] = ( iLoopZ * m_sMapHeader.m_iNumVertsPerRow + iLoopX);			//   ε
			iTempIndex[1] = ( iLoopZ * m_sMapHeader.m_iNumVertsPerRow + iLoopX + 1);		//   ε
			iTempIndex[2] = ((iLoopZ + 1) * m_sMapHeader.m_iNumVertsPerRow + iLoopX );		//  ϴ ε
			iTempIndex[3] = ((iLoopZ + 1) * m_sMapHeader.m_iNumVertsPerRow + iLoopX + 1 );	//  ϴ ε

			GtPoint3f vPos[4];
			ConvertGtPoint3fFromD3DVector( &vPos[0], &m_pVMap[ iTempIndex[0] ].m_vPos);
			ConvertGtPoint3fFromD3DVector( &vPos[1], &m_pVMap[ iTempIndex[1] ].m_vPos);
			ConvertGtPoint3fFromD3DVector( &vPos[2], &m_pVMap[ iTempIndex[2] ].m_vPos);
			ConvertGtPoint3fFromD3DVector( &vPos[3], &m_pVMap[ iTempIndex[3] ].m_vPos);

			bCullCheck[0] = m_Frustum.IsInPoint( &vPos[0] );								// ˻
			bCullCheck[1] = m_Frustum.IsInPoint( &vPos[1] );								// ˻
			bCullCheck[2] = m_Frustum.IsInPoint( &vPos[2] );								//ϴ ˻
			if( bCullCheck[0] || bCullCheck[1] || bCullCheck[2] )							//øüũ  ε 
			{
				sIdx._0 = iTempIndex[0];
				sIdx._1 = iTempIndex[1];
				sIdx._2 = iTempIndex[2];
				*pIndexBuffer++ = sIdx;
				m_iTriangles++;	
			}

			bCullCheck[2] = m_Frustum.IsInPoint( &vPos[2] );								//ϴ ˻
			bCullCheck[1] = m_Frustum.IsInPoint( &vPos[1] );								// ˻
			bCullCheck[3] = m_Frustum.IsInPoint( &vPos[3] );								//ϴ ˻
			if( bCullCheck[2] || bCullCheck[1] || bCullCheck[3] )							//øüũ  ε 
			{
				sIdx._0 = iTempIndex[2];
				sIdx._1 = iTempIndex[1];
				sIdx._2 = iTempIndex[3];
				*pIndexBuffer++ = sIdx;
				m_iTriangles++;
			}
		}
	}
	return true;
}

//------------------------------------------------------------------------
// Լ  : bool SetCullingIndexBuffer(GtMatrix4 matView, GtMatrix4 matProj)
//     : ε  ø  Ʈ , ׸ ŷ  ó
// 	   : matView :  
//		   : matProj :  
//     : ڷ  2ķ ü ѵڿ
//			  Ʈ  ϴٸ Ʈ   
//			 ŷ ó  ̸ Ʈ
//------------------------------------------------------------------------
bool GtTerrain::SetCullingIndexBuffer(GtMatrix4 matView, GtMatrix4 matProj)
{
	// ü ڷ  Viewİ Projķ ü Ѵ
	m_Frustum.MakeFrustum(matView, matProj);

	// ε ۸ ϴ κ
	m_iTriangles = 0;
	LPGT_MAPINDEX pIndexBuffer;
	if(FAILED(m_pIBMap->Lock(0, m_sMapHeader.m_iNumTriangles * sizeof(sGT_MAPINDEX), (void**)&pIndexBuffer, 0)))
		return false;

	//  Ʈ    Ʈ ߻ ε ۸ Ѵ.
	//  
	// 1. IndexBuffer  ε 
	// 2. IndexBuffer ÿ ʿ ؽ ̺
	// 3. ü  Ʈ  ϱ   ü
	// 4.  Ʈ  LOD 
	if(m_bMapSize)
		m_iTriangles = m_pQuadTree->GenerateIndex(pIndexBuffer, m_pVMap, &m_Frustum, m_sParameter.m_fLODLevel * 0.001);
	else
		if(!FrustumCull(pIndexBuffer))
			return false;

	// ε ۸  ѵڿ ŷ ǽ Ѵ.
	// m_pPicking  NULL ƴϸ ŷ  
	//  
	// 1. m_Picingü
	// 2. ŷϴµ ʿ  IndexBuffer
	if(m_pPicking)
		SearchPicking(m_pPicking, pIndexBuffer);

	// Ʈ  ؽ Ʈ ϴ Լ
	if(m_sParameter.m_bIsLClick == true)
	{
		switch(m_sParameter.m_iTabPage)
		{
		case 0:
			ControlHeightMapUpAndDown();
			break;
		case 1:
			DrawAlphaMap(m_pPicking, m_iCurrentDrawAlphaMapIdx);
			break;
		}
	}

	// ControlHeightMapUpAndDown   ؽ ۸ m_pVBMap 
	SetVBMap(pIndexBuffer);

	m_pIBMap->Unlock();

	return true;
}

//------------------------------------------------------------------------
// Լ  : bool CheckMapSize(int iSize)
//     :  Ʈ  Ұ üũ
// 	   : iSize : 
//------------------------------------------------------------------------
bool GtTerrain::CheckMapSize(int iSize)
{
	int iMod = iSize--;
	while(iSize > 1)
	{
		iMod = iSize % 2;
		iSize /= 2;
	}
	return iMod ? false : true;
}

//------------------------------------------------------------------------
// Լ  : bool SearchPicking(GtPicking *pPicking, sGT_MAPINDEX *pIdx)
//     : ŷ  ã
// 	   : pPicking : Picking ü
//		   : pIndexBuffer : ε 
//     : ڷ  ŷ ü IndexBuffer ŷ 
//			 ﰢ 浹 Ǵ üũ
//------------------------------------------------------------------------
bool GtTerrain::SearchPicking(GtPicking *pPicking, sGT_MAPINDEX *pIndexBuffer)
{
	bool bResult = false;

	// ŷ ˻( 浹 ˻  ﰢ T(Ÿ)   ﰢ )
	D3DXVECTOR3 vPickPos;
	float fMin = float(m_sMapHeader.m_iDepth);
	for(int iLoop = 0; iLoop<m_iTriangles; ++iLoop)
	{
		//   ŷƮ ϱ( T(Ÿ)   )
		if(pPicking->CheckTriangleCollision(m_pVMap[ pIndexBuffer[iLoop]._0 ].m_vPos, m_pVMap[ pIndexBuffer[iLoop]._1 ].m_vPos, m_pVMap[ pIndexBuffer[iLoop]._2 ].m_vPos))
		{
			if(pPicking->m_sHL.m_fT < fMin)
			{
				bResult = true;
				fMin = pPicking->m_sHL.m_fT;
				vPickPos = pPicking->m_vPickPos;
			}
		}
	}

	pPicking->m_vPickPos = vPickPos;
	pPicking->m_vPickPos.y = GetHeightMapY(pPicking->m_vPickPos.x, pPicking->m_vPickPos.y);
	return bResult;
}

//------------------------------------------------------------------------
// Լ  : bool SetPickCircle(bool bPickingUse)
//     : ŷ ؽ    
// 	   : bPickingUse :    
//     : ڷ  bool true 
//			 ŷ ؽ  ( ū  2)
//			 false   ִ ؽ    
//			 Ǿմ ŷ NULL
//------------------------------------------------------------------------
bool GtTerrain::SetPickCircle(bool bPickingUse)
{
	if(bPickingUse)
	{
		//ŷ  
		if( !m_pVBPicking )
		{
			// 720     360 * 2 ̱⶧   ū ؽ ̴.
			if( FAILED(m_pDev->CreateVertexBuffer(720 * sizeof(GtPoint3f), 0, D3DFVF_CUSTOMCIRCLE, D3DPOOL_DEFAULT, &m_pVBPicking, NULL)) )
				return false;
		}
	}
	else
	{
		if(m_pVBPicking)
			m_pVBPicking->Release();
		m_pVBPicking = NULL;
		m_pPicking = NULL;
	}
	return true;
}

//------------------------------------------------------------------------
// Լ  : bool SetPickingCircleVertexBuffer(GtPicking *pPicking)
//     : ŷ  ؽ   
// 	   : pPicking : ŷ ü
//     : ڷ  Picking ġ  2 ̵ .
//			 1 0~360 2 360~720  ϳ  2  ġ 
//------------------------------------------------------------------------
bool GtTerrain::SetPickingCircleVertexBuffer(GtPicking *pPicking)
{
	LPGT_PICKINGCIRCLEVERTEX	pVertex;
	if(FAILED(m_pVBPicking->Lock(0, 720 * sizeof(sGT_PICKINGCIRCLEVERTEX), (void**)&pVertex, 0)))
		return false;

	//ۼ
	for(int iLoop=0; iLoop<360; ++iLoop)
	{
		pVertex[iLoop].m_vPos.x = m_sParameter.m_iBigCircle * sin(iLoop * 3.14f / 180.0f) + pPicking->m_vPickPos.x;
		pVertex[iLoop].m_vPos.z = m_sParameter.m_iBigCircle * cos(iLoop * 3.14f / 180.0f) + pPicking->m_vPickPos.z;
		pVertex[iLoop].m_vPos.y = GetHeightMapY(pVertex[iLoop].m_vPos.x, pVertex[iLoop].m_vPos.z) + 1.0f;
	}
	for(int iLoop=360; iLoop<720; ++iLoop)
	{
		pVertex[iLoop].m_vPos.x = m_sParameter.m_iSmallCircle * sin((iLoop-360) * 3.14f / 180.0f) + pPicking->m_vPickPos.x;
		pVertex[iLoop].m_vPos.z = m_sParameter.m_iSmallCircle * cos((iLoop-360) * 3.14f / 180.0f) + pPicking->m_vPickPos.z;
		pVertex[iLoop].m_vPos.y = GetHeightMapY(pVertex[iLoop].m_vPos.x, pVertex[iLoop].m_vPos.z) + 1.0f;
	}

	m_pVBPicking->Unlock();
	return true;
}

//------------------------------------------------------------------------
// Լ  : void SetPicking(GtPicking* pPicking)
//     :   ŷ ü 
// 	   : pPicking ŷ ü
//------------------------------------------------------------------------
void GtTerrain::SetPicking(GtPicking* pPicking)
{
	m_pPicking = pPicking;
}

//------------------------------------------------------------------------
// Լ  : D3DXVECTOR3 GetHeightMap(int iRow, int iCol)
//     :   ̸  ȹ
// 	   : iRow :  , iCol : 
//------------------------------------------------------------------------
D3DXVECTOR3 GtTerrain::GetHeightMap(int iRow, int iCol)
{
	if(0 > iRow || iRow > m_sMapHeader.m_iNumCellsPerRow) return D3DXVECTOR3(0.0f,0.0f,0.0f);
	if(0 > iCol || iCol > m_sMapHeader.m_iNumCellsPerRow) return D3DXVECTOR3(0.0f,0.0f,0.0f);
	return m_pVMap[ iCol * m_sMapHeader.m_iNumVertsPerRow + iRow ].m_vPos;
}

sGT_Parameter* GtTerrain::GetParameter(void)
{
	return &m_sParameter;
}

//------------------------------------------------------------------------
// Լ  : void GetHeightMapIndex(float fX, float fZ, int *iRow, int *iCol)
//     : x, z ̸   
// 	   : fX : x, fZ : zr, *iRow : , *iCol : 
//------------------------------------------------------------------------
void GtTerrain::GetHeightMapIndex(float fX, float fZ, int* iRow, int* iCol)
{
	//fX, fZ    ̵
	fX = m_sMapHeader.m_iWidth / 2.0f + fX;
	fZ = m_sMapHeader.m_iDepth / 2.0f - fZ;

	//    1 ȯ(ε ġ ã Ϸ)
	fX /= m_sMapHeader.m_vCellSpacing.x;
	fZ /= m_sMapHeader.m_vCellSpacing.z;
	*iRow = int( floorf(fX) );
	*iCol = int( floorf(fZ) );

	//ġ׽Ʈ(ﰢ Ʒﰢ)
	float fdx = fX - *iRow;
	float fdz = fZ - *iCol;

	//Ʒﰢ ε 
	if(fdz >= 1.0f - fdx)
	{
		++iRow;
		++iCol;
	}
}

//------------------------------------------------------------------------
// Լ  : void SetHeightMapY(int iRow, int iCol, float fY)
//     : ڷ   κ y 
// 	   : iRow : , iCol : , fY : ̸ʿ  y
//------------------------------------------------------------------------
void GtTerrain::SetHeightMapY(int iRow, int iCol, float fY)
{
	if(0 > iRow || iRow > m_sMapHeader.m_iNumCellsPerRow)
		return;
	if(0 > iCol || iCol > m_sMapHeader.m_iNumCellsPerCol)
		return;

	if(0.0f == fY)
		m_pVMap[ (iCol * m_sMapHeader.m_iNumVertsPerRow) + iRow ].m_vPos.y = 0.0f;
	else
		m_pVMap[ (iCol * m_sMapHeader.m_iNumVertsPerRow) + iRow ].m_vPos.y += fY;
}

//------------------------------------------------------------------------
// Լ  : float GetHeightMapY(float fX, float fZ)
//     : x, z   y 
// 	   : fX : x, fY : y
//------------------------------------------------------------------------
float GtTerrain::GetHeightMapY(float fX, float fZ)
{
	// fX, fZ     ̵
	fX = m_sMapHeader.m_iWidth / 2.0f + fX;
	fZ = m_sMapHeader.m_iDepth / 2.0f - fZ;

	//      1 ȯ(ε ġ ã Ϸ)
	fX /= m_sMapHeader.m_vCellSpacing.x;
	fZ /= m_sMapHeader.m_vCellSpacing.z;
	int iRow = int( floorf(fX) );
	int iCol = int( floorf(fZ) );

	// ġ ׽Ʈ
	D3DXVECTOR3 v;
	float fA, fB, fC, fD;

	//  
	v = GetHeightMap(iRow	, iCol	);		fA = v.y;
	v = GetHeightMap(iRow+1 , iCol	);		fB = v.y;
	v = GetHeightMap(iRow	, iCol+1);		fC = v.y;
	v = GetHeightMap(iRow+1 , iCol+1);		fD = v.y;

	//  
	float fdx = fX - iRow;
	float fdz = fZ - iCol;

	//̰ 
	float fHeight, fUy, fVy;
	if(fdz < 1.0f - fdx)
	{
		fUy = fB - fA;		fVy = fC - fA;
		fHeight = fA + Lerp(0.0f, fUy, fdx) + Lerp(0.0f, fVy, fdz);
	}
	else
	{
		fUy = fC - fD;		fVy = fB - fD;
		fHeight = fD + Lerp(0.0f, fUy, 1.0f - fdx) + Lerp(0.0f, fVy, 1.0f - fdz);
	}

	return fHeight;
}

//------------------------------------------------------------------------
// Լ  : float Lerp(float fA, float fB, float fT)
//     : a~b  
// 	   : fA : ù , fB : ι° , fT : ġ
//------------------------------------------------------------------------
float GtTerrain::Lerp(float fA, float fB, float fT)
{
	return fA - (fA * fT) + (fB * fT);
}

void GtTerrain::Update(void)
{
	// Device ø   İ   ´.
	GtMatrix4 matView;
	matView = GtD3DRenderer::GetInstance()->m_matTM[eRPLM_VIEW];
	matView.m43 += CULLING_SUB_VALUE;																	//  ø  ణ  Ͽ ī޶ ġ ̵
	SetCullingIndexBuffer(matView, GtD3DRenderer::GetInstance()->m_matTM[eRPLM_PROJ]);
}

void GtTerrain::Render(void)
{
	LPDIRECT3DDEVICE9 m_pDev = GtD3DRenderer::GetInstance()->GetDevice();
	if(m_pDev)
	{
		m_pDev->SetRenderState(D3DRS_FILLMODE, m_eFillMode);

	    m_pDev->SetStreamSource(0, m_pVBMap, 0, sizeof(sGT_CUSTOMVERTEX));

		m_pDev->SetFVF(D3DFVF_TERRAIN);

		m_pDev->SetIndices(m_pIBMap);

		if(m_sTerrainTextureInfo.m_iTextureIndex != -1 && m_sTerrainTextureInfo.m_iTextureIndex < m_vTerrainTextureTable.size())
		{
			if(m_vTerrainTextureTable[m_sTerrainTextureInfo.m_iTextureIndex] && m_vTerrainTextureTable.size() > m_sTerrainTextureInfo.m_iTextureIndex)
				m_pDev->SetTexture(0, m_vTerrainTextureTable[m_sTerrainTextureInfo.m_iTextureIndex]->m_pTexture);
			else
				m_pDev->SetTexture(0, NULL);

			m_pDev->SetTextureStageState(0, D3DTSS_COLOROP, D3DTOP_MODULATE);
			m_pDev->SetTextureStageState(0, D3DTSS_COLORARG1, D3DTA_TEXTURE);

			m_pDev->SetSamplerState(0, D3DSAMP_MAGFILTER, D3DTEXF_LINEAR);
			m_pDev->SetSamplerState(0, D3DSAMP_MINFILTER, D3DTEXF_LINEAR);
			m_pDev->SetSamplerState(0, D3DSAMP_MIPFILTER, D3DTEXF_LINEAR);

			m_pDev->SetTextureStageState(0, D3DTSS_TEXTURETRANSFORMFLAGS, D3DTTFF_COUNT2 );
			m_pDev->SetTextureStageState(0, D3DTSS_TEXCOORDINDEX, 0);

			m_pDev->SetSamplerState(0, D3DSAMP_ADDRESSU, D3DTADDRESS_WRAP);
			m_pDev->SetSamplerState(0, D3DSAMP_ADDRESSV, D3DTADDRESS_WRAP);

			if(m_eFillMode == D3DFILL_SOLID)
			{
				D3DXMATRIXA16 matScale;
				D3DXMatrixScaling(&matScale, m_sTerrainTextureInfo.m_fScale, m_sTerrainTextureInfo.m_fScale, 1.0f);
				m_pDev->SetTransform( D3DTS_TEXTURE0, &matScale );
			}
		}
		m_pDev->DrawIndexedPrimitive(D3DPT_TRIANGLELIST, 0, 0, m_sMapHeader.m_iNumVertices, 0, m_iTriangles);

		if(m_eFillMode == D3DFILL_SOLID)
		{
			m_pDev->SetTextureStageState( 0, D3DTSS_TEXTURETRANSFORMFLAGS, D3DTTFF_DISABLE );

			//ĺ 
			m_pDev->SetRenderState( D3DRS_ALPHABLENDENABLE, TRUE );

			//ĸ
			m_pDev->SetTextureStageState(0, D3DTSS_ALPHAOP, D3DTOP_MODULATE);
			m_pDev->SetTextureStageState(0, D3DTSS_ALPHAARG1, D3DTA_TEXTURE);

			//ؽ
			m_pDev->SetTextureStageState(1, D3DTSS_COLOROP, D3DTOP_MODULATE);
			m_pDev->SetTextureStageState(1, D3DTSS_COLORARG1, D3DTA_TEXTURE);
			m_pDev->SetTextureStageState(1, D3DTSS_ALPHAOP, D3DTOP_MODULATE);
			m_pDev->SetTextureStageState(1, D3DTSS_ALPHAARG1, D3DTA_CURRENT);

			m_pDev->SetSamplerState(1, D3DSAMP_MAGFILTER, D3DTEXF_LINEAR);
			m_pDev->SetSamplerState(1, D3DSAMP_MINFILTER, D3DTEXF_LINEAR);
			m_pDev->SetSamplerState(1, D3DSAMP_MIPFILTER, D3DTEXF_LINEAR);

			m_pDev->SetTextureStageState(0, D3DTSS_TEXTURETRANSFORMFLAGS, D3DTTFF_DISABLE);
			m_pDev->SetTextureStageState(1, D3DTSS_TEXTURETRANSFORMFLAGS, D3DTTFF_COUNT2);
			m_pDev->SetTextureStageState(1, D3DTSS_TEXCOORDINDEX, 0);

			m_pDev->SetSamplerState(0, D3DSAMP_ADDRESSU, D3DTADDRESS_CLAMP);
			m_pDev->SetSamplerState(0, D3DSAMP_ADDRESSV, D3DTADDRESS_CLAMP);
			m_pDev->SetSamplerState(1, D3DSAMP_ADDRESSU, D3DTADDRESS_WRAP);
			m_pDev->SetSamplerState(1, D3DSAMP_ADDRESSV, D3DTADDRESS_WRAP);

			//  ؽ 
			for(int i=0; i<m_vSplatingTextureInfo.size(); i++)
			{
				D3DXMATRIXA16 matAlphaMapScale;
				D3DXMatrixScaling(&matAlphaMapScale, m_vSplatingTextureInfo[i].m_fScale, m_vSplatingTextureInfo[i].m_fScale, 1.0f);
				m_pDev->SetTransform( D3DTS_TEXTURE0, &matAlphaMapScale);
				m_pDev->SetTransform( D3DTS_TEXTURE1, &matAlphaMapScale);
				m_pDev->SetTexture(0, m_vAlphaMapTable[i]->m_pTexture);
				m_pDev->SetTexture(1, m_vTerrainTextureTable[m_vSplatingTextureInfo[i].m_iTextureIndex]->m_pTexture);
				m_pDev->DrawIndexedPrimitive(D3DPT_TRIANGLELIST, 0, 0, m_sMapHeader.m_iNumVertices, 0, m_iTriangles);
			}

			//1 ؽ ؽǥ Ͷ ޵ Ѵ.
			m_pDev->SetTextureStageState( 1, D3DTSS_TEXTURETRANSFORMFLAGS, D3DTTFF_DISABLE );

			// Ƽؽ 0, 1 ؽĿ  Ѵ.
			m_pDev->SetTextureStageState( 0, D3DTSS_COLOROP, D3DTOP_DISABLE );
			m_pDev->SetTextureStageState( 0, D3DTSS_ALPHAOP, D3DTOP_DISABLE );
			m_pDev->SetTextureStageState( 1, D3DTSS_COLOROP, D3DTOP_DISABLE );
			m_pDev->SetTextureStageState( 1, D3DTSS_ALPHAOP, D3DTOP_DISABLE );

			//ĺ   Ѵ.
			m_pDev->SetRenderState( D3DRS_ALPHABLENDENABLE, FALSE );

		}
	}
	if(m_pPicking)
	{
		if(!SetPickingCircleVertexBuffer(m_pPicking))
			return ;
		m_pDev->SetStreamSource(0, m_pVBPicking, 0, sizeof(sGT_PICKINGCIRCLEVERTEX));
		m_pDev->SetFVF(D3DFVF_PICKINGCIRCLE);

		m_pDev->SetTexture(0, NULL);
		// λ
		D3DMATERIAL9	mtrl;
		ZeroMemory(&mtrl, sizeof(D3DMATERIAL9));
		mtrl.Emissive.r	= 1.0f;	mtrl.Emissive.g	= 0.0f;	mtrl.Emissive.b	= 1.0f;	mtrl.Emissive.a = 1.0f;		// RGBA(255, 0, 255, 255)
		m_pDev->SetMaterial(&mtrl);

		// ū׸
		m_pDev->DrawPrimitive(D3DPT_LINELIST, 0, 180);
		m_pDev->DrawPrimitive(D3DPT_LINELIST, 1, 179);

		// λ
		mtrl.Emissive.r	= 1.0f;	mtrl.Emissive.g	= 0.2f;	mtrl.Emissive.b	= 0.2f;	mtrl.Emissive.a = 1.0f;		// RGBA(255, 51, 51, 255)
		m_pDev->SetMaterial(&mtrl);

		// ׸
		m_pDev->DrawPrimitive(D3DPT_LINELIST, 360, 180);
		m_pDev->DrawPrimitive(D3DPT_LINELIST, 361, 179);

		//  ŷ  Ͽ   ٽ   
		mtrl.Specular.r = mtrl.Diffuse.r = mtrl.Ambient.r = 1.0f;
		mtrl.Specular.b = mtrl.Diffuse.g = mtrl.Ambient.g = 1.0f;
		mtrl.Specular.g = mtrl.Diffuse.b = mtrl.Ambient.b = 1.0f;
		mtrl.Specular.a = mtrl.Diffuse.a = mtrl.Ambient.a = 1.0f;
		mtrl.Power = 8.0f;
		m_pDev->SetMaterial(&mtrl);
	}
}