#pragma once

#include "GtNode.h"
#include "GtFrustum.h"
#include "GtDib.h"
#include "GtPicking.h"
#include <windows.h>
#include <d3d9.h>
#include <d3dx9.h>

#define D3DFVF_CUSTOMCIRCLE D3DFVF_XYZ

typedef struct sGtMapIndex
{
	DWORD _0, _1, _2;
}sGT_MAPINDEX, *LPGT_MAPINDEX;

typedef struct sGtCustomPickingCircleVertex
{
	D3DXVECTOR3		m_vPos;
}sGT_PICKINGCIRCLEVERTEX, *LPGT_PICKINGCIRCLEVERTEX;
#define D3DFVF_PICKINGCIRCLE	D3DFVF_XYZ

typedef struct sGtCustomVertex
{
	D3DXVECTOR3		m_vPos;
	D3DXVECTOR3		m_vNormal;
	float			u, v;
}sGT_CUSTOMVERTEX, *LPGT_CUSTOMVERTEX;

#define D3DFVF_TERRAIN		(D3DFVF_XYZ | D3DFVF_NORMAL | D3DFVF_TEX1 | D3DFVF_TEXCOORDSIZE2(0) )

typedef struct sGtHeightMap
{
	int			m_iNumVertsPerRow;	// ؽ 
	int			m_iNumVertsPerCol;	// ؽ 
	int			m_iNumCellsPerRow;	//  
	int			m_iNumCellsPerCol;	//  
	int			m_iWidth;			// 
	int			m_iDepth;			// 
	int			m_iNumVertices;		// ؽ 
	int			m_iNumTriangles;	// ﰢ 
	float		m_fHeightScale;		// ̹
	D3DXVECTOR3	m_vCellSpacing;		// ϰ

	void SetHeightMap(int iNumVertsPerRow, int iNumVertsPerCol, GtPoint3f vCellSpacing, float fHeightScale);
}sGT_HeightMap, *LPGT_HEIGHTMAP;

enum eTerrainState
{
	eTS_NONE,
	eTS_UP,
	eTS_DOWN,
	eTS_FLATLAND,

	eTS_END
};

typedef struct sGtParameter
{
	bool			m_bIsLClick;			// 콺 Ŭ 
	int				m_iTabPage;				//   ѹ
	eTerrainState	m_eTerrainControlState;	//   
	int				m_iSplateControl;		// ׸, 
	int				m_iBigCircle;			// ū  ũ
	int				m_iSmallCircle;			//   ũ
	float			m_fSmoothSize;			// ÷  ũ
	float			m_fPower;				//  Ŀ
	float			m_fLODLevel;			// LOD 

	sGtParameter(void)
	{
		m_bIsLClick				= false;
		m_iTabPage				= 0;
		m_eTerrainControlState	= eTS_NONE;
		m_iSplateControl		= 1;
		m_iBigCircle			= 100;
		m_iSmallCircle			= 50;
		m_fSmoothSize			= 0.0f;
		m_fPower				= 0.1f;
		m_fLODLevel				= 0.001f;
	}
	void			SetTerrainControlState(int iState)
	{
		m_eTerrainControlState = (eTerrainState)iState;
	}
}sGT_Parameter, *LPGT_PARAMETER;

typedef struct sGtTerrainTexture
{
	LPDIRECT3DTEXTURE9			m_pTexture;
	sGtTerrainTexture(void) : m_pTexture(NULL)
	{
	}
	~sGtTerrainTexture(void)
	{
		if(m_pTexture)
		{
			m_pTexture->Release();
			m_pTexture = NULL;
		}
	}
}sGT_TERRAINTEXTURE, *LPGT_TERRAINTEXTURE;

typedef struct sGtTerrainTextureInfo
{
	float					m_fScale;
	int						m_iTextureIndex;
	sGtTerrainTextureInfo(void) : m_fScale(1.0f), m_iTextureIndex(0)
	{
	}
	~sGtTerrainTextureInfo(void)
	{
	}
}sGT_TERRAINTEXTUREINFO;

#define CULLING_SUB_VALUE			200.0f

//////////////////////////////////////////////////////////////////////////
/** 
@class	  GtTerrain
@brief	    Ŭ̴. Picking    ˼ ִ.
@date     2011-01-08
@author   ghostkyow@naver.com 
**/
//////////////////////////////////////////////////////////////////////////
class GtQuadTree;
class GtTerrain : public GtNode
{
private:
	D3DFILLMODE					m_eFillMode;					//  FillMode
private:			// DX 
	LPDIRECT3DVERTEXBUFFER9				m_pVBPicking;			// ŷ 
	LPDIRECT3DVERTEXBUFFER9				m_pVBMap;				//  
	LPDIRECT3DINDEXBUFFER9				m_pIBMap;				//  ε 
	LPDIRECT3DDEVICE9					m_pDev;					//  ϸ ȵ˴ϴ.

private:			// 
	LPGT_CUSTOMVERTEX					m_pVMap;				// ̸ ؽ  ̺

private:			// ؽ
	LPGT_TERRAINTEXTURE					m_pBaseTexture;
	LPGT_TERRAINTEXTURE					m_pAlphaMap;

	int									m_iAlphaMapX;
	int									m_iAlphaMapY;
	int									m_iCurrentDrawAlphaMapIdx;	//  ׸ ĸ ε
	std::vector<LPGT_TERRAINTEXTURE>	m_vTerrainTextureTable;		// ؽĵ ̺
	std::vector<LPGT_TERRAINTEXTURE>	m_vAlphaMapTable;			//    ؽ ̺
	sGT_TERRAINTEXTUREINFO				m_sTerrainTextureInfo;		//  ؽ (ũ, ε)
	std::vector<sGT_TERRAINTEXTUREINFO>	m_vSplatingTextureInfo;		// ÷ ؽ 
private:			// ø 
	int									m_iTriangles;				// ﰢ 
	int									m_bMapSize;					// Ʈ ø  üũ
	GtFrustum							m_Frustum;					// ü
	GtQuadTree*							m_pQuadTree;				//  Ʈ

private:			// ̸ 
	sGT_HeightMap						m_sMapHeader;				//   
	sGT_Parameter						m_sParameter;				// Ķ ( Ʈ   )

private:
	GtPicking*							m_pPicking;					//  ġ ϱ 

public:
	GtTerrain(void);
	GtTerrain(const GtString& strName);
	~GtTerrain(void);

	// ̽ 
	void								SetDevice(LPDIRECT3DDEVICE9 pDev);								// DX Device 

	//   , ε
	bool								CreateHeightMap(int iWidth, int iHeight, int iCellSpacing);		// ̸   (y  0)
	bool								LoadHeightMap(char* pFileName);									// Ǿִ ̸ ҷ
	
	//   
	bool								CreateVBMap(void);												//   
	bool								SetVBMap(sGT_MAPINDEX* pIndexBuffer);							// Ʈ  
	void								ControlHeightMapUpAndDown(void);								//   ٿ Ʈ
	void								SetFillMode(D3DFILLMODE eFillMode);
	LPGT_CUSTOMVERTEX					GetVertexMap(void);												// ؽ  
	sGT_HeightMap*						GetHeigtMapHeader(void);										// ̸  

	// ؽ 
	bool								LoadTexture(wchar_t* pFileName);								// ؽ ε
	bool								CreateAlphaMaps(void);											//  ĸ 
	bool								AddTexture(wchar_t* pFileName);									// ؽ ̺ ֱ Ͽ
	bool								RemoveTexture(int iTextureIndex);								// ؽ 
	bool								SetCurrentTerrainTexIndex(int iCurrentTerrainTexIndex);			//  ؽ ε 
	int									GetCurrentTerrainTexIndex(void);								//  ؽ ε ҷ
	bool								SetTerrainTextureScale(float fScale);		//  ؽ  
	float								GetTerrainTextureScale(void);					//  ؽ  
	
	bool								AddAlphaMap(void);												// ĸ 
	bool								RemoveAlphaMap(int iAlphaMapIndex);
	bool								DrawAlphaMap(GtPicking* pPicking, int iIndex);					// ĸ ׸ κ

	bool								AddSplatingTextureInfo(int iSplatingTextureInfoIndex);
	bool								RemoveSplatingTextureInfo(int iSplatingTextureInfoIndex);
	bool								SetCurrentDrawAlphaMapIndex(int iDrawAlphaMapIndex);			// ׸ ĸ ε 
	int									GetCurrentDrawAlphaMapIndex(void);								// ĸ ε 
	bool								SetCurrentSplatingTextureScale(float fScale);
	float								GetCurrentSplatingTextureScale(void);

	bool								SwapLayer(int iLayer1, int iLayer2);


	// ü ø  ε  
	bool								FrustumCull(LPGT_MAPINDEX pIBIdx);								// ü øؼ IdxBuffer 
	bool								SetCullingIndexBuffer(GtMatrix4 matView, GtMatrix4 matProj);	// ø ε  
	bool								CheckMapSize(int iSize);										// ø   üũ

	//  ŷ
	bool								SearchPicking(GtPicking* pPicking, sGT_MAPINDEX* pIdx);			// ŷ  ã
	bool								SetPickCircle(bool bPickingUse = false);						// ŷϴ ġ     
	bool								SetPickingCircleVertexBuffer(GtPicking* pPicking);				// ŷ   ū ؽ  
	void								SetPicking(GtPicking* pPicking);								// ŷ ü 

	//   Լ
	D3DXVECTOR3							GetHeightMap(int iRow, int iCol);								//   ̸  
	sGT_Parameter*						GetParameter(void);												// Ķ 
	void								GetHeightMapIndex(float fX, float fZ, int* iRow, int* iCol);	// x, z  ̸ ε 
	float								GetHeightMapY(float fX, float fZ);								// x, z  Y 
	void								SetHeightMapY(int iRow, int iCol, float fY);					//   ̸  y 
	float								Lerp(float fA, float fB, float fT);								// A  B T  

	void								Update(void);
	void								Render(void);
};